@@ -1,7 +1,7 @@
 // samplv1.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -57,16 +57,29 @@ public:
 
 	samplv1_sample *sample() const;
 
-	void setReverse(bool bReverse);
+	void setReverse(bool bReverse, bool bSync = false);
 	bool isReverse() const;
 
-	void setLoop(bool bLoop);
+	void setOffset(bool bOffset, bool bSync = false);
+	bool isOffset() const;
+
+	void setOffsetRange(uint32_t iOffsetStart, uint32_t iOffsetEnd);
+	uint32_t offsetStart() const;
+	uint32_t offsetEnd() const;
+
+	void setLoop(bool bLoop, bool bSync = false);
 	bool isLoop() const;
 
 	void setLoopRange(uint32_t iLoopStart, uint32_t iLoopEnd);
 	uint32_t loopStart() const;
 	uint32_t loopEnd() const;
 
+	void setLoopFade(uint32_t iLoopFade);
+	uint32_t loopFade() const;
+
+	void setLoopZero(bool bLoopZero);
+	bool isLoopZero() const;
+
 	void setBufferSize(uint32_t nsize);
 	uint32_t bufferSize() const;
 
@@ -77,11 +90,17 @@ public:
 
 		GEN1_SAMPLE = 0,
 		GEN1_REVERSE,
+		GEN1_OFFSET,
+		GEN1_OFFSET_1,
+		GEN1_OFFSET_2,
 		GEN1_LOOP,
+		GEN1_LOOP_1,
+		GEN1_LOOP_2,
 		GEN1_OCTAVE,
 		GEN1_TUNING,
 		GEN1_GLIDE,
 		GEN1_ENVTIME,
+		DCF1_ENABLED,
 		DCF1_CUTOFF,
 		DCF1_RESO,
 		DCF1_TYPE,
@@ -91,6 +110,7 @@ public:
 		DCF1_DECAY,
 		DCF1_SUSTAIN,
 		DCF1_RELEASE,
+		LFO1_ENABLED,
 		LFO1_SHAPE,
 		LFO1_WIDTH,
 		LFO1_BPM,
@@ -106,6 +126,7 @@ public:
 		LFO1_DECAY,
 		LFO1_SUSTAIN,
 		LFO1_RELEASE,
+		DCA1_ENABLED,
 		DCA1_VOLUME,
 		DCA1_ATTACK,
 		DCA1_DECAY,
@@ -149,6 +170,9 @@ public:
 		DYN1_COMPRESS,
 		DYN1_LIMITER,
 
+		KEY1_LOW,
+		KEY1_HIGH,
+
 		NUM_PARAMS
 	};
 
@@ -158,6 +182,9 @@ public:
 	void setParamValue(ParamIndex index, float fValue);
 	float paramValue(ParamIndex index) const;
 
+	bool running(bool on);
+
+	void stabilize();
 	void reset();
 
 	samplv1_controls *controls() const;
@@ -166,7 +193,7 @@ public:
 	void process_midi(uint8_t *data, uint32_t size);
 	void process(float **ins, float **outs, uint32_t nframes);
 
-	bool sampleLoopTest() const;
+	void sampleOffsetLoopTest();
 
 	virtual void updatePreset(bool bDirty) = 0;
 
@@ -175,7 +202,24 @@ public:
 
 	void directNoteOn(int note, int vel);
 
-protected:
+	void setTuningEnabled(bool enabled);
+	bool isTuningEnabled() const;
+
+	void setTuningRefPitch(float refPitch);
+	float tuningRefPitch() const;
+
+	void setTuningRefNote(int refNote);
+	int tuningRefNote() const;
+
+	void setTuningScaleFile(const char *pszScaleFile);
+	const char *tuningScaleFile() const;
+
+	void setTuningKeyMapFile(const char *pszKeyMapFile);
+	const char *tuningKeyMapFile() const;
+
+	void resetTuning();
+
+	virtual void updateTuning() = 0;
 
 	virtual void updateSample() = 0;
 
