@@ -1,7 +1,7 @@
 // samplv1_controls.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -72,6 +72,16 @@ public:
 				return (param < key.param);
 		}
 
+		// copy assignment operator.
+		Key& operator= (const Key& key)
+		{
+			if (this != &key) {
+				status = key.status;
+				param  = key.param;
+			}
+			return *this;
+		}
+
 		unsigned short status;
 		unsigned short param;
 	};
@@ -83,9 +93,6 @@ public:
 	struct Data
 	{
 		Data () : index(-1), flags(0), val(0.0f), sync(false) {}
-		Data (const Data& data)
-			: index(data.index), flags(data.flags),
-			  val(data.val), sync(data.sync) {}
 
 		int index;
 		int flags;
@@ -97,7 +104,6 @@ public:
 
 	// controller events.
 	struct Event
-
 	{
 		Key key;
 		unsigned short value;
