@@ -1,7 +1,7 @@
 // samplv1_jack.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -67,10 +67,14 @@ public:
 	void sessionEvent(void *pvSessionArg);
 #endif
 
+	void shutdown();
+	void shutdown_close();
+
 protected:
 
 	void updatePreset(bool bDirty);
 	void updateSample();
+	void updateTuning();
 
 private:
 
@@ -116,6 +120,9 @@ class samplv1widget_jack;
 class samplv1_nsm;
 #endif
 
+#ifdef HAVE_SIGNAL_H
+class QSocketNotifier;
+#endif
 
 class samplv1_jack_application : public QObject
 {
@@ -132,19 +139,33 @@ public:
 	// Facade method.
 	int exec();
 
-#ifdef CONFIG_NSM
+	// JACK shutdown handler.
+	void shutdown();
+
+	// Pseudo-singleton accessor.
+	static samplv1_jack_application *getInstance();
+
+signals:
+
+	void shutdown_signal();
 
 protected slots:
 
+#ifdef CONFIG_NSM
 	// NSM callback slots.
 	void openSession();
 	void saveSession();
-
 	void hideSession();
 	void showSession();
-
 #endif	// CONFIG_NSM
 
+#ifdef HAVE_SIGNAL_H
+	// SIGTERM signal handler.
+	void handle_sigterm();
+#endif
+
+	void shutdown_slot();
+
 protected:
 
 	// Argument parser method.
@@ -158,6 +179,7 @@ private:
 	// Instance variables.
 	QCoreApplication *m_pApp;
 	bool m_bGui;
+
 	QStringList m_presets;
 
 	samplv1_jack *m_pSampl;
@@ -166,6 +188,12 @@ private:
 #ifdef CONFIG_NSM
 	samplv1_nsm *m_pNsmClient;
 #endif
+
+#ifdef HAVE_SIGNAL_H
+	QSocketNotifier *m_pSigtermNotifier;
+#endif
+
+	static samplv1_jack_application *g_pInstance;
 };
 
 
