@@ -1,7 +1,7 @@
 // samplv1_lv2.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -20,7 +20,7 @@
 *****************************************************************************/
 
 #include "samplv1_lv2.h"
-
+#include "samplv1_config.h"
 #include "samplv1_sched.h"
 #include "samplv1_sample.h"
 
@@ -54,10 +54,13 @@
 #define LV2_STATE__StateChanged LV2_STATE_PREFIX "StateChanged"
 #endif
 
-#include <stdlib.h>
 #include <stdlib.h>
 #include <math.h>
 
+#include <QApplication>
+#include <QDomDocument>
+#include <QFileInfo>
+
 
 //-------------------------------------------------------------------------
 // samplv1_lv2 - impl.
@@ -76,27 +79,63 @@ samplv1_lv2::samplv1_lv2 (
 {
 	::memset(&m_urids, 0, sizeof(m_urids));
 
-	m_urid_map = NULL;
-	m_atom_in  = NULL;
-	m_atom_out = NULL;
-	m_schedule = NULL;
+	m_urid_map = nullptr;
+	m_atom_in  = nullptr;
+	m_atom_out = nullptr;
+	m_schedule = nullptr;
 	m_ndelta   = 0;
 
-	const LV2_Options_Option *host_options = NULL;
+	const LV2_Options_Option *host_options = nullptr;
 
 	for (int i = 0; host_features && host_features[i]; ++i) {
 		const LV2_Feature *host_feature = host_features[i];
 		if (::strcmp(host_feature->URI, LV2_URID_MAP_URI) == 0) {
 			m_urid_map = (LV2_URID_Map *) host_feature->data;
 			if (m_urid_map) {
- 				m_urids.gen1_sample = m_urid_map->map(
- 					m_urid_map->handle, SAMPLV1_LV2_PREFIX "GEN1_SAMPLE");
- 				m_urids.gen1_loop_start = m_urid_map->map(
- 					m_urid_map->handle, SAMPLV1_LV2_PREFIX "GEN1_LOOP_START");
- 				m_urids.gen1_loop_end = m_urid_map->map(
- 					m_urid_map->handle, SAMPLV1_LV2_PREFIX "GEN1_LOOP_END");
+			#if 1//SAMPLV1_LV2_LEGACY
+				m_urids.gen1_sample = m_urid_map->map(
+					m_urid_map->handle, SAMPLV1_LV2_PREFIX "GEN1_SAMPLE");
+				m_urids.gen1_offset_start = m_urid_map->map(
+					m_urid_map->handle, SAMPLV1_LV2_PREFIX "GEN1_OFFSET_START");
+				m_urids.gen1_offset_end = m_urid_map->map(
+					m_urid_map->handle, SAMPLV1_LV2_PREFIX "GEN1_OFFSET_END");
+				m_urids.gen1_loop_start = m_urid_map->map(
+					m_urid_map->handle, SAMPLV1_LV2_PREFIX "GEN1_LOOP_START");
+				m_urids.gen1_loop_end = m_urid_map->map(
+					m_urid_map->handle, SAMPLV1_LV2_PREFIX "GEN1_LOOP_END");
+				m_urids.gen1_loop_fade = m_urid_map->map(
+					m_urid_map->handle, SAMPLV1_LV2_PREFIX "GEN1_LOOP_FADE");
+				m_urids.gen1_loop_zero = m_urid_map->map(
+					m_urid_map->handle, SAMPLV1_LV2_PREFIX "GEN1_LOOP_ZERO");
+			#endif
+				m_urids.p101_sample_file = m_urid_map->map(
+					m_urid_map->handle, SAMPLV1_LV2_PREFIX "P101_SAMPLE_FILE");
+				m_urids.p102_offset_start = m_urid_map->map(
+					m_urid_map->handle, SAMPLV1_LV2_PREFIX "P102_OFFSET_START");
+				m_urids.p103_offset_end = m_urid_map->map(
+					m_urid_map->handle, SAMPLV1_LV2_PREFIX "P103_OFFSET_END");
+				m_urids.p104_loop_start = m_urid_map->map(
+					m_urid_map->handle, SAMPLV1_LV2_PREFIX "P104_LOOP_START");
+				m_urids.p105_loop_end = m_urid_map->map(
+					m_urid_map->handle, SAMPLV1_LV2_PREFIX "P105_LOOP_END");
+				m_urids.p106_loop_fade = m_urid_map->map(
+					m_urid_map->handle, SAMPLV1_LV2_PREFIX "P106_LOOP_FADE");
+				m_urids.p107_loop_zero = m_urid_map->map(
+					m_urid_map->handle, SAMPLV1_LV2_PREFIX "P107_LOOP_ZERO");
 				m_urids.gen1_update = m_urid_map->map(
 					m_urid_map->handle, SAMPLV1_LV2_PREFIX "GEN1_UPDATE");
+				m_urids.p201_tuning_enabled = m_urid_map->map(
+					m_urid_map->handle, SAMPLV1_LV2_PREFIX "P201_TUNING_ENABLED");
+				m_urids.p202_tuning_refPitch = m_urid_map->map(
+					m_urid_map->handle, SAMPLV1_LV2_PREFIX "P202_TUNING_REF_PITCH");
+				m_urids.p203_tuning_refNote = m_urid_map->map(
+					m_urid_map->handle, SAMPLV1_LV2_PREFIX "P203_TUNING_REF_NOTE");
+				m_urids.p204_tuning_scaleFile = m_urid_map->map(
+					m_urid_map->handle, SAMPLV1_LV2_PREFIX "P204_TUNING_SCALE_FILE");
+				m_urids.p205_tuning_keyMapFile = m_urid_map->map(
+					m_urid_map->handle, SAMPLV1_LV2_PREFIX "P205_TUNING_KEYMAP_FILE");
+				m_urids.tun1_update = m_urid_map->map(
+					m_urid_map->handle, SAMPLV1_LV2_PREFIX "TUN1_UPDATE");
 				m_urids.atom_Blank = m_urid_map->map(
 					m_urid_map->handle, LV2_ATOM__Blank);
 				m_urids.atom_Object = m_urid_map->map(
@@ -105,6 +144,8 @@ samplv1_lv2::samplv1_lv2 (
 					m_urid_map->handle, LV2_ATOM__Float);
 				m_urids.atom_Int = m_urid_map->map(
 					m_urid_map->handle, LV2_ATOM__Int);
+				m_urids.atom_Bool = m_urid_map->map(
+					m_urid_map->handle, LV2_ATOM__Bool);
 				m_urids.atom_Path = m_urid_map->map(
 					m_urid_map->handle, LV2_ATOM__Path);
 				m_urids.time_Position = m_urid_map->map(
@@ -113,12 +154,12 @@ samplv1_lv2::samplv1_lv2 (
 					m_urid_map->handle, LV2_TIME__beatsPerMinute);
 				m_urids.midi_MidiEvent = m_urid_map->map(
 					m_urid_map->handle, LV2_MIDI__MidiEvent);
- 				m_urids.midi_MidiEvent = m_urid_map->map(
- 					m_urid_map->handle, LV2_MIDI__MidiEvent);
+				m_urids.midi_MidiEvent = m_urid_map->map(
+					m_urid_map->handle, LV2_MIDI__MidiEvent);
 				m_urids.bufsz_minBlockLength = m_urid_map->map(
 					m_urid_map->handle, LV2_BUF_SIZE__minBlockLength);
- 				m_urids.bufsz_maxBlockLength = m_urid_map->map(
- 					m_urid_map->handle, LV2_BUF_SIZE__maxBlockLength);
+				m_urids.bufsz_maxBlockLength = m_urid_map->map(
+					m_urid_map->handle, LV2_BUF_SIZE__maxBlockLength);
 			#ifdef LV2_BUF_SIZE__nominalBlockLength
 				m_urids.bufsz_nominalBlockLength = m_urid_map->map(
 					m_urid_map->handle, LV2_BUF_SIZE__nominalBlockLength);
@@ -127,7 +168,7 @@ samplv1_lv2::samplv1_lv2 (
 					m_urid_map->handle, LV2_STATE__StateChanged);
 			#ifdef CONFIG_LV2_PATCH
 				m_urids.patch_Get = m_urid_map->map(
- 					m_urid_map->handle, LV2_PATCH__Get);
+					m_urid_map->handle, LV2_PATCH__Get);
 				m_urids.patch_Set = m_urid_map->map(
 					m_urid_map->handle, LV2_PATCH__Set);
 				m_urids.patch_Put = m_urid_map->map(
@@ -135,7 +176,7 @@ samplv1_lv2::samplv1_lv2 (
 				m_urids.patch_body = m_urid_map->map(
 					m_urid_map->handle, LV2_PATCH__body);
 				m_urids.patch_property = m_urid_map->map(
- 					m_urid_map->handle, LV2_PATCH__property);
+					m_urid_map->handle, LV2_PATCH__property);
 				m_urids.patch_value = m_urid_map->map(
  					m_urid_map->handle, LV2_PATCH__value);
 			#endif
@@ -179,7 +220,7 @@ samplv1_lv2::samplv1_lv2 (
 	m_ins  = new float * [nchannels];
 	m_outs = new float * [nchannels];
 	for (uint16_t k = 0; k < nchannels; ++k)
-		m_ins[k] = m_outs[k] = NULL;
+		m_ins[k] = m_outs[k] = nullptr;
 }
 
 
@@ -237,7 +278,7 @@ void samplv1_lv2::run ( uint32_t nframes )
 
 	if (m_atom_in) {
 		LV2_ATOM_SEQUENCE_FOREACH(m_atom_in, event) {
-			if (event == NULL)
+			if (event == nullptr)
 				continue;
 			if (event->body.type == m_urids.midi_MidiEvent) {
 				uint8_t *data = (uint8_t *) LV2_ATOM_BODY(&event->body);
@@ -260,9 +301,9 @@ void samplv1_lv2::run ( uint32_t nframes )
 				const LV2_Atom_Object *object
 					= (LV2_Atom_Object *) &event->body;
 				if (object->body.otype == m_urids.time_Position) {
-					LV2_Atom *atom = NULL;
+					LV2_Atom *atom = nullptr;
 					lv2_atom_object_get(object,
-						m_urids.time_beatsPerMinute, &atom, NULL);
+						m_urids.time_beatsPerMinute, &atom, nullptr);
 					if (atom && atom->type == m_urids.atom_Float) {
 						const float host_bpm = ((LV2_Atom_Float *) atom)->body;
 						if (::fabsf(host_bpm - samplv1::tempo()) > 0.001f)
@@ -273,16 +314,19 @@ void samplv1_lv2::run ( uint32_t nframes )
 				else 
 				if (object->body.otype == m_urids.patch_Set) {
 					// set property value
-					const LV2_Atom *property = NULL;
-					const LV2_Atom *value = NULL;
+					const LV2_Atom *property = nullptr;
+					const LV2_Atom *value = nullptr;
 					lv2_atom_object_get(object,
 						m_urids.patch_property, &property,
 						m_urids.patch_value, &value, 0);
 					if (property && value && property->type == m_forge.URID) {
 						const uint32_t key = ((const LV2_Atom_URID *) property)->body;
 						const LV2_URID type = value->type;
-						if (key == m_urids.gen1_sample
-							&& type == m_urids.atom_Path) {
+						if ((key == m_urids.p101_sample_file
+						#if 1//SAMPLV1_LV2_LEGACY
+							 || key == m_urids.gen1_sample
+						#endif
+							 ) && type == m_urids.atom_Path) {
 							if (m_schedule) {
 								samplv1_lv2_worker_message mesg;
 								mesg.atom.type = key;
@@ -295,32 +339,134 @@ void samplv1_lv2::run ( uint32_t nframes )
 							}
 						}
 						else
-						if (key == m_urids.gen1_loop_start
-							&& type == m_urids.atom_Int) {
+						if ((key == m_urids.p102_offset_start
+						#if 1//SAMPLV1_LV2_LEGACY
+							|| key == m_urids.gen1_offset_start
+						#endif
+							) && type == m_urids.atom_Int) {
+							samplv1_sample *pSample = samplv1::sample();
+							if (pSample) {
+								const uint32_t offset_start
+									= *(uint32_t *) LV2_ATOM_BODY_CONST(value);
+								const uint32_t offset_end
+									= pSample->offsetEnd();
+								setOffsetRange(offset_start, offset_end);
+							}
+						}
+						else
+						if ((key == m_urids.p103_offset_end
+						#if 1//SAMPLV1_LV2_LEGACY
+							|| key == m_urids.gen1_offset_end
+						#endif
+							) && type == m_urids.atom_Int) {
+							samplv1_sample *pSample = samplv1::sample();
+							if (pSample) {
+								const uint32_t offset_start
+									= pSample->offsetStart();
+								const uint32_t offset_end
+									= *(uint32_t *) LV2_ATOM_BODY_CONST(value);
+								setOffsetRange(offset_start, offset_end);
+							}
+						}
+						else
+						if ((key == m_urids.p104_loop_start
+						#if 1//SAMPLV1_LV2_LEGACY
+							|| key == m_urids.gen1_loop_start
+						#endif
+							) && type == m_urids.atom_Int) {
 							samplv1_sample *pSample = samplv1::sample();
 							if (pSample) {
 								const uint32_t loop_start
 									= *(uint32_t *) LV2_ATOM_BODY_CONST(value);
-								const uint32_t loop_end = pSample->loopEnd();
+								const uint32_t loop_end
+									= pSample->loopEnd();
 								setLoopRange(loop_start, loop_end);
 							}
 						}
 						else
-						if (key == m_urids.gen1_loop_end
-							&& type == m_urids.atom_Int) {
+						if ((key == m_urids.p105_loop_end
+						#if 1//SAMPLV1_LV2_LEGACY
+							|| key == m_urids.gen1_loop_end
+						#endif
+							) && type == m_urids.atom_Int) {
 							samplv1_sample *pSample = samplv1::sample();
 							if (pSample) {
-								const uint32_t loop_start = pSample->loopStart();
+								const uint32_t loop_start
+									= pSample->loopStart();
 								const uint32_t loop_end
 									= *(uint32_t *) LV2_ATOM_BODY_CONST(value);
 								setLoopRange(loop_start, loop_end);
 							}
 						}
+						else
+						if ((key == m_urids.p106_loop_fade
+						#if 1//SAMPLV1_LV2_LEGACY
+							|| key == m_urids.gen1_loop_fade
+						#endif
+							) && type == m_urids.atom_Int) {
+							const uint32_t loop_fade
+								= *(uint32_t *) LV2_ATOM_BODY_CONST(value);
+							setLoopFade(loop_fade);
+						}
+						else
+						if ((key == m_urids.p107_loop_zero
+						#if 1//SAMPLV1_LV2_LEGACY
+							|| key == m_urids.gen1_loop_zero
+						#endif
+							) && (type == m_urids.atom_Bool
+						#if 1//SAMPLV1_LV2_LEGACY
+							|| type == m_urids.atom_Int
+						#endif
+							)) {
+							const uint32_t loop_zero
+								= *(uint32_t *) LV2_ATOM_BODY_CONST(value);
+							setLoopZero(loop_zero > 0);
+						}
+						else
+						if (key == m_urids.p201_tuning_enabled
+							&& type == m_urids.atom_Bool) {
+							const uint32_t enabled
+								= *(uint32_t *) LV2_ATOM_BODY_CONST(value);
+							samplv1::setTuningEnabled(enabled > 0);
+							updateTuning();
+						}
+						else
+						if (key == m_urids.p202_tuning_refPitch
+							&& type == m_urids.atom_Float) {
+							const float refPitch
+								= *(float *) LV2_ATOM_BODY_CONST(value);
+							samplv1::setTuningRefPitch(refPitch);
+							updateTuning();
+						}
+						else
+						if (key == m_urids.p203_tuning_refNote
+							&& type == m_urids.atom_Int) {
+							const uint32_t refNote
+								= *(uint32_t *) LV2_ATOM_BODY_CONST(value);
+							samplv1::setTuningRefNote(refNote);
+							updateTuning();
+						}
+						else
+						if (key == m_urids.p204_tuning_scaleFile
+							&& type == m_urids.atom_Path) {
+							const char *scaleFile
+								= (const char *) LV2_ATOM_BODY_CONST(value);
+							samplv1::setTuningScaleFile(scaleFile);
+							updateTuning();
+						}
+						else
+						if (key == m_urids.p205_tuning_keyMapFile
+							&& type == m_urids.atom_Path) {
+							const char *keyMapFile
+								= (const char *) LV2_ATOM_BODY_CONST(value);
+							samplv1::setTuningKeyMapFile(keyMapFile);
+							updateTuning();
+						}
 					}
 				}
 				else
 				if (object->body.otype == m_urids.patch_Get) {
-					// put property values (probably to UI)
+					// put all property values (probably to UI)
 					patch_put(ndelta);
 				}
 			#endif	// CONFIG_LV2_PATCH
@@ -328,15 +474,14 @@ void samplv1_lv2::run ( uint32_t nframes )
 		}
 		// remember last time for worker response
 		m_ndelta = ndelta;
-	//	m_atom_in = NULL;
+	//	m_atom_in = nullptr;
 	}
 
 	if (nframes > ndelta)
 		samplv1::process(ins, outs, nframes - ndelta);
 
-	// test for current sample loop change
-	if (samplv1::sampleLoopTest())
-		updateSample();
+	// test for sample offset/loop changes
+	samplv1::sampleOffsetLoopTest();
 }
 
 
@@ -358,6 +503,42 @@ uint32_t samplv1_lv2::urid_map ( const char *uri ) const
 }
 
 
+//-------------------------------------------------------------------------
+// samplv1_lv2 - Instantiation and cleanup.
+//
+
+QApplication *samplv1_lv2::g_qapp_instance = nullptr;
+unsigned int  samplv1_lv2::g_qapp_refcount = 0;
+
+
+void samplv1_lv2::qapp_instantiate (void)
+{
+	if (qApp == nullptr && g_qapp_instance == nullptr) {
+		static int s_argc = 1;
+		static const char *s_argv[] = { __func__, nullptr };
+		g_qapp_instance = new QApplication(s_argc, (char **) s_argv);
+	}
+
+	if (g_qapp_instance)
+		g_qapp_refcount++;
+}
+
+
+void samplv1_lv2::qapp_cleanup (void)
+{
+	if (g_qapp_instance && --g_qapp_refcount == 0) {
+		delete g_qapp_instance;
+		g_qapp_instance = nullptr;
+	}
+}
+
+
+QApplication *samplv1_lv2::qapp_instance (void)
+{
+	return g_qapp_instance;
+}
+
+
 //-------------------------------------------------------------------------
 // samplv1_lv2 - LV2 State interface.
 //
@@ -367,10 +548,10 @@ static LV2_State_Status samplv1_lv2_state_save ( LV2_Handle instance,
 	uint32_t flags, const LV2_Feature *const *features )
 {
 	samplv1_lv2 *pPlugin = static_cast<samplv1_lv2 *> (instance);
-	if (pPlugin == NULL)
+	if (pPlugin == nullptr)
 		return LV2_STATE_ERR_UNKNOWN;
 
-	LV2_State_Map_Path *map_path = NULL;
+	LV2_State_Map_Path *map_path = nullptr;
 	for (int i = 0; features && features[i]; ++i) {
 		if (::strcmp(features[i]->URI, LV2_STATE__mapPath) == 0) {
 			map_path = (LV2_State_Map_Path *) features[i]->data;
@@ -378,7 +559,7 @@ static LV2_State_Status samplv1_lv2_state_save ( LV2_Handle instance,
 		}
 	}
 
-	uint32_t key = pPlugin->urid_map(SAMPLV1_LV2_PREFIX "GEN1_SAMPLE");
+	uint32_t key = pPlugin->urid_map(SAMPLV1_LV2_PREFIX "P101_SAMPLE_FILE");
 	if (key == 0)
 		return LV2_STATE_ERR_NO_PROPERTY;
 
@@ -397,37 +578,97 @@ static LV2_State_Status samplv1_lv2_state_save ( LV2_Handle instance,
 	if (value && map_path)
 		value = (*map_path->abstract_path)(map_path->handle, value);
 
-	if (value == NULL)
+	if (value == nullptr)
 		return LV2_STATE_ERR_UNKNOWN;
 
 	size_t size = ::strlen(value) + 1;
 
-	const LV2_State_Status result
-		= (*store)(handle, key, value, size, type, flags);
+	(*store)(handle, key, value, size, type, flags);
 
 	if (map_path)
 		::free((void *) value);
 
-	// Save extra loop points...
-	uint32_t loop_start = pPlugin->loopStart();
-	uint32_t loop_end   = pPlugin->loopEnd();
-
-	if (loop_start < loop_end) {
-		type = pPlugin->urid_map(LV2_ATOM__Int);
-		if (type) {
-			size = sizeof(uint32_t);
-			value = (const char *) &loop_start;
-			key = pPlugin->urid_map(SAMPLV1_LV2_PREFIX "GEN1_LOOP_START");
+	// Save state properties...
+	size = sizeof(uint32_t);
+	type = pPlugin->urid_map(LV2_ATOM__Int);
+	if (type) {
+		// Offset state...
+		uint32_t offset_start = pPlugin->offsetStart();
+		uint32_t offset_end   = pPlugin->offsetEnd();
+		if (offset_start < offset_end) {
+			value = (const char *) &offset_start;
+			key = pPlugin->urid_map(SAMPLV1_LV2_PREFIX "P102_OFFSET_START");
 			if (key)
 				(*store)(handle, key, value, size, type, flags);
-			value = (const char *) &loop_end;
-			key = pPlugin->urid_map(SAMPLV1_LV2_PREFIX "GEN1_LOOP_END");
+			value = (const char *) &offset_end;
+			key = pPlugin->urid_map(SAMPLV1_LV2_PREFIX "P103_OFFSET_END");
 			if (key)
 				(*store)(handle, key, value, size, type, flags);
 		}
+		// Loop state...
+		uint32_t loop_start = pPlugin->loopStart();
+		uint32_t loop_end   = pPlugin->loopEnd();
+		if (loop_start < loop_end) {
+			value = (const char *) &loop_start;
+			key = pPlugin->urid_map(SAMPLV1_LV2_PREFIX "P104_LOOP_START");
+			if (key)
+				(*store)(handle, key, value, size, type, flags);
+			value = (const char *) &loop_end;
+			key = pPlugin->urid_map(SAMPLV1_LV2_PREFIX "P105_LOOP_END");
+			if (key)
+				(*store)(handle, key, value, size, type, flags);
+		}
+		uint32_t loop_fade = pPlugin->loopFade();
+		value = (const char *) &loop_fade;
+		key = pPlugin->urid_map(SAMPLV1_LV2_PREFIX "P106_LOOP_FADE");
+		if (key)
+			(*store)(handle, key, value, size, type, flags);
 	}
 
-	return result;
+	type = pPlugin->urid_map(LV2_ATOM__Bool);
+	if (type) {
+		uint32_t loop_zero = pPlugin->isLoopZero() ? 1 : 0;
+		value = (const char *) &loop_zero;
+		key = pPlugin->urid_map(SAMPLV1_LV2_PREFIX "P107_LOOP_ZERO");
+		if (key)
+			(*store)(handle, key, value, size, type, flags);
+	}
+
+	// FIXME: At this time, only micro-tonal (aka. tuning) settings
+	// are posed to be saved into some XML chunk as state...
+	if (!pPlugin->isTuningEnabled())
+		return LV2_STATE_SUCCESS;
+
+	// Save all remaining state as XML chunk...
+	//
+	key = pPlugin->urid_map(SAMPLV1_LV2_PREFIX "state");
+	if (key == 0)
+		return LV2_STATE_ERR_NO_PROPERTY;
+
+	type = pPlugin->urid_map(LV2_ATOM__Chunk);
+	if (type == 0)
+		return LV2_STATE_ERR_BAD_TYPE;
+#if 0
+	if ((flags & (LV2_STATE_IS_POD | LV2_STATE_IS_PORTABLE)) == 0)
+		return LV2_STATE_ERR_BAD_FLAGS;
+#else
+	flags |= (LV2_STATE_IS_POD | LV2_STATE_IS_PORTABLE);
+#endif
+
+	QDomDocument doc(SAMPLV1_TITLE);
+	QDomElement eState = doc.createElement("state");
+
+	QDomElement eTuning = doc.createElement("tuning");
+	samplv1_param::saveTuning(pPlugin, doc, eTuning);
+	eState.appendChild(eTuning);
+
+	doc.appendChild(eState);
+
+	const QByteArray data(doc.toByteArray());
+	value = data.constData();
+	size = data.size();
+
+	return (*store)(handle, key, value, size, type, flags);
 }
 
 
@@ -436,10 +677,10 @@ static LV2_State_Status samplv1_lv2_state_restore ( LV2_Handle instance,
 	uint32_t flags, const LV2_Feature *const *features )
 {
 	samplv1_lv2 *pPlugin = static_cast<samplv1_lv2 *> (instance);
-	if (pPlugin == NULL)
+	if (pPlugin == nullptr)
 		return LV2_STATE_ERR_UNKNOWN;
 
-	LV2_State_Map_Path *map_path = NULL;
+	LV2_State_Map_Path *map_path = nullptr;
 	for (int i = 0; features && features[i]; ++i) {
 		if (::strcmp(features[i]->URI, LV2_STATE__mapPath) == 0) {
 			map_path = (LV2_State_Map_Path *) features[i]->data;
@@ -447,7 +688,7 @@ static LV2_State_Status samplv1_lv2_state_restore ( LV2_Handle instance,
 		}
 	}
 
-	uint32_t key = pPlugin->urid_map(SAMPLV1_LV2_PREFIX "GEN1_SAMPLE");
+	uint32_t key = pPlugin->urid_map(SAMPLV1_LV2_PREFIX "P101_SAMPLE_FILE");
 	if (key == 0)
 		return LV2_STATE_ERR_NO_PROPERTY;
 
@@ -465,6 +706,16 @@ static LV2_State_Status samplv1_lv2_state_restore ( LV2_Handle instance,
 
 	const char *value
 		= (const char *) (*retrieve)(handle, key, &size, &type, &flags);
+#if 1//SAMPLV1_LV2_LEGACY
+	if (value == nullptr) {
+		key = pPlugin->urid_map(SAMPLV1_LV2_PREFIX "GEN1_SAMPLE");
+		if (key) {
+			size = 0;
+			type = 0;
+			value = (const char *) (*retrieve)(handle, key, &size, &type, &flags);
+		}
+#endif
+	}
 
 	if (size < 2)
 		return LV2_STATE_ERR_UNKNOWN;
@@ -478,41 +729,187 @@ static LV2_State_Status samplv1_lv2_state_restore ( LV2_Handle instance,
 	if (value && map_path)
 		value = (*map_path->absolute_path)(map_path->handle, value);
 
-	if (value == NULL)
+	if (value == nullptr)
 		return LV2_STATE_ERR_UNKNOWN;
 
-	pPlugin->setSampleFile((const char *) value);
+	// Make sure to get rid of any symlinks...
+	pPlugin->setSampleFile(
+		QFileInfo(QString::fromUtf8(value))
+			.canonicalFilePath().toUtf8().constData());
 
 	if (map_path)
 		::free((void *) value);
 
-	// Restore extra loop points.
+	// Restore state properties...
+	uint32_t offset_start = 0;
+	uint32_t offset_end = 0;
 	uint32_t loop_start = 0;
 	uint32_t loop_end   = 0;
+	uint32_t loop_fade  = 0;
+	uint32_t loop_zero  = 1; // default: true.
 
 	const uint32_t int_type = pPlugin->urid_map(LV2_ATOM__Int);
 	if (int_type) {
-		key = pPlugin->urid_map(SAMPLV1_LV2_PREFIX "GEN1_LOOP_START");
+		key = pPlugin->urid_map(SAMPLV1_LV2_PREFIX "P102_OFFSET_START");
 		if (key) {
 			size = 0;
 			type = 0;
 			value = (const char *) (*retrieve)(handle, key, &size, &type, &flags);
+		#if 1//SAMPLV1_LV2_LEGACY
+			if (value == nullptr) {
+				key = pPlugin->urid_map(SAMPLV1_LV2_PREFIX "GEN1_OFFSET_START");
+				if (key) {
+					size = 0;
+					type = 0;
+					value = (const char *) (*retrieve)(handle, key, &size, &type, &flags);
+				}
+			}
+		#endif
+			if (value && size == sizeof(uint32_t) && type == int_type)
+				offset_start = *(uint32_t *) value;
+		}
+		key = pPlugin->urid_map(SAMPLV1_LV2_PREFIX "P103_OFFSET_END");
+		if (key) {
+			size = 0;
+			type = 0;
+			value = (const char *) (*retrieve)(handle, key, &size, &type, &flags);
+		#if 1//SAMPLV1_LV2_LEGACY
+			if (value == nullptr) {
+				key = pPlugin->urid_map(SAMPLV1_LV2_PREFIX "GEN1_OFFSET_END");
+				if (key) {
+					size = 0;
+					type = 0;
+					value = (const char *) (*retrieve)(handle, key, &size, &type, &flags);
+				}
+			}
+		#endif
+			if (value && size == sizeof(uint32_t) && type == int_type)
+				offset_end = *(uint32_t *) value;
+		}
+		key = pPlugin->urid_map(SAMPLV1_LV2_PREFIX "P104_LOOP_START");
+		if (key) {
+			size = 0;
+			type = 0;
+			value = (const char *) (*retrieve)(handle, key, &size, &type, &flags);
+		#if 1//SAMPLV1_LV2_LEGACY
+			if (value == nullptr) {
+				key = pPlugin->urid_map(SAMPLV1_LV2_PREFIX "GEN1_LOOP_START");
+				if (key) {
+					size = 0;
+					type = 0;
+					value = (const char *) (*retrieve)(handle, key, &size, &type, &flags);
+				}
+			}
+		#endif
 			if (value && size == sizeof(uint32_t) && type == int_type)
 				loop_start = *(uint32_t *) value;
 		}
-		key = pPlugin->urid_map(SAMPLV1_LV2_PREFIX "GEN1_LOOP_END");
+		key = pPlugin->urid_map(SAMPLV1_LV2_PREFIX "P105_LOOP_END");
 		if (key) {
 			size = 0;
 			type = 0;
 			value = (const char *) (*retrieve)(handle, key, &size, &type, &flags);
+		#if 1//SAMPLV1_LV2_LEGACY
+			if (value == nullptr) {
+				key = pPlugin->urid_map(SAMPLV1_LV2_PREFIX "GEN1_LOOP_END");
+				if (key) {
+					size = 0;
+					type = 0;
+					value = (const char *) (*retrieve)(handle, key, &size, &type, &flags);
+				}
+			}
+		#endif
 			if (value && size == sizeof(uint32_t) && type == int_type)
 				loop_end = *(uint32_t *) value;
 		}
+		key = pPlugin->urid_map(SAMPLV1_LV2_PREFIX "P106_LOOP_FADE");
+		if (key) {
+			size = 0;
+			type = 0;
+			value = (const char *) (*retrieve)(handle, key, &size, &type, &flags);
+		#if 1//SAMPLV1_LV2_LEGACY
+			if (value == nullptr) {
+				key = pPlugin->urid_map(SAMPLV1_LV2_PREFIX "GEN1_LOOP_FADE");
+				if (key) {
+					size = 0;
+					type = 0;
+					value = (const char *) (*retrieve)(handle, key, &size, &type, &flags);
+				}
+			}
+		#endif
+			if (value && size == sizeof(uint32_t) && type == int_type)
+				loop_fade = *(uint32_t *) value;
+		}
 	}
 
+	const uint32_t bool_type = pPlugin->urid_map(LV2_ATOM__Bool);
+	if (int_type || bool_type) {
+		key = pPlugin->urid_map(SAMPLV1_LV2_PREFIX "P107_LOOP_ZERO");
+		if (key) {
+			size = 0;
+			type = 0;
+			value = (const char *) (*retrieve)(handle, key, &size, &type, &flags);
+		#if 1//SAMPLV1_LV2_LEGACY
+			if (value == nullptr) {
+				key = pPlugin->urid_map(SAMPLV1_LV2_PREFIX "GEN1_LOOP_ZERO");
+				if (key) {
+					size = 0;
+					type = 0;
+					value = (const char *) (*retrieve)(handle, key, &size, &type, &flags);
+				}
+			}
+		#endif
+			if (value && size == sizeof(uint32_t)
+				&& (type == int_type || type == bool_type))
+				loop_zero = *(uint32_t *) value;
+		}
+	}
+
+	pPlugin->setLoopZero(loop_zero > 0);
+	pPlugin->setLoopFade(loop_fade);
+
 	if (loop_start < loop_end)
 		pPlugin->setLoopRange(loop_start, loop_end);
 
+	if (offset_start < offset_end)
+		pPlugin->setOffsetRange(offset_start, offset_end);
+
+	// Retrieve any remaining state as XML chunk...
+	//
+	key = pPlugin->urid_map(SAMPLV1_LV2_PREFIX "state");
+	if (key == 0)
+		return LV2_STATE_ERR_NO_PROPERTY;
+
+	const uint32_t chunk_type = pPlugin->urid_map(LV2_ATOM__Chunk);
+	if (chunk_type == 0)
+		return LV2_STATE_ERR_BAD_TYPE;
+
+	size = 0;
+	type = 0;
+//	flags = 0;
+
+	value = (const char *) (*retrieve)(handle, key, &size, &type, &flags);
+
+	if (value != nullptr && size > 2 && type == chunk_type
+		&& (flags & (LV2_STATE_IS_POD | LV2_STATE_IS_PORTABLE))) {
+		
+		QDomDocument doc(SAMPLV1_TITLE);
+		if (doc.setContent(QByteArray(value, size))) {
+			QDomElement eState = doc.documentElement();
+			if (eState.tagName() == "state") {
+				for (QDomNode nChild = eState.firstChild();
+						!nChild.isNull();
+							nChild = nChild.nextSibling()) {
+					QDomElement eChild = nChild.toElement();
+					if (eChild.isNull())
+						continue;
+					if (eChild.tagName() == "tuning")
+						samplv1_param::loadTuning(pPlugin, eChild);
+				}
+			}
+		}
+	}
+
 	pPlugin->reset();
 
 	samplv1_sched::sync_notify(pPlugin, samplv1_sched::Sample, 1);
@@ -555,7 +952,7 @@ const LV2_Program_Descriptor *samplv1_lv2::get_program ( uint32_t index )
 		}
 	}
 
-	return NULL;
+	return nullptr;
 }
 
 void samplv1_lv2::select_program ( uint32_t bank, uint32_t program )
@@ -572,7 +969,7 @@ void samplv1_lv2::updatePreset ( bool /*bDirty*/ )
 		samplv1_lv2_worker_message mesg;
 		mesg.atom.type = m_urids.state_StateChanged;
 		mesg.atom.size = 0; // nothing else matters.
-		mesg.sample_path = NULL;
+		mesg.sample_path = nullptr;
 		m_schedule->schedule_work(
 			m_schedule->handle, sizeof(mesg), &mesg);
 	}
@@ -592,8 +989,23 @@ void samplv1_lv2::updateSample (void)
 }
 
 
-bool samplv1_lv2::worker_work ( const void *data, uint32_t /*size*/ )
+void samplv1_lv2::updateTuning (void)
 {
+	if (m_schedule) {
+		samplv1_lv2_worker_message mesg;
+		mesg.atom.type = m_urids.tun1_update;
+		mesg.atom.size = 0; // nothing else matters.
+		m_schedule->schedule_work(
+			m_schedule->handle, sizeof(mesg), &mesg);
+	}
+}
+
+
+bool samplv1_lv2::worker_work ( const void *data, uint32_t size )
+{
+	if (size != sizeof(samplv1_lv2_worker_message))
+		return false;
+
 	const samplv1_lv2_worker_message *mesg
 		= (const samplv1_lv2_worker_message *) data;
 
@@ -603,17 +1015,29 @@ bool samplv1_lv2::worker_work ( const void *data, uint32_t /*size*/ )
 	if (mesg->atom.type == m_urids.gen1_update)
 		return true;
 	else
-	if (mesg->atom.type == m_urids.gen1_sample) {
+	if (mesg->atom.type == m_urids.p101_sample_file
+	#if 0//SAMPLV1_LV2_LEGACY
+		|| mesg->atom.type == m_urids.gen1_sample
+	#endif
+		) {
 		samplv1::setSampleFile(mesg->sample_path);
 		return true;
 	}
+	else
+	if (mesg->atom.type == m_urids.tun1_update) {
+		samplv1::resetTuning();
+		return true;
+	}
 
 	return false;
 }
 
 
-bool samplv1_lv2::worker_response ( const void *data, uint32_t /*size*/ )
+bool samplv1_lv2::worker_response ( const void *data, uint32_t size )
 {
+	if (size != sizeof(samplv1_lv2_worker_message))
+		return false;
+
 	const samplv1_lv2_worker_message *mesg
 		= (const samplv1_lv2_worker_message *) data;
 	if (mesg->atom.type == m_urids.state_StateChanged)
@@ -623,7 +1047,7 @@ bool samplv1_lv2::worker_response ( const void *data, uint32_t /*size*/ )
 	samplv1_sched::sync_notify(this, samplv1_sched::Sample, 0);
 
 #ifdef CONFIG_LV2_PATCH
-	return patch_put(m_ndelta);
+	return patch_put(m_ndelta, mesg->atom.type);
 #else
 	return true;
 #endif
@@ -644,15 +1068,16 @@ bool samplv1_lv2::state_changed (void)
 
 #ifdef CONFIG_LV2_PATCH
 
-bool samplv1_lv2::patch_put ( uint32_t ndelta )
+bool samplv1_lv2::patch_put ( uint32_t ndelta, uint32_t type )
 {
 	static char s_szNull[1] = {'\0'};
-	const char *pszSampleFile = NULL;
+
+	if (type == m_urids.patch_Put)
+		type = 0;
+
 	samplv1_sample *pSample = samplv1::sample();
-	if (pSample)
-		pszSampleFile = pSample->filename();
-	if (pszSampleFile == NULL)
-		pszSampleFile = s_szNull;
+	if (pSample == nullptr)
+		return false;
 
 	lv2_atom_forge_frame_time(&m_forge, ndelta);
 
@@ -662,12 +1087,65 @@ bool samplv1_lv2::patch_put ( uint32_t ndelta )
 
 	LV2_Atom_Forge_Frame body_frame;
 	lv2_atom_forge_object(&m_forge, &body_frame, 0, 0);
-	lv2_atom_forge_key(&m_forge, m_urids.gen1_sample);
-	lv2_atom_forge_path(&m_forge, pszSampleFile, ::strlen(pszSampleFile) + 1);
-	lv2_atom_forge_key(&m_forge, m_urids.gen1_loop_start);
-	lv2_atom_forge_int(&m_forge, pSample->loopStart());
-	lv2_atom_forge_key(&m_forge, m_urids.gen1_loop_end);
-	lv2_atom_forge_int(&m_forge, pSample->loopEnd());
+
+	if (type == 0 || type == m_urids.p101_sample_file) {
+		const char *pszSampleFile = pSample->filename();
+		if (pszSampleFile == nullptr)
+			pszSampleFile = s_szNull;
+		lv2_atom_forge_key(&m_forge, m_urids.p101_sample_file);
+		lv2_atom_forge_path(&m_forge, pszSampleFile, ::strlen(pszSampleFile) + 1);
+	}
+	if (type == 0 || type == m_urids.p102_offset_start) {
+		lv2_atom_forge_key(&m_forge, m_urids.p102_offset_start);
+		lv2_atom_forge_int(&m_forge, pSample->offsetStart());
+	}
+	if (type == 0 || type == m_urids.p103_offset_end) {
+		lv2_atom_forge_key(&m_forge, m_urids.p103_offset_end);
+		lv2_atom_forge_int(&m_forge, pSample->offsetEnd());
+	}
+	if (type == 0 || type == m_urids.p104_loop_start) {
+		lv2_atom_forge_key(&m_forge, m_urids.p104_loop_start);
+		lv2_atom_forge_int(&m_forge, pSample->loopStart());
+	}
+	if (type == 0 || type == m_urids.p105_loop_end) {
+		lv2_atom_forge_key(&m_forge, m_urids.p105_loop_end);
+		lv2_atom_forge_int(&m_forge, pSample->loopEnd());
+	}
+	if (type == 0 || type == m_urids.p106_loop_fade) {
+		lv2_atom_forge_key(&m_forge, m_urids.p106_loop_fade);
+		lv2_atom_forge_int(&m_forge, pSample->loopCrossFade());
+	}
+	if (type == 0 || type == m_urids.p107_loop_zero) {
+		lv2_atom_forge_key(&m_forge, m_urids.p107_loop_zero);
+		lv2_atom_forge_bool(&m_forge, pSample->isLoopZeroCrossing());
+	}
+
+	if (type == 0 || type == m_urids.p201_tuning_enabled) {
+		lv2_atom_forge_key(&m_forge, m_urids.p201_tuning_enabled);
+		lv2_atom_forge_bool(&m_forge, samplv1::isTuningEnabled());
+	}
+	if (type == 0 || type == m_urids.p202_tuning_refPitch) {
+		lv2_atom_forge_key(&m_forge, m_urids.p202_tuning_refPitch);
+		lv2_atom_forge_float(&m_forge, samplv1::tuningRefPitch());
+	}
+	if (type == 0 || type == m_urids.p203_tuning_refNote) {
+		lv2_atom_forge_key(&m_forge, m_urids.p203_tuning_refNote);
+		lv2_atom_forge_int(&m_forge, samplv1::tuningRefNote());
+	}
+	if (type == 0 || type == m_urids.p204_tuning_scaleFile) {
+		const char *pszScaleFile = samplv1::tuningScaleFile();
+		if (pszScaleFile == nullptr)
+			pszScaleFile = s_szNull;
+		lv2_atom_forge_key(&m_forge, m_urids.p204_tuning_scaleFile);
+		lv2_atom_forge_path(&m_forge, pszScaleFile, ::strlen(pszScaleFile) + 1);
+	}
+	if (type == 0 || type == m_urids.p205_tuning_keyMapFile) {
+		const char *pszKeyMapFile = samplv1::tuningKeyMapFile();
+		if (pszKeyMapFile == nullptr)
+			pszKeyMapFile = s_szNull;
+		lv2_atom_forge_key(&m_forge, m_urids.p205_tuning_keyMapFile);
+		lv2_atom_forge_path(&m_forge, pszKeyMapFile, ::strlen(pszKeyMapFile) + 1);
+	}
 
 	lv2_atom_forge_pop(&m_forge, &body_frame);
 	lv2_atom_forge_pop(&m_forge, &patch_frame);
@@ -686,6 +1164,8 @@ static LV2_Handle samplv1_lv2_instantiate (
 	const LV2_Descriptor *, double sample_rate, const char *,
 	const LV2_Feature *const *host_features )
 {
+	samplv1_lv2::qapp_instantiate();
+
 	return new samplv1_lv2(sample_rate, host_features);
 }
 
@@ -728,6 +1208,8 @@ static void samplv1_lv2_cleanup ( LV2_Handle instance )
 	samplv1_lv2 *pPlugin = static_cast<samplv1_lv2 *> (instance);
 	if (pPlugin)
 		delete pPlugin;
+
+	samplv1_lv2::qapp_cleanup();
 }
 
 
@@ -740,7 +1222,7 @@ static const LV2_Program_Descriptor *samplv1_lv2_programs_get_program (
 	if (pPlugin)
 		return pPlugin->get_program(index);
 	else
-		return NULL;
+		return nullptr;
 }
 
 static void samplv1_lv2_programs_select_program (
@@ -781,7 +1263,7 @@ static LV2_Worker_Status samplv1_lv2_worker_response (
 	if (pSampl && pSampl->worker_response(data, size))
 		return LV2_WORKER_SUCCESS;
 	else
-		return LV2_WORKER_SUCCESS;
+		return LV2_WORKER_ERR_UNKNOWN;
 }
 
 
@@ -789,7 +1271,7 @@ static const LV2_Worker_Interface samplv1_lv2_worker_interface =
 {
 	samplv1_lv2_worker_work,
 	samplv1_lv2_worker_response,
-	NULL
+	nullptr
 };
 
 
@@ -806,7 +1288,7 @@ static const void *samplv1_lv2_extension_data ( const char *uri )
 	if (::strcmp(uri, LV2_STATE__interface) == 0)
 		return &samplv1_lv2_state_interface;
 
-	return NULL;
+	return nullptr;
 }
 
 
@@ -825,7 +1307,7 @@ static const LV2_Descriptor samplv1_lv2_descriptor =
 
 LV2_SYMBOL_EXPORT const LV2_Descriptor *lv2_descriptor ( uint32_t index )
 {
-	return (index == 0 ? &samplv1_lv2_descriptor : NULL);
+	return (index == 0 ? &samplv1_lv2_descriptor : nullptr);
 }
 
 
