@@ -1,7 +1,7 @@
 // samplv1_lv2ui.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2016, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -24,17 +24,22 @@
 
 #include "samplv1_ui.h"
 
-#include "lv2.h"
 #include "lv2/lv2plug.in/ns/extensions/ui/ui.h"
 
 
 #define SAMPLV1_LV2UI_URI SAMPLV1_LV2_PREFIX "ui"
 
-#ifdef CONFIG_LV2_EXTERNAL_UI
+#ifdef CONFIG_LV2_UI_X11
+#include <QWindow>
+#define SAMPLV1_LV2UI_X11_URI SAMPLV1_LV2_PREFIX "ui_x11"
+#endif
+
+#ifdef CONFIG_LV2_UI_EXTERNAL
 #include "lv2_external_ui.h"
 #define SAMPLV1_LV2UI_EXTERNAL_URI SAMPLV1_LV2_PREFIX "ui_external"
 #endif
 
+
 // Forward decls.
 class samplv1_lv2;
 
