@@ -1,7 +1,7 @@
 // samplv1_param.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -50,11 +50,17 @@ struct ParamInfo {
 	// name            type,           def,    min,    max
 	{ "GEN1_SAMPLE",   PARAM_INT,    60.0f,   0.0f, 127.0f }, // GEN1 Sample
 	{ "GEN1_REVERSE",  PARAM_BOOL,    0.0f,   0.0f,   1.0f }, // GEN1 Reverse
+	{ "GEN1_OFFSET",   PARAM_BOOL,    0.0f,   0.0f,   1.0f }, // GEN1 Offset
+	{ "GEN1_OFFSET_1", PARAM_FLOAT,   0.0f,   0.0f,   1.0f }, // GEN1 Offset Start
+	{ "GEN1_OFFSET_2", PARAM_FLOAT,   1.0f,   0.0f,   1.0f }, // GEN1 Offset End
 	{ "GEN1_LOOP",     PARAM_BOOL,    0.0f,   0.0f,   1.0f }, // GEN1 Loop
+	{ "GEN1_LOOP_1",   PARAM_FLOAT,   0.0f,   0.0f,   1.0f }, // GEN1 Loop Start
+	{ "GEN1_LOOP_2",   PARAM_FLOAT,   1.0f,   0.0f,   1.0f }, // GEN1 Loop End
 	{ "GEN1_OCTAVE",   PARAM_FLOAT,   0.0f,  -4.0f,   4.0f }, // GEN1 Octave
 	{ "GEN1_TUNING",   PARAM_FLOAT,   0.0f,  -1.0f,   1.0f }, // GEN1 Tuning
 	{ "GEN1_GLIDE",    PARAM_FLOAT,   0.0f,   0.0f,   1.0f }, // GEN1 Glide
 	{ "GEN1_ENVTIME",  PARAM_FLOAT,   0.5f,   0.0f,   1.0f }, // GEN1 Env.Time
+	{ "DCF1_ENABLED",  PARAM_BOOL,    1.0f,   0.0f,   1.0f }, // DCF1 Enabled
 	{ "DCF1_CUTOFF",   PARAM_FLOAT,   1.0f,   0.0f,   1.0f }, // DCF1 Cutoff
 	{ "DCF1_RESO",     PARAM_FLOAT,   0.0f,   0.0f,   1.0f }, // DCF1 Resonance
 	{ "DCF1_TYPE",     PARAM_INT,     0.0f,   0.0f,   3.0f }, // DCF1 Type
@@ -64,6 +70,7 @@ struct ParamInfo {
 	{ "DCF1_DECAY",    PARAM_FLOAT,   0.2f,   0.0f,   1.0f }, // DCF1 Decay
 	{ "DCF1_SUSTAIN",  PARAM_FLOAT,   0.5f,   0.0f,   1.0f }, // DCF1 Sustain
 	{ "DCF1_RELEASE",  PARAM_FLOAT,   0.5f,   0.0f,   1.0f }, // DCF1 Release
+	{ "LFO1_ENABLED",  PARAM_BOOL,    1.0f,   0.0f,   1.0f }, // LFO1 Enabled
 	{ "LFO1_SHAPE",    PARAM_INT,     1.0f,   0.0f,   4.0f }, // LFO1 Wave Shape
 	{ "LFO1_WIDTH",    PARAM_FLOAT,   1.0f,   0.0f,   1.0f }, // LFO1 Wave Width
 	{ "LFO1_BPM",      PARAM_FLOAT, 180.0f,   0.0f, 360.0f }, // LFO1 BPM
@@ -79,6 +86,7 @@ struct ParamInfo {
 	{ "LFO1_DECAY",    PARAM_FLOAT,   0.1f,   0.0f,   1.0f }, // LFO1 Decay
 	{ "LFO1_SUSTAIN",  PARAM_FLOAT,   1.0f,   0.0f,   1.0f }, // LFO1 Sustain
 	{ "LFO1_RELEASE",  PARAM_FLOAT,   0.5f,   0.0f,   1.0f }, // LFO1 Release
+	{ "DCA1_ENABLED",  PARAM_BOOL,    1.0f,   0.0f,   1.0f }, // DCA1 Enabled
 	{ "DCA1_VOLUME",   PARAM_FLOAT,   0.5f,   0.0f,   1.0f }, // DCA1 Volume
 	{ "DCA1_ATTACK",   PARAM_FLOAT,   0.0f,   0.0f,   1.0f }, // DCA1 Attack
 	{ "DCA1_DECAY",    PARAM_FLOAT,   0.1f,   0.0f,   1.0f }, // DCA1 Decay
@@ -120,7 +128,10 @@ struct ParamInfo {
 	{ "REV1_FEEDB",    PARAM_FLOAT,   0.5f,   0.0f,   1.0f }, // Reverb Feedback
 	{ "REV1_WIDTH",    PARAM_FLOAT,   0.0f,  -1.0f,   1.0f }, // Reverb Width
 	{ "DYN1_COMPRESS", PARAM_BOOL,    0.0f,   0.0f,   1.0f }, // Dynamic Compressor
-	{ "DYN1_LIMITER",  PARAM_BOOL,    1.0f,   0.0f,   1.0f }  // Dynamic Limiter
+	{ "DYN1_LIMITER",  PARAM_BOOL,    1.0f,   0.0f,   1.0f }, // Dynamic Limiter
+
+	{ "KEY1_LOW",      PARAM_INT,     0.0f,   0.0f, 127.0f }, // Keyboard Low
+	{ "KEY1_HIGH",     PARAM_INT,   127.0f,   0.0f, 127.0f }  // Keyboard High
 };
 
 
@@ -136,6 +147,25 @@ float samplv1_param::paramDefaultValue ( samplv1::ParamIndex index )
 }
 
 
+float samplv1_param::paramSafeValue ( samplv1::ParamIndex index, float fValue )
+{
+	const ParamInfo& param = samplv1_params[index];
+
+	if (param.type == PARAM_BOOL)
+		return (fValue > 0.5f ? 1.0f : 0.0f);
+
+	if (fValue < param.min)
+		return param.min;
+	if (fValue > param.max)
+		return param.max;
+
+	if (param.type == PARAM_INT)
+		return ::rintf(fValue);
+	else
+		return fValue;
+}
+
+
 float samplv1_param::paramValue ( samplv1::ParamIndex index, float fScale )
 {
 	const ParamInfo& param = samplv1_params[index];
@@ -152,103 +182,6 @@ float samplv1_param::paramValue ( samplv1::ParamIndex index, float fScale )
 }
 
 
-// Sample serialization methods.
-void samplv1_param::loadSamples (
-	samplv1 *pSampl, const QDomElement& eSamples )
-{
-	if (pSampl == NULL)
-		return;
-
-	for (QDomNode nSample = eSamples.firstChild();
-			!nSample.isNull();
-				nSample = nSample.nextSibling()) {
-		QDomElement eSample = nSample.toElement();
-		if (eSample.isNull())
-			continue;
-		if (eSample.tagName() == "sample") {
-		//	int index = eSample.attribute("index").toInt();
-			QString sFilename;
-			uint32_t iLoopStart = 0;
-			uint32_t iLoopEnd = 0;
-			for (QDomNode nChild = eSample.firstChild();
-					!nChild.isNull();
-						nChild = nChild.nextSibling()) {
-				QDomElement eChild = nChild.toElement();
-				if (eChild.isNull())
-					continue;
-				if (eChild.tagName() == "filename") {
-					sFilename = eChild.text();
-				}
-				else
-				if (eChild.tagName() == "loop-start") {
-					iLoopStart = eChild.text().toULong();
-				}
-				else
-				if (eChild.tagName() == "loop-end") {
-					iLoopEnd = eChild.text().toULong();
-				}
-			}
-			// Legacy loader...
-			if (sFilename.isEmpty())
-				sFilename = eSample.text();
-			// Done it.
-			pSampl->setSampleFile(
-				QFileInfo(sFilename).canonicalFilePath().toUtf8().constData());
-			// Set actual sample loop points...
-			pSampl->setLoopRange(iLoopStart, iLoopEnd);
-		}
-	}
-}
-
-
-void samplv1_param::saveSamples (
-	samplv1 *pSampl, QDomDocument& doc, QDomElement& eSamples, bool bSymLink )
-{
-	if (pSampl == NULL)
-		return;
-
-	const char *pszSampleFile = pSampl->sampleFile();
-	if (pszSampleFile == NULL)
-		return;
-
-	QFileInfo fi(QString::fromUtf8(pszSampleFile));
-	if (bSymLink) {
-		const QString& sPath = fi.absoluteFilePath();
-		const QString& sName = fi.baseName();
-		const QString& sExt  = fi.completeSuffix();
-		const QString& sLink = sName
-			+ '-' + QString::number(qHash(sPath), 16)
-			+ '.' + sExt;
-		QFile(sPath).link(sLink);
-		fi.setFile(QDir::current(), sLink);
-	}
-
-	QDomElement eSample = doc.createElement("sample");
-	eSample.setAttribute("index", 0);
-	eSample.setAttribute("name", "GEN1_SAMPLE");
-
-	QDomElement eFilename = doc.createElement("filename");
-	eFilename.appendChild(doc.createTextNode(
-		QDir::current().relativeFilePath(fi.absoluteFilePath())));
-	eSample.appendChild(eFilename);
-
-	const uint32_t iLoopStart = pSampl->loopStart();
-	const uint32_t iLoopEnd   = pSampl->loopEnd();
-	if (iLoopStart < iLoopEnd) {
-		QDomElement eLoopStart = doc.createElement("loop-start");
-		eLoopStart.appendChild(doc.createTextNode(
-			QString::number(iLoopStart)));
-		eSample.appendChild(eLoopStart);
-		QDomElement eLoopEnd = doc.createElement("loop-end");
-		eLoopEnd.appendChild(doc.createTextNode(
-			QString::number(iLoopEnd)));
-		eSample.appendChild(eLoopEnd);
-	}
-
-	eSamples.appendChild(eSample);
-}
-
-
 float samplv1_param::paramScale ( samplv1::ParamIndex index, float fValue )
 {
 	const ParamInfo& param = samplv1_params[index];
@@ -271,12 +204,11 @@ bool samplv1_param::paramFloat ( samplv1::ParamIndex index )
 }
 
 
-
 // Preset serialization methods.
 bool samplv1_param::loadPreset (
 	samplv1 *pSampl, const QString& sFilename )
 {
-	if (pSampl == NULL)
+	if (pSampl == nullptr)
 		return false;
 
 	QFileInfo fi(sFilename);
@@ -297,6 +229,11 @@ bool samplv1_param::loadPreset (
 	if (!file.open(QIODevice::ReadOnly))
 		return false;
 
+	const bool running = pSampl->running(false);
+
+	pSampl->setTuningEnabled(false);
+	pSampl->reset();
+
 	static QHash<QString, samplv1::ParamIndex> s_hash;
 	if (s_hash.isEmpty()) {
 		for (uint32_t i = 0; i < samplv1::NUM_PARAMS; ++i) {
@@ -319,10 +256,6 @@ bool samplv1_param::loadPreset (
 				QDomElement eChild = nChild.toElement();
 				if (eChild.isNull())
 					continue;
-				if (eChild.tagName() == "samples") {
-					samplv1_param::loadSamples(pSampl, eChild);
-				}
-				else
 				if (eChild.tagName() == "params") {
 					for (QDomNode nParam = eChild.firstChild();
 							!nParam.isNull();
@@ -340,17 +273,28 @@ bool samplv1_param::loadPreset (
 								index = s_hash.value(sName);
 							}
 							const float fValue = eParam.text().toFloat();
-							pSampl->setParamValue(index, fValue);
+							pSampl->setParamValue(index,
+								samplv1_param::paramSafeValue(index, fValue));
 						}
 					}
 				}
+				else
+				if (eChild.tagName() == "samples") {
+					samplv1_param::loadSamples(pSampl, eChild);
+				}
+				else
+				if (eChild.tagName() == "tuning") {
+					samplv1_param::loadTuning(pSampl, eChild);
+				}
 			}
 		}
 	}
 
 	file.close();
 
+	pSampl->stabilize();
 	pSampl->reset();
+	pSampl->running(running);
 
 	QDir::setCurrent(currentDir.absolutePath());
 
@@ -361,9 +305,11 @@ bool samplv1_param::loadPreset (
 bool samplv1_param::savePreset (
 	samplv1 *pSampl, const QString& sFilename, bool bSymLink )
 {
-	if (pSampl == NULL)
+	if (pSampl == nullptr)
 		return false;
 
+	pSampl->stabilize();
+
 	const QFileInfo fi(sFilename);
 	const QDir currentDir(QDir::current());
 	QDir::setCurrent(fi.absolutePath());
@@ -388,6 +334,13 @@ bool samplv1_param::savePreset (
 		eParams.appendChild(eParam);
 	}
 	ePreset.appendChild(eParams);
+
+	if (pSampl->isTuningEnabled()) {
+		QDomElement eTuning = doc.createElement("tuning");
+		samplv1_param::saveTuning(pSampl, doc, eTuning, bSymLink);
+		ePreset.appendChild(eTuning);
+	}
+
 	doc.appendChild(ePreset);
 
 	QFile file(fi.filePath());
@@ -403,4 +356,260 @@ bool samplv1_param::savePreset (
 }
 
 
+// Sample serialization methods.
+void samplv1_param::loadSamples (
+	samplv1 *pSampl, const QDomElement& eSamples )
+{
+	if (pSampl == nullptr)
+		return;
+
+	for (QDomNode nSample = eSamples.firstChild();
+			!nSample.isNull();
+				nSample = nSample.nextSibling()) {
+		QDomElement eSample = nSample.toElement();
+		if (eSample.isNull())
+			continue;
+		if (eSample.tagName() == "sample") {
+		//	int index = eSample.attribute("index").toInt();
+			QString sSampleFile;
+			uint32_t iOffsetStart = 0;
+			uint32_t iOffsetEnd = 0;
+			uint32_t iLoopStart = 0;
+			uint32_t iLoopEnd = 0;
+			uint32_t iLoopFade = 0;
+			bool bLoopZero = true;
+			for (QDomNode nChild = eSample.firstChild();
+					!nChild.isNull();
+						nChild = nChild.nextSibling()) {
+				QDomElement eChild = nChild.toElement();
+				if (eChild.isNull())
+					continue;
+				if (eChild.tagName() == "filename") {
+					sSampleFile = eChild.text();
+				}
+				else
+				if (eChild.tagName() == "offset-start") {
+					iOffsetStart = eChild.text().toULong();
+				}
+				else
+				if (eChild.tagName() == "offset-end") {
+					iOffsetEnd = eChild.text().toULong();
+				}
+				else
+				if (eChild.tagName() == "loop-start") {
+					iLoopStart = eChild.text().toULong();
+				}
+				else
+				if (eChild.tagName() == "loop-end") {
+					iLoopEnd = eChild.text().toULong();
+				}
+				else
+				if (eChild.tagName() == "loop-fade") {
+					iLoopFade = eChild.text().toULong();
+				}
+				else
+				if (eChild.tagName() == "loop-zero") {
+					bLoopZero = (eChild.text().toInt() > 0);
+				}
+			}
+			// Legacy loader...
+			if (sSampleFile.isEmpty())
+				sSampleFile = eSample.text();
+			// Done it.
+			const QByteArray aSampleFile
+				= samplv1_param::loadFilename(sSampleFile).toUtf8();
+			pSampl->setSampleFile(aSampleFile.constData());
+			// Set actual sample loop points...
+			pSampl->setLoopZero(bLoopZero);
+			pSampl->setLoopFade(iLoopFade);
+			pSampl->setLoopRange(iLoopStart, iLoopEnd);
+			pSampl->setOffsetRange(iOffsetStart, iOffsetEnd);
+		}
+	}
+}
+
+
+void samplv1_param::saveSamples (
+	samplv1 *pSampl, QDomDocument& doc, QDomElement& eSamples, bool bSymLink )
+{
+	if (pSampl == nullptr)
+		return;
+
+	const char *pszSampleFile = pSampl->sampleFile();
+	if (pszSampleFile == nullptr)
+		return;
+
+	QDomElement eSample = doc.createElement("sample");
+	eSample.setAttribute("index", 0);
+	eSample.setAttribute("name", "GEN1_SAMPLE");
+
+	QDomElement eFilename = doc.createElement("filename");
+	eFilename.appendChild(doc.createTextNode(
+		QDir::current().relativeFilePath(
+			samplv1_param::saveFilename(
+				QString::fromUtf8(pszSampleFile), bSymLink))));
+	eSample.appendChild(eFilename);
+
+	const uint32_t iOffsetStart = pSampl->offsetStart();
+	const uint32_t iOffsetEnd   = pSampl->offsetEnd();
+	if (iOffsetStart < iOffsetEnd) {
+		QDomElement eOffsetStart = doc.createElement("offset-start");
+		eOffsetStart.appendChild(doc.createTextNode(
+			QString::number(iOffsetStart)));
+		eSample.appendChild(eOffsetStart);
+		QDomElement eOffsetEnd = doc.createElement("offset-end");
+		eOffsetEnd.appendChild(doc.createTextNode(
+			QString::number(iOffsetEnd)));
+		eSample.appendChild(eOffsetEnd);
+	}
+
+	const uint32_t iLoopStart = pSampl->loopStart();
+	const uint32_t iLoopEnd   = pSampl->loopEnd();
+	const uint32_t iLoopFade  = pSampl->loopFade();
+	const bool     bLoopZero  = pSampl->isLoopZero();
+	if (iLoopStart < iLoopEnd) {
+		QDomElement eLoopStart = doc.createElement("loop-start");
+		eLoopStart.appendChild(doc.createTextNode(
+			QString::number(iLoopStart)));
+		eSample.appendChild(eLoopStart);
+		QDomElement eLoopEnd = doc.createElement("loop-end");
+		eLoopEnd.appendChild(doc.createTextNode(
+			QString::number(iLoopEnd)));
+		eSample.appendChild(eLoopEnd);
+		QDomElement eLoopFade = doc.createElement("loop-fade");
+		eLoopFade.appendChild(doc.createTextNode(
+			QString::number(iLoopFade)));
+		eSample.appendChild(eLoopFade);
+		QDomElement eLoopZero = doc.createElement("loop-zero");
+		eLoopZero.appendChild(doc.createTextNode(
+			QString::number(int(bLoopZero))));
+		eSample.appendChild(eLoopZero);
+	}
+
+	eSamples.appendChild(eSample);
+}
+
+
+// Tuning serialization methods.
+void samplv1_param::loadTuning (
+	samplv1 *pSampl, const QDomElement& eTuning )
+{
+	if (pSampl == nullptr)
+		return;
+
+	pSampl->setTuningEnabled(eTuning.attribute("enabled").toInt() > 0);
+
+	for (QDomNode nChild = eTuning.firstChild();
+			!nChild.isNull();
+				nChild = nChild.nextSibling()) {
+		QDomElement eChild = nChild.toElement();
+		if (eChild.isNull())
+			continue;
+		if (eChild.tagName() == "enabled") {
+			pSampl->setTuningEnabled(eChild.text().toInt() > 0);
+		}
+		if (eChild.tagName() == "ref-pitch") {
+			pSampl->setTuningRefPitch(eChild.text().toFloat());
+		}
+		else
+		if (eChild.tagName() == "ref-note") {
+			pSampl->setTuningRefNote(eChild.text().toInt());
+		}
+		else
+		if (eChild.tagName() == "scale-file") {
+			const QString& sScaleFile
+				= eChild.text();
+			const QByteArray aScaleFile
+				= samplv1_param::loadFilename(sScaleFile).toUtf8();
+			pSampl->setTuningScaleFile(aScaleFile.constData());
+		}
+		else
+		if (eChild.tagName() == "keymap-file") {
+			const QString& sKeyMapFile
+				= eChild.text();
+			const QByteArray aKeyMapFile
+				= samplv1_param::loadFilename(sKeyMapFile).toUtf8();
+			pSampl->setTuningScaleFile(aKeyMapFile.constData());
+		}
+	}
+
+	// Consolidate tuning state...
+	pSampl->resetTuning();
+}
+
+
+void samplv1_param::saveTuning (
+	samplv1 *pSampl, QDomDocument& doc, QDomElement& eTuning, bool bSymLink )
+{
+	if (pSampl == nullptr)
+		return;
+
+	eTuning.setAttribute("enabled", int(pSampl->isTuningEnabled()));
+
+	QDomElement eRefPitch = doc.createElement("ref-pitch");
+	eRefPitch.appendChild(doc.createTextNode(
+		QString::number(pSampl->tuningRefPitch())));
+	eTuning.appendChild(eRefPitch);
+
+	QDomElement eRefNote = doc.createElement("ref-note");
+	eRefNote.appendChild(doc.createTextNode(
+		QString::number(pSampl->tuningRefNote())));
+	eTuning.appendChild(eRefNote);
+
+	const char *pszScaleFile = pSampl->tuningScaleFile();
+	if (pszScaleFile) {
+		const QString& sScaleFile
+			= QString::fromUtf8(pszScaleFile);
+		if (!sScaleFile.isEmpty()) {
+			QDomElement eScaleFile = doc.createElement("scale-file");
+			eScaleFile.appendChild(doc.createTextNode(
+				QDir::current().relativeFilePath(
+					samplv1_param::saveFilename(sScaleFile, bSymLink))));
+			eTuning.appendChild(eScaleFile);
+		}
+	}
+
+	const char *pszKeyMapFile = pSampl->tuningKeyMapFile();
+	if (pszKeyMapFile) {
+		const QString& sKeyMapFile
+			= QString::fromUtf8(pszKeyMapFile);
+		if (!sKeyMapFile.isEmpty()) {
+			QDomElement eKeyMapFile = doc.createElement("keymap-file");
+			eKeyMapFile.appendChild(doc.createTextNode(
+				QDir::current().relativeFilePath(
+					samplv1_param::saveFilename(sKeyMapFile, bSymLink))));
+			eTuning.appendChild(eKeyMapFile);
+		}
+	}
+}
+
+
+// Load/save and convert canonical/absolute filename helpers.
+QString samplv1_param::loadFilename ( const QString& sFilename )
+{
+	QFileInfo fi(sFilename);
+	if (fi.isSymLink())
+		fi.setFile(fi.symLinkTarget());
+	return fi.filePath();
+}
+
+
+QString samplv1_param::saveFilename ( const QString& sFilename, bool bSymLink )
+{
+	QFileInfo fi(sFilename);
+	if (bSymLink && fi.absolutePath() != QDir::current().absolutePath()) {
+		const QString& sPath = fi.absoluteFilePath();
+		const QString& sName = fi.baseName();
+		const QString& sExt  = fi.completeSuffix();
+		const QString& sLink = sName
+			+ '-' + QString::number(qHash(sPath), 16)
+			+ '.' + sExt;
+		QFile(sPath).link(sLink);
+		fi.setFile(QDir::current(), sLink);
+	}
+	else if (fi.isSymLink()) fi.setFile(fi.symLinkTarget());
+	return fi.absoluteFilePath();
+}
+
+
 // end of samplv1_param.cpp
