@@ -1,7 +1,7 @@
 // samplv1_sample.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -26,44 +26,21 @@
 #include <sndfile.h>
 
 
-//-------------------------------------------------------------------------
-// samplv1_reverse_sched - local module schedule thread stuff.
-//
-
-#include "samplv1_sched.h"
-
-
-class samplv1_reverse_sched : public samplv1_sched
-{
-public:
-
-	// ctor.
-	samplv1_reverse_sched (samplv1 *pSampl, samplv1_sample *sample)
-		: samplv1_sched(pSampl, Sample), m_sample(sample) {}
-
-	// process reverse (virtual).
-	void process(int)
-		{ m_sample->reverse_sync(); }
-
-private:
-
-	// instance variables.
-	samplv1_sample *m_sample;
-};
-
-
 //-------------------------------------------------------------------------
 // samplv1_sample - sampler wave table.
 //
 
 // ctor.
-samplv1_sample::samplv1_sample ( samplv1 *pSampl, float srate )
-	: m_srate(srate), m_filename(NULL), m_nchannels(0),
+samplv1_sample::samplv1_sample ( float srate )
+	: m_srate(srate), m_filename(nullptr), m_nchannels(0),
 		m_rate0(0.0f), m_freq0(1.0f), m_ratio(0.0f),
-		m_nframes(0), m_pframes(NULL), m_reverse(false),
-		m_loop(false), m_loop_start(0), m_loop_end(0)
+		m_nframes(0), m_pframes(nullptr), m_reverse(false),
+		m_offset(false), m_offset_start(0), m_offset_end(0),
+		m_offset_phase0(0.0f), m_offset_end2(0),
+		m_loop(false), m_loop_start(0), m_loop_end(0),
+		m_loop_phase1(0.0f), m_loop_phase2(0.0f),
+		m_loop_xfade(0), m_loop_xzero(true)
 {
-	m_reverse_sched = new samplv1_reverse_sched(pSampl, this);
 }
 
 
@@ -71,15 +48,13 @@ samplv1_sample::samplv1_sample ( samplv1 *pSampl, float srate )
 samplv1_sample::~samplv1_sample (void)
 {
 	close();
-
-	delete m_reverse_sched;
 }
 
 
 // init.
 bool samplv1_sample::open ( const char *filename, float freq0 )
 {
-	if (filename == NULL)
+	if (filename == nullptr)
 		return false;
 
 	close();
@@ -90,7 +65,7 @@ bool samplv1_sample::open ( const char *filename, float freq0 )
 	::memset(&info, 0, sizeof(info));
 
 	SNDFILE *file = ::sf_open(m_filename, SFM_READ, &info);
-	if (file == NULL)
+	if (file == nullptr)
 		return false;
 
 	m_nchannels = info.channels;
@@ -149,7 +124,8 @@ bool samplv1_sample::open ( const char *filename, float freq0 )
 
 	reset(freq0);
 
-	setLoop(m_loop);
+	updateOffset();
+	updateLoop();
 	return true;
 }
 
@@ -160,7 +136,7 @@ void samplv1_sample::close (void)
 		for (uint16_t k = 0; k < m_nchannels; ++k)
 			delete [] m_pframes[k];
 		delete [] m_pframes;
-		m_pframes = NULL;
+		m_pframes = nullptr;
 	}
 
 	m_nframes   = 0;
@@ -169,19 +145,13 @@ void samplv1_sample::close (void)
 	m_rate0     = 0.0f;
 	m_nchannels = 0;
 
+	setOffsetRange(0, 0);
+	setLoopRange(0, 0);
+
 	if (m_filename) {
 		::free(m_filename);
-		m_filename = NULL;
+		m_filename = nullptr;
 	}
-
-	setLoopRange(0, 0);
-}
-
-
-// schedule sample reverse.
-void samplv1_sample::reverse_sched (void)
-{
-	m_reverse_sched->schedule();
 }
 
 
@@ -204,4 +174,142 @@ void samplv1_sample::reverse_sync (void)
 }
 
 
+// offset range.
+void samplv1_sample::setOffsetRange ( uint32_t start, uint32_t end )
+{
+	if (start > m_nframes)
+		start = m_nframes;
+
+	if (end > m_nframes || start >= end)
+		end = m_nframes;
+
+	if (start < end) {
+		m_offset_start = start;
+		m_offset_end = end;
+	} else {
+		m_offset_start = 0;
+		m_offset_end = m_nframes;
+	}
+
+	if (m_offset && m_offset_start < m_offset_end) {
+		m_offset_phase0 = float(zero_crossing(m_offset_start, nullptr));
+		m_offset_end2 = zero_crossing(m_offset_end, nullptr);
+	} else {
+		m_offset_phase0 = 0.0f;
+		m_offset_end2 = m_nframes;
+	}
+
+	// offset/loop range stabilizer...
+	int loop_update = 0;
+
+	uint32_t loop_start = m_loop_start;
+	uint32_t loop_end = m_loop_end;
+
+	if (loop_start < m_offset_start && m_offset_start < m_offset_end) {
+		loop_start = m_offset_start;
+		++loop_update;
+	}
+
+	if (loop_end > m_offset_end && m_offset_start < m_offset_end) {
+		loop_end = m_offset_end;
+		++loop_update;
+	}
+
+	if (loop_update > 0 && loop_start < loop_end)
+		setLoopRange(loop_start, loop_end);
+}
+
+
+// offset updater.
+void samplv1_sample::updateOffset (void)
+{
+	setOffsetRange(m_offset_start, m_offset_end);
+}
+
+
+// loop range.
+void samplv1_sample::setLoopRange ( uint32_t start, uint32_t end )
+{
+	if (m_offset_start < m_offset_end) {
+		if (start < m_offset_start)
+			start = m_offset_start;
+		if (start > m_offset_end)
+			start = m_offset_end;
+		if (end > m_offset_end)
+			end = m_offset_end;
+		if (end < m_offset_start)
+			end = m_offset_start;
+	} else {
+		if (start > m_nframes)
+			start = m_nframes;
+		if (end > m_nframes)
+			end = m_nframes;
+	}
+
+	if (start < end) {
+		m_loop_start = start;
+		m_loop_end = end;
+	} else {
+		m_loop_start = 0;
+		m_loop_end = m_nframes;
+	}
+
+	if (m_loop && m_loop_start < m_loop_end) {
+		uint32_t start = m_loop_start;
+		uint32_t end = m_loop_end;
+		if (m_loop_xzero) {
+			int slope = 0;
+			end = zero_crossing(m_loop_end, &slope);
+			start = zero_crossing(m_loop_start, &slope);
+			if (start >= end) {
+				start = m_loop_start;
+				end = m_loop_end;
+			}
+		}
+		m_loop_phase1 = float(end - start);
+		m_loop_phase2 = float(end);
+	} else {
+		m_loop_phase1 = m_loop_phase2 = 0.0f;
+	}
+}
+
+
+// loop updater.
+void samplv1_sample::updateLoop (void)
+{
+	setLoopRange(m_loop_start, m_loop_end);
+}
+
+
+// zero-crossing aliasing (all channels).
+uint32_t samplv1_sample::zero_crossing ( uint32_t i, int *slope ) const
+{
+	const int s0 = (slope ? *slope : 0);
+
+	if (i > 0) --i;
+	float v0 = zero_crossing_k(i);
+	for (++i; i < m_nframes; ++i) {
+		const float v1 = zero_crossing_k(i);
+		if ((0 >= s0 && v0 >= 0.0f && 0.0f >= v1) ||
+			(s0 >= 0 && v1 >= 0.0f && 0.0f >= v0)) {
+			if (slope && s0 == 0) *slope = (v1 < v0 ? -1 : +1);
+			return i;
+		}
+		v0 = v1;
+	}
+
+	return m_nframes;
+}
+
+
+// zero-crossing aliasing (median).
+float samplv1_sample::zero_crossing_k ( uint32_t i ) const
+{
+	float sum = 0.0f;
+	for (uint16_t k = 0; k < m_nchannels; ++k)
+		sum += m_pframes[k][i];
+	return (sum / float(m_nchannels));
+}
+
+
 // end of samplv1_sample.cpp
