@@ -1,7 +1,7 @@
 // samplv1_sample.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -27,10 +27,11 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include <math.h>
+
 
 // forward decls.
 class samplv1;
-class samplv1_reverse_sched;
 
 
 //-------------------------------------------------------------------------
@@ -42,7 +43,7 @@ class samplv1_sample
 public:
 
 	// ctor.
-	samplv1_sample(samplv1 *pSampl, float srate = 44100.0f);
+	samplv1_sample(float srate = 44100.0f);
 
 	// dtor.
 	~samplv1_sample();
@@ -55,64 +56,75 @@ public:
 
 	// reverse mode.
 	void setReverse(bool reverse)
-		{ reverse_test(reverse); }
-
-	bool isReverse() const
-		{ return m_reverse; }
-
-	// schedule sample reverse.
-	void reverse_test(bool reverse)
 	{
 		if (( m_reverse && !reverse) ||
 			(!m_reverse &&  reverse)) {
 			m_reverse = reverse;
-			reverse_sched();
+			reverse_sync();
 		}
 	}
 
-	// reverse sample buffer.
-	void reverse_sched();
-	void reverse_sync();
+	bool isReverse() const
+		{ return m_reverse; }
+
+	// offset mode.
+	void setOffset(bool offset)
+	{
+		m_offset = offset;
+
+		updateOffset();
+	}
+
+	bool isOffset() const
+		{ return m_offset; }
+
+	// offset range.
+	void setOffsetRange(uint32_t start, uint32_t end);
+
+	uint32_t offsetStart() const
+		{ return m_offset_start; }
+	uint32_t offsetEnd() const
+		{ return m_offset_end; }
+
+	float offsetPhase0() const
+		{ return (m_offset ? m_offset_phase0 : 0.0f); }
 
 	// loop mode.
 	void setLoop(bool loop)
 	{
 		m_loop = loop;
 
-		if (m_loop && m_loop_start >= m_loop_end) {
-			m_loop_start = 0;
-			m_loop_end = m_nframes;
-			m_loop_phase1 = m_loop_phase2 = float(m_nframes);
-		}
+		updateLoop();
 	}
 
 	bool isLoop() const
-		{ return m_loop && (m_loop_start < m_loop_end); }
-
-	// loop change.
-	bool loop_test(bool loop)
-	{
-		if ((m_loop && !loop) || (!m_loop && loop)) {
-			setLoop(loop);
-			return true;
-		} else {
-			return false;
-		}
-	}
+		{ return m_loop; }
 
 	// loop range.
 	void setLoopRange(uint32_t start, uint32_t end);
 
-	uint32_t loopStart (void) const
+	uint32_t loopStart() const
 		{ return m_loop_start; }
-	uint32_t loopEnd (void) const
+	uint32_t loopEnd() const
 		{ return m_loop_end; }
 
-	float loopPhase1 (void) const
+	float loopPhase1() const
 		{ return m_loop_phase1; }
-	float loopPhase2 (void) const
+	float loopPhase2() const
 		{ return m_loop_phase2; }
 
+	// loop cross-fade (in number of frames)
+	void setLoopCrossFade(uint32_t xfade)
+		{ m_loop_xfade = xfade; }
+	uint32_t loopCrossFade() const
+		{ return m_loop_xfade; }
+
+	// loop zero-crossing detection
+	void setLoopZeroCrossing(bool xzero)
+		{ m_loop_xzero = xzero; }
+	bool isLoopZeroCrossing() const
+		{ return m_loop_xzero; }
+
 	// init.
 	bool open(const char *filename, float freq0 = 1.0f);
 	void close();
@@ -145,14 +157,21 @@ public:
 		{ return m_pframes[k]; }
 
 	// predicate.
-	bool isOver(uint32_t frame) const
-		{ return !m_pframes || (frame >= m_nframes); }
+	bool isOver(uint32_t index) const
+		{ return !m_pframes || (index >= m_offset_end2); }
 
 protected:
 
+	// reverse sample buffer.
+	void reverse_sync();
+
 	// zero-crossing aliasing .
-	uint32_t zero_crossing_k(uint32_t i, uint16_t k, int *slope) const;
 	uint32_t zero_crossing(uint32_t i, int *slope) const;
+	float zero_crossing_k(uint32_t i) const;
+
+	// offset/loop update.
+	void updateOffset();
+	void updateLoop();
 
 private:
 
@@ -167,13 +186,19 @@ private:
 	float  **m_pframes;
 	bool     m_reverse;
 
+	bool     m_offset;
+	uint32_t m_offset_start;
+	uint32_t m_offset_end;
+	float    m_offset_phase0;
+	uint32_t m_offset_end2;
+
 	bool     m_loop;
 	uint32_t m_loop_start;
 	uint32_t m_loop_end;
 	float    m_loop_phase1;
 	float    m_loop_phase2;
-
-	samplv1_reverse_sched *m_reverse_sched;
+	uint32_t m_loop_xfade;
+	bool     m_loop_xzero;
 };
 
 
@@ -185,7 +210,7 @@ class samplv1_generator
 public:
 
 	// ctor.
-	samplv1_generator(samplv1_sample *sample = NULL) { reset(sample); }
+	samplv1_generator(samplv1_sample *sample = nullptr) { reset(sample); }
 
 	// sample accessor.
 	samplv1_sample *sample() const
@@ -196,14 +221,7 @@ public:
 	{
 		m_sample = sample;
 
-		m_phase = 0.0f;
-		m_index = 0;
-		m_alpha = 0.0f;
-		m_frame = 0;
-
-		m_loop = false;
-		m_loop_phase1 = 0.0f;
-		m_loop_phase2 = 0.0f;
+		start();
 	}
 
 	// reset loop.
@@ -211,7 +229,7 @@ public:
 	{
 		m_loop = loop;
 
-		if (m_loop) {
+		if (m_loop && m_sample) {
 			m_loop_phase1 = m_sample->loopPhase1();
 			m_loop_phase2 = m_sample->loopPhase2();
 		} else {
@@ -221,33 +239,70 @@ public:
 	}
 
 	// begin.
-	void start(void)
+	void start()
 	{
-		m_phase = 0.0f;
-		m_index = 0;
-		m_alpha = 0.0f;
-		m_frame = 0;
+		m_phase0 = (m_sample ? m_sample->offsetPhase0() : 0.0f);
+		m_phase  = m_phase0;
+		m_index  = 0;
+		m_alpha  = 0.0f;
 
-		setLoop(m_sample->isLoop());
+		m_phase1 = 0.0f;
+		m_index1 = 0;
+		m_alpha1 = 0.0f;
+		m_xgain1 = 1.0f;
+
+		setLoop(m_sample ? m_sample->isLoop() : false);
 	}
 
 	// iterate.
 	void next(float freq)
 	{
-		const float delta = freq * m_sample->ratio();
+		const float delta = freq * (m_sample ? m_sample->ratio() : 1.0f);
 
-		m_index  = int(m_phase);
+		m_index  = uint32_t(m_phase);
 		m_alpha  = m_phase - float(m_index);
 		m_phase += delta;
 
-		if (m_loop && m_phase >= m_loop_phase2) {
-			m_phase -= m_loop_phase1;
-			if (m_phase < 0.0f)
-				m_phase = 0.0f;
+		if (m_loop && m_sample) {
+			const uint32_t xfade = m_sample->loopCrossFade(); // nframes.
+			if (xfade > 0) {
+				const float xfade1 = float(xfade); // nframes.
+				if (m_phase >= m_loop_phase2 - xfade1) {
+					if (//m_sample->isOver(m_index) ||
+						m_phase >= m_loop_phase2) {
+						m_phase -= m_loop_phase1 * ::ceilf(delta / m_loop_phase1);
+						if (m_phase < m_phase0)
+							m_phase = m_phase0;
+					}
+					if (m_phase1 > 0.0f) {
+						m_index1 = int(m_phase1);
+						m_alpha1 = m_phase1 - float(m_index1);
+						m_phase1 += delta;
+						m_xgain1 -= delta / xfade1;
+						if (m_xgain1 < 0.0f)
+							m_xgain1 = 0.0f;
+					} else {
+						m_phase1 = m_phase - m_loop_phase1;
+						if (m_phase1 < m_phase0)
+							m_phase1 = m_phase0;
+						m_xgain1 = 1.0f;
+					}
+				}
+				else
+				if (m_phase1 > 0.0f) {
+					m_phase1 = 0.0f;
+					m_index1 = 0;
+					m_alpha1 = 0.0f;
+					m_xgain1 = 1.0f;
+				}
+			}
+			else
+			if (m_phase >= m_loop_phase2) {
+				m_phase -= m_loop_phase1 * ::ceilf(delta / m_loop_phase1);
+				if (m_phase < m_phase0)
+					m_phase = m_phase0;
+			}
 		}
-
-		if (m_frame < m_index)
-			m_frame = m_index;
 	}
 
 	// sample.
@@ -256,12 +311,28 @@ public:
 		if (isOver())
 			return 0.0f;
 
+		float ret = m_xgain1 * interp(k, m_index, m_alpha);
+		if (m_index1 > 0)
+			ret += (1.0f - m_xgain1) * interp(k, m_index1, m_alpha1);
+
+		return ret;
+	}
+
+	// predicate.
+	bool isOver() const
+		{ return !m_loop && (m_sample ? m_sample->isOver(m_index) : true); }
+
+protected:
+
+	// sample (cubic interpolate).
+	float interp(uint16_t k, uint32_t index, float alpha) const
+	{
 		const float *frames = m_sample->frames(k);
 
-		const float x0 = frames[m_index];
-		const float x1 = frames[m_index + 1];
-		const float x2 = frames[m_index + 2];
-		const float x3 = frames[m_index + 3];
+		const float x0 = frames[index];
+		const float x1 = frames[index + 1];
+		const float x2 = frames[index + 2];
+		const float x3 = frames[index + 3];
 
 		const float c1 = (x2 - x0) * 0.5f;
 		const float b1 = (x1 - x2);
@@ -269,26 +340,27 @@ public:
 		const float c3 = (x3 - x1) * 0.5f + b2 + b1;
 		const float c2 = (c3 + b2);
 
-		return (((c3 * m_alpha) - c2) * m_alpha + c1) * m_alpha + x1;
+		return (((c3 * alpha) - c2) * alpha + c1) * alpha + x1;
 	}
 
-	// predicate.
-	bool isOver() const
-		{ return !m_loop && m_sample->isOver(m_frame); }
-
 private:
 
 	// iterator variables.
 	samplv1_sample *m_sample;
 
+	float    m_phase0;
 	float    m_phase;
 	uint32_t m_index;
 	float    m_alpha;
-	uint32_t m_frame;
 
 	bool     m_loop;
 	float    m_loop_phase1;
 	float    m_loop_phase2;
+
+	float    m_phase1;
+	uint32_t m_index1;
+	float    m_alpha1;
+	float    m_xgain1;
 };
 
 
