@@ -1,7 +1,7 @@
 // samplv1_ui.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -49,6 +49,13 @@ public:
 	void setReverse(bool bReverse);
 	bool isReverse() const;
 
+	void setOffset(bool bOffset);
+	bool isOffset() const;
+
+	void setOffsetRange(uint32_t iOffsetStart, uint32_t iOffsetEnd);
+	uint32_t offsetStart() const;
+	uint32_t offsetEnd() const;
+
 	void setLoop(bool bLoop);
 	bool isLoop() const;
 
@@ -56,6 +63,12 @@ public:
 	uint32_t loopStart() const;
 	uint32_t loopEnd() const;
 
+	void setLoopFade(uint32_t iLoopFade);
+	uint32_t loopFade() const;
+
+	void setLoopZero(bool bLoopZero);
+	bool isLoopZero() const;
+
 	bool loadPreset(const QString& sFilename);
 	bool savePreset(const QString& sFilename);
 
@@ -74,6 +87,26 @@ public:
 
 	void directNoteOn(int note, int vel);
 
+	void setTuningEnabled(bool enabled);
+	bool isTuningEnabled() const;
+
+	void setTuningRefPitch(float refPitch);
+	float tuningRefPitch() const;
+
+	void setTuningRefNote(int refNote);
+	int tuningRefNote() const;
+
+	void setTuningScaleFile(const char *pszScaleFile);
+	const char *tuningScaleFile() const;
+
+	void setTuningKeyMapFile(const char *pszKeyMapFile);
+	const char *tuningKeyMapFile() const;
+
+	void resetTuning();
+
+	// MIDI note/octave name helper.
+	static QString noteName(int note);
+
 private:
 
 	samplv1 *m_pSampl;
