@@ -1,7 +1,7 @@
 // samplv1widget.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -28,6 +28,8 @@
 #include "samplv1widget_config.h"
 #include "samplv1widget_control.h"
 
+#include "samplv1widget_keybd.h"
+
 #include "samplv1_controls.h"
 #include "samplv1_programs.h"
 
@@ -40,6 +42,10 @@
 #include <QShowEvent>
 #include <QHideEvent>
 
+#include <math.h>
+
+#include <random>
+
 
 //-------------------------------------------------------------------------
 // samplv1widget - impl.
@@ -51,7 +57,7 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 {
 	Q_INIT_RESOURCE(samplv1);
 
-#if QT_VERSION >= 0x050000
+#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
 	// HACK: Dark themes grayed/disabled color group fix...
 	QPalette pal;
 	if (pal.base().color().value() < 0x7f) {
@@ -73,12 +79,15 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	m_ui.setupUi(this);
 
 	// Init sched notifier.
-	m_sched_notifier = NULL;
+	m_sched_notifier = nullptr;
 
 	// Init swapable params A/B to default.
 	for (uint32_t i = 0; i < samplv1::NUM_PARAMS; ++i)
 		m_params_ab[i] = samplv1_param::paramDefaultValue(samplv1::ParamIndex(i));
 
+	// Some property temporary holder.
+	m_iLoopFade = 1;
+
 	// Start clean.
 	m_iUpdate = 0;
 
@@ -86,26 +95,40 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	for (int iTab = 0; iTab < m_ui.StackedWidget->count(); ++iTab)
 		m_ui.TabBar->addTab(m_ui.StackedWidget->widget(iTab)->windowTitle());
 
-	// Loop range font.
-	const QFont& font = m_ui.Gen1LoopKnob->font();
+	// Offset/Loop range font.
+	const QFont& font = m_ui.Gen1ReverseKnob->font();
+	m_ui.Gen1OffsetRangeLabel->setFont(font);
+	m_ui.Gen1OffsetStartSpinBox->setFont(font);
+	m_ui.Gen1OffsetEndSpinBox->setFont(font);
 	m_ui.Gen1LoopRangeLabel->setFont(font);
 	m_ui.Gen1LoopStartSpinBox->setFont(font);
 	m_ui.Gen1LoopEndSpinBox->setFont(font);
+	m_ui.Gen1LoopFadeCheckBox->setFont(font);
+	m_ui.Gen1LoopFadeSpinBox->setFont(font);
 
 	const QFontMetrics fm(font);
+	m_ui.Gen1OffsetStartSpinBox->setMaximumHeight(fm.height() + 6);
+	m_ui.Gen1OffsetEndSpinBox->setMaximumHeight(fm.height() + 6);
 	m_ui.Gen1LoopStartSpinBox->setMaximumHeight(fm.height() + 6);
 	m_ui.Gen1LoopEndSpinBox->setMaximumHeight(fm.height() + 6);
+	m_ui.Gen1LoopFadeSpinBox->setMaximumHeight(fm.height() + 6);
 
+	m_ui.Gen1OffsetStartSpinBox->setAccelerated(true);
+	m_ui.Gen1OffsetEndSpinBox->setAccelerated(true);
 	m_ui.Gen1LoopStartSpinBox->setAccelerated(true);
 	m_ui.Gen1LoopEndSpinBox->setAccelerated(true);
+	m_ui.Gen1LoopFadeSpinBox->setAccelerated(true);
 
+	m_ui.Gen1OffsetStartSpinBox->setMinimum(0);
+	m_ui.Gen1OffsetEndSpinBox->setMinimum(0);
 	m_ui.Gen1LoopStartSpinBox->setMinimum(0);
 	m_ui.Gen1LoopEndSpinBox->setMinimum(0);
+	m_ui.Gen1LoopFadeSpinBox->setMinimum(0);
 
 	// Note names.
 	QStringList notes;
 	for (int note = 0; note < 128; ++note)
-		notes << noteName(note);
+		notes << samplv1_ui::noteName(note).remove(QRegExp("/\\S+"));
 
 	m_ui.Gen1SampleKnob->setScale(1000.0f);
 	m_ui.Gen1SampleKnob->insertItems(0, notes);
@@ -151,6 +174,7 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	states << tr("On");
 #if 0
 	m_ui.Gen1ReverseKnob->insertItems(0, states);
+	m_ui.Gen1OffsetKnob->insertItems(0, states);
 	m_ui.Gen1LoopKnob->insertItems(0, states);
 
 	m_ui.Lfo1SyncKnob->insertItems(0, states);
@@ -248,6 +272,7 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	// GEN1
 	setParamKnob(samplv1::GEN1_SAMPLE,  m_ui.Gen1SampleKnob);
 	setParamKnob(samplv1::GEN1_REVERSE, m_ui.Gen1ReverseKnob);
+	setParamKnob(samplv1::GEN1_OFFSET,  m_ui.Gen1OffsetKnob);
 	setParamKnob(samplv1::GEN1_LOOP,    m_ui.Gen1LoopKnob);
 	setParamKnob(samplv1::GEN1_OCTAVE,  m_ui.Gen1OctaveKnob);
 	setParamKnob(samplv1::GEN1_TUNING,  m_ui.Gen1TuningKnob);
@@ -255,6 +280,7 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	setParamKnob(samplv1::GEN1_ENVTIME, m_ui.Gen1EnvTimeKnob);
 
 	// DCF1
+	setParamKnob(samplv1::DCF1_ENABLED,  m_ui.Dcf1GroupBox->param());
 	setParamKnob(samplv1::DCF1_CUTOFF,   m_ui.Dcf1CutoffKnob);
 	setParamKnob(samplv1::DCF1_RESO,     m_ui.Dcf1ResoKnob);
 	setParamKnob(samplv1::DCF1_TYPE,     m_ui.Dcf1TypeKnob);
@@ -315,6 +341,7 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 		m_ui.Dcf1Env, SLOT(setRelease(float)));
 
 	// LFO1
+	setParamKnob(samplv1::LFO1_ENABLED, m_ui.Lfo1GroupBox->param());
 	setParamKnob(samplv1::LFO1_SHAPE,   m_ui.Lfo1ShapeKnob);
 	setParamKnob(samplv1::LFO1_WIDTH,   m_ui.Lfo1WidthKnob);
 	setParamKnob(samplv1::LFO1_BPM,     m_ui.Lfo1BpmKnob);
@@ -373,6 +400,7 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 		m_ui.Lfo1Env, SLOT(setRelease(float)));
 
 	// DCA1
+	setParamKnob(samplv1::DCA1_ENABLED, m_ui.Dca1GroupBox->param());
 	setParamKnob(samplv1::DCA1_VOLUME,  m_ui.Dca1VolumeKnob);
 	setParamKnob(samplv1::DCA1_ATTACK,  m_ui.Dca1AttackKnob);
 	setParamKnob(samplv1::DCA1_DECAY,   m_ui.Dca1DecayKnob);
@@ -456,6 +484,8 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	setParamKnob(samplv1::DYN1_COMPRESS, m_ui.Dyn1CompressKnob);
 	setParamKnob(samplv1::DYN1_LIMITER,  m_ui.Dyn1LimiterKnob);
 
+	// Make status-bar keyboard range active.
+	m_ui.StatusBar->keybd()->setNoteRange(true);
 
 	// Sample management...
 	QObject::connect(m_ui.Gen1Sample,
@@ -476,23 +506,63 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 		SIGNAL(resetPresetFile()),
 		SLOT(resetParams()));
 
-	// Sample context menu...
+	// Common context menu policies...
 	m_ui.Gen1Sample->setContextMenuPolicy(Qt::CustomContextMenu);
+	m_ui.Gen1OffsetStartSpinBox->setContextMenuPolicy(Qt::CustomContextMenu);
+	m_ui.Gen1OffsetEndSpinBox->setContextMenuPolicy(Qt::CustomContextMenu);
+	m_ui.Gen1LoopStartSpinBox->setContextMenuPolicy(Qt::CustomContextMenu);
+	m_ui.Gen1LoopEndSpinBox->setContextMenuPolicy(Qt::CustomContextMenu);
 
 	QObject::connect(m_ui.Gen1Sample,
 		SIGNAL(customContextMenuRequested(const QPoint&)),
 		SLOT(contextMenuRequest(const QPoint&)));
 
+	QObject::connect(m_ui.Gen1Sample,
+		SIGNAL(offsetRangeChanged()),
+		SLOT(offsetRangeChanged()));
 	QObject::connect(m_ui.Gen1Sample,
 		SIGNAL(loopRangeChanged()),
 		SLOT(loopRangeChanged()));
 
+	QObject::connect(m_ui.Gen1OffsetStartSpinBox,
+		SIGNAL(valueChanged(uint32_t)),
+		SLOT(offsetStartChanged()));
+	QObject::connect(m_ui.Gen1OffsetEndSpinBox,
+		SIGNAL(valueChanged(uint32_t)),
+		SLOT(offsetEndChanged()));
 	QObject::connect(m_ui.Gen1LoopStartSpinBox,
-		SIGNAL(valueChanged(int)),
+		SIGNAL(valueChanged(uint32_t)),
 		SLOT(loopStartChanged()));
 	QObject::connect(m_ui.Gen1LoopEndSpinBox,
-		SIGNAL(valueChanged(int)),
+		SIGNAL(valueChanged(uint32_t)),
 		SLOT(loopEndChanged()));
+	QObject::connect(m_ui.Gen1LoopFadeCheckBox,
+		SIGNAL(valueChanged(float)),
+		SLOT(loopFadeChanged()));
+	QObject::connect(m_ui.Gen1LoopFadeSpinBox,
+		SIGNAL(valueChanged(int)),
+		SLOT(loopFadeChanged()));
+	QObject::connect(m_ui.Gen1LoopZeroCheckBox,
+		SIGNAL(valueChanged(float)),
+		SLOT(loopZeroChanged()));
+
+	QObject::connect(m_ui.Gen1OffsetStartSpinBox,
+		SIGNAL(customContextMenuRequested(const QPoint&)),
+		SLOT(spinboxContextMenu(const QPoint&)));
+	QObject::connect(m_ui.Gen1OffsetEndSpinBox,
+		SIGNAL(customContextMenuRequested(const QPoint&)),
+		SLOT(spinboxContextMenu(const QPoint&)));
+	QObject::connect(m_ui.Gen1LoopStartSpinBox,
+		SIGNAL(customContextMenuRequested(const QPoint&)),
+		SLOT(spinboxContextMenu(const QPoint&)));
+	QObject::connect(m_ui.Gen1LoopEndSpinBox,
+		SIGNAL(customContextMenuRequested(const QPoint&)),
+		SLOT(spinboxContextMenu(const QPoint&)));
+
+	// Randomize params...
+	QObject::connect(m_ui.RandomParamsButton,
+		SIGNAL(clicked()),
+		SLOT(randomParams()));
 
 	// Swap params A/B
 	QObject::connect(m_ui.SwapParamsAButton,
@@ -506,6 +576,14 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	QObject::connect(m_ui.TabBar, SIGNAL(currentChanged(int)),
 		m_ui.StackedWidget, SLOT(setCurrentIndex(int)));
 
+	// Direct status-bar keyboard input
+	QObject::connect(m_ui.StatusBar->keybd(),
+		SIGNAL(noteOnClicked(int, int)),
+		SLOT(directNoteOn(int, int)));
+	QObject::connect(m_ui.StatusBar->keybd(),
+		SIGNAL(noteRangeChanged()),
+		SLOT(noteRangeChanged()));
+
 	// Menu actions
 	QObject::connect(m_ui.helpConfigureAction,
 		SIGNAL(triggered(bool)),
@@ -517,13 +595,19 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 		SIGNAL(triggered(bool)),
 		SLOT(helpAboutQt()));
 
-	// General knob/dial  behavior init...
+	// General knob/dial behavior init...
 	samplv1_config *pConfig = samplv1_config::getInstance();
 	if (pConfig) {
 		samplv1widget_dial::setDialMode(
 			samplv1widget_dial::DialMode(pConfig->iKnobDialMode));
 		samplv1widget_edit::setEditMode(
 			samplv1widget_edit::EditMode(pConfig->iKnobEditMode));
+		const samplv1widget_spinbox::Format format
+			= samplv1widget_spinbox::Format(pConfig->iFrameTimeFormat);
+		m_ui.Gen1OffsetStartSpinBox->setFormat(format);
+		m_ui.Gen1OffsetEndSpinBox->setFormat(format);
+		m_ui.Gen1LoopStartSpinBox->setFormat(format);
+		m_ui.Gen1LoopEndSpinBox->setFormat(format);
 	}
 
 	// Epilog.
@@ -552,7 +636,7 @@ void samplv1widget::openSchedNotifier (void)
 		return;
 
 	samplv1_ui *pSamplUi = ui_instance();
-	if (pSamplUi == NULL)
+	if (pSamplUi == nullptr)
 		return;
 
 	m_sched_notifier = new samplv1widget_sched(pSamplUi->instance(), this);
@@ -569,7 +653,7 @@ void samplv1widget::closeSchedNotifier (void)
 {
 	if (m_sched_notifier) {
 		delete m_sched_notifier;
-		m_sched_notifier = NULL;
+		m_sched_notifier = nullptr;
 	}
 
 	samplv1_ui *pSamplUi = ui_instance();
@@ -616,21 +700,21 @@ void samplv1widget::setParamKnob ( samplv1::ParamIndex index, samplv1widget_para
 
 samplv1widget_param *samplv1widget::paramKnob ( samplv1::ParamIndex index ) const
 {
-	return m_paramKnobs.value(index, NULL);
+	return m_paramKnobs.value(index, nullptr);
 }
 
 
 // Param port accessors.
 void samplv1widget::setParamValue (
-	samplv1::ParamIndex index, float fValue, bool bDefault )
+	samplv1::ParamIndex index, float fValue, bool bIter )
 {
 	++m_iUpdate;
 
 	samplv1widget_param *pParam = paramKnob(index);
 	if (pParam)
-		pParam->setValue(fValue, bDefault);
+		pParam->setValue(fValue);
 
-	updateParamEx(index, fValue);
+	updateParamEx(index, fValue, bIter);
 
 	--m_iUpdate;
 }
@@ -672,30 +756,61 @@ void samplv1widget::paramChanged ( float fValue )
 
 
 // Update local tied widgets.
-void samplv1widget::updateParamEx ( samplv1::ParamIndex index, float fValue )
+void samplv1widget::updateParamEx (
+	samplv1::ParamIndex index, float fValue, bool bIter )
 {
 	samplv1_ui *pSamplUi = ui_instance();
-	if (pSamplUi == NULL)
+	if (pSamplUi == nullptr)
 		return;
 
 	++m_iUpdate;
 
 	switch (index) {
-#if 1//--updateSchedNotify(samplv1_sched::Sample, 0);
-	case samplv1::GEN1_REVERSE: {
-		const bool bReverse = bool(fValue > 0.0f);
-		pSamplUi->setReverse(bReverse);
-		updateSample(pSamplUi->sample());
+	case samplv1::GEN1_SAMPLE:
+		m_ui.StatusBar->keybd()->setNoteKey(int(fValue));
+		break;
+	case samplv1::GEN1_REVERSE:
+		pSamplUi->setReverse(bool(fValue > 0.0f));
+		if (!bIter) updateSample(pSamplUi->sample());
+		break;
+	case samplv1::GEN1_OFFSET:
+		pSamplUi->setOffset(bool(fValue > 0.0f));
+		if (!bIter) updateOffsetLoop(pSamplUi->sample());
+		break;
+	case samplv1::GEN1_LOOP:
+		pSamplUi->setLoop(bool(fValue > 0.0f));
+		if (!bIter) updateOffsetLoop(pSamplUi->sample());
+		break;
+	case samplv1::DCF1_ENABLED:
+		if (m_ui.Lfo1GroupBox->isChecked()) {
+			const bool bDcf1Enabled = (fValue > 0.5f);
+			m_ui.Lfo1CutoffKnob->setEnabled(bDcf1Enabled);
+			m_ui.Lfo1ResoKnob->setEnabled(bDcf1Enabled);
+		}
+		break;
+	case samplv1::LFO1_ENABLED:
+		if (fValue > 0.5f) {
+			const bool bDcf1Enabled = m_ui.Dcf1GroupBox->isChecked();
+			m_ui.Lfo1CutoffKnob->setEnabled(bDcf1Enabled);
+			m_ui.Lfo1ResoKnob->setEnabled(bDcf1Enabled);
+		}
 		break;
-	}
-#endif
 	case samplv1::DCF1_SLOPE:
-		m_ui.Dcf1TypeKnob->setEnabled(int(fValue) != 3); // !Formant
+		if (m_ui.Dcf1GroupBox->isChecked())
+			m_ui.Dcf1TypeKnob->setEnabled(int(fValue) != 3); // !Formant
 		break;
-	case samplv1::GEN1_LOOP: {
-		const bool bLoop = bool(fValue > 0.0f);
-		pSamplUi->setLoop(bLoop);
-		updateSampleLoop(pSamplUi->sample());
+	case samplv1::LFO1_SHAPE:
+		m_ui.Lfo1Wave->setWaveShape(fValue);
+		break;
+	case samplv1::KEY1_LOW:
+		m_ui.StatusBar->keybd()->setNoteLow(int(fValue));
+		break;
+	case samplv1::KEY1_HIGH:
+		m_ui.StatusBar->keybd()->setNoteHigh(int(fValue));
+		break;
+	case samplv1::DEF1_VELOCITY: {
+		const int vel = int(79.375f * fValue + 47.625f) & 0x7f;
+		m_ui.StatusBar->keybd()->setVelocity(vel);
 		break;
 	}
 	default:
@@ -713,7 +828,7 @@ void samplv1widget::updateSchedParam ( samplv1::ParamIndex index, float fValue )
 
 	samplv1widget_param *pParam = paramKnob(index);
 	if (pParam) {
-		pParam->setValue(fValue, false);
+		pParam->setValue(fValue);
 		updateParam(index, fValue);
 		updateParamEx(index, fValue);
 		m_ui.StatusBar->showMessage(QString("%1: %2")
@@ -730,7 +845,7 @@ void samplv1widget::updateSchedParam ( samplv1::ParamIndex index, float fValue )
 void samplv1widget::resetParams (void)
 {
 	samplv1_ui *pSamplUi = ui_instance();
-	if (pSamplUi == NULL)
+	if (pSamplUi == nullptr)
 		return;
 
 	pSamplUi->reset();
@@ -743,27 +858,97 @@ void samplv1widget::resetParams (void)
 		samplv1widget_param *pParam = paramKnob(index);
 		if (pParam && pParam->isDefaultValue())
 			fValue = pParam->defaultValue();
-		setParamValue(index, fValue);
+		setParamValue(index, fValue, true);
 		updateParam(index, fValue);
-	//	updateParamEx(index, fValue);
 		m_params_ab[i] = fValue;
 	}
 
+	updateSample(pSamplUi->sample());
+
 	m_ui.StatusBar->showMessage(tr("Reset preset"), 5000);
 	updateDirtyPreset(false);
 }
 
 
+// Randomize params (partial).
+void samplv1widget::randomParams (void)
+{
+	samplv1_ui *pSamplUi = ui_instance();
+	if (pSamplUi == nullptr)
+		return;
+
+	float p = 1.0f;
+
+	samplv1_config *pConfig = samplv1_config::getInstance();
+	if (pConfig)
+		p = 0.01f * pConfig->fRandomizePercent;
+
+	if (QMessageBox::warning(this,
+		tr("Warning"),
+		tr("About to randomize current parameter values:\n\n"
+		"-/+ %1%.\n\n"
+		"Are you sure?").arg(100.0f * p),
+		QMessageBox::Ok | QMessageBox::Cancel) == QMessageBox::Cancel)
+		return;
+
+	std::default_random_engine re(::time(nullptr));
+
+	for (uint32_t i = 0; i < samplv1::NUM_PARAMS; ++i) {
+		const samplv1::ParamIndex index = samplv1::ParamIndex(i);
+		// Filter out some non-randomizable parameters!...
+		if (index == samplv1::GEN1_SAMPLE   ||
+		    index == samplv1::GEN1_OFFSET   ||
+		    index == samplv1::GEN1_OFFSET_1 ||
+		    index == samplv1::GEN1_OFFSET_2 ||
+		    index == samplv1::GEN1_LOOP     ||
+		    index == samplv1::GEN1_LOOP_1   ||
+		    index == samplv1::GEN1_LOOP_2   ||
+			index == samplv1::GEN1_OCTAVE   ||
+			index == samplv1::GEN1_TUNING   ||
+			index == samplv1::GEN1_ENVTIME  ||
+			index == samplv1::DCF1_ENABLED  ||
+			index == samplv1::LFO1_ENABLED  ||
+			index == samplv1::DCA1_ENABLED) 
+			continue;
+		if (index >= samplv1::OUT1_WIDTH)
+			break;
+		samplv1widget_param *pParam = paramKnob(index);
+		if (pParam) {
+			std::normal_distribution<float> nd;
+			const float q = p * (pParam->maximum() - pParam->minimum());
+			float fValue = pParam->value();
+			if (samplv1_param::paramFloat(index))
+				fValue += 0.5f * q * nd(re);
+			else
+				fValue = std::round(fValue + q * nd(re));
+			if (fValue < pParam->minimum())
+				fValue = pParam->minimum();
+			else
+			if (fValue > pParam->maximum())
+				fValue = pParam->maximum();
+			setParamValue(index, fValue);
+			updateParam(index, fValue);
+		}
+	}
+
+	m_ui.StatusBar->showMessage(tr("Randomize"), 5000);
+	updateDirtyPreset(true);
+}
+
+
 // Swap params A/B.
 void samplv1widget::swapParams ( bool bOn )
 {
 	if (m_iUpdate > 0 || !bOn)
 		return;
 
+	samplv1_ui *pSamplUi = ui_instance();
+	if (pSamplUi == nullptr)
+		return;
+
 #ifdef CONFIG_DEBUG
 	qDebug("samplv1widget::swapParams(%d)", int(bOn));
 #endif
-//	resetParamKnobs();
 
 	for (uint32_t i = 0; i < samplv1::NUM_PARAMS; ++i) {
 		const samplv1::ParamIndex index = samplv1::ParamIndex(i);
@@ -771,12 +956,14 @@ void samplv1widget::swapParams ( bool bOn )
 		if (pParam) {
 			const float fOldValue = pParam->value();
 			const float fNewValue = m_params_ab[i];
-			setParamValue(index, fNewValue);
+			setParamValue(index, fNewValue, true);
 			updateParam(index, fNewValue);
 			m_params_ab[i] = fOldValue;
 		}
 	}
 
+	updateSample(pSamplUi->sample());
+
 	const bool bSwapA = m_ui.SwapParamsAButton->isChecked();
 	m_ui.StatusBar->showMessage(tr("Swap %1").arg(bSwapA ? 'A' : 'B'), 5000);
 	updateDirtyPreset(true);
@@ -798,17 +985,18 @@ void samplv1widget::updateParamValues (void)
 	resetSwapParams();
 
 	samplv1_ui *pSamplUi = ui_instance();
+	if (pSamplUi == nullptr)
+		return;
 
 	for (uint32_t i = 0; i < samplv1::NUM_PARAMS; ++i) {
 		const samplv1::ParamIndex index = samplv1::ParamIndex(i);
-		const float fValue = (pSamplUi
-			? pSamplUi->paramValue(index)
-			: samplv1_param::paramDefaultValue(index));
+		const float fValue = pSamplUi->paramValue(index);
 		setParamValue(index, fValue, true);
 		updateParam(index, fValue);
-	//	updateParamEx(index, fValue);
 		m_params_ab[i] = fValue;
 	}
+
+	updateSample(pSamplUi->sample());
 }
 
 
@@ -817,14 +1005,19 @@ void samplv1widget::resetParamValues (void)
 {
 	resetSwapParams();
 
+	samplv1_ui *pSamplUi = ui_instance();
+	if (pSamplUi == nullptr)
+		return;
+
 	for (uint32_t i = 0; i < samplv1::NUM_PARAMS; ++i) {
 		const samplv1::ParamIndex index = samplv1::ParamIndex(i);
 		const float fValue = samplv1_param::paramDefaultValue(index);
 		setParamValue(index, fValue, true);
 		updateParam(index, fValue);
-	//	updateParamEx(index, fValue);
 		m_params_ab[i] = fValue;
 	}
+
+	updateSample(pSamplUi->sample());
 }
 
 
@@ -868,7 +1061,7 @@ void samplv1widget::newPreset (void)
 	m_ui.StatusBar->showMessage(tr("New preset"), 5000);
 	updateDirtyPreset(false);
 
-	m_ui.Gen1Sample->openSample();
+//	openSample();
 }
 
 
@@ -923,9 +1116,12 @@ void samplv1widget::clearSample (void)
 // Sample file loader slot.
 void samplv1widget::loadSample ( const QString& sFilename )
 {
-	loadSampleFile(QFileInfo(sFilename).canonicalFilePath());
+	const QFileInfo info(sFilename);
 
-	m_ui.StatusBar->showMessage(tr("Load sample: %1").arg(sFilename), 5000);
+	loadSampleFile(info.canonicalFilePath());
+
+	m_ui.StatusBar->showMessage(
+		tr("Load sample: %1").arg(info.fileName()), 5000);
 	updateDirtyPreset(true);
 }
 
@@ -946,9 +1142,9 @@ void samplv1widget::clearSampleFile (void)
 
 	samplv1_ui *pSamplUi = ui_instance();
 	if (pSamplUi)
-		pSamplUi->setSampleFile(0);
+		pSamplUi->setSampleFile(nullptr);
 
-	updateSample(0);
+	updateSample(nullptr);
 }
 
 
@@ -971,32 +1167,23 @@ void samplv1widget::loadSampleFile ( const QString& sFilename )
 void samplv1widget::updateSample ( samplv1_sample *pSample, bool bDirty )
 {
 	samplv1_ui *pSamplUi = ui_instance();
-	if (m_ui.Gen1Sample->instance() == NULL)
+	if (m_ui.Gen1Sample->instance() == nullptr)
 		m_ui.Gen1Sample->setInstance(pSamplUi);
 
 	m_ui.Gen1Sample->setSample(pSample);
 
 	++m_iUpdate;
 	if (pSample) {
-		const bool bLoop = pSample->isLoop();
-		m_ui.Gen1Sample->setLoop(bLoop);
-		const uint32_t iLoopStart = pSample->loopStart();
-		const uint32_t iLoopEnd = pSample->loopEnd();
-		m_ui.Gen1Sample->setLoopStart(iLoopStart);
-		m_ui.Gen1Sample->setLoopEnd(iLoopEnd);
-		activateParamKnobs(pSample->filename() != NULL);
-		updateSampleLoop(pSample);
+		activateParamKnobs(pSample->filename() != nullptr);
+		updateOffsetLoop(pSample);
 		// Set current preset name if empty...
 		if (pSample->filename() && m_ui.Preset->preset().isEmpty()) {
 			m_ui.Preset->setPreset(
 				QFileInfo(pSample->filename()).completeBaseName());
 		}
 	} else {
-		m_ui.Gen1Sample->setLoop(false);
-		m_ui.Gen1Sample->setLoopStart(0);
-		m_ui.Gen1Sample->setLoopEnd(0);
 		activateParamKnobs(false);
-		updateSampleLoop(NULL);
+		updateOffsetLoop(nullptr);
 	}
 	--m_iUpdate;
 
@@ -1016,15 +1203,6 @@ void samplv1widget::playSample (void)
 }
 
 
-// MIDI note/octave name helper (static).
-QString samplv1widget::noteName ( int note )
-{
-	static const char *notes[] =
-		{ "C", "C#", "D", "D#", "E", "F", "F#", "G", "G#", "A", "A#", "B" };
-	return QString("%1 %2").arg(notes[note % 12]).arg((note / 12) - 1);
-}
-
-
 // Dirty close prompt,
 bool samplv1widget::queryClose (void)
 {
@@ -1032,8 +1210,8 @@ bool samplv1widget::queryClose (void)
 }
 
 
-// Loop range change.
-void samplv1widget::loopRangeChanged (void)
+// Offset start change.
+void samplv1widget::offsetStartChanged (void)
 {
 	if (m_iUpdate > 0)
 		return;
@@ -1041,10 +1219,28 @@ void samplv1widget::loopRangeChanged (void)
 	++m_iUpdate;
 	samplv1_ui *pSamplUi = ui_instance();
 	if (pSamplUi) {
-		const uint32_t iLoopStart = m_ui.Gen1Sample->loopStart();
-		const uint32_t iLoopEnd = m_ui.Gen1Sample->loopEnd();
-		pSamplUi->setLoopRange(iLoopStart, iLoopEnd);
-		updateSampleLoop(pSamplUi->sample(), true);
+		const uint32_t iOffsetStart = m_ui.Gen1OffsetStartSpinBox->value();
+		const uint32_t iOffsetEnd = pSamplUi->offsetEnd();
+		pSamplUi->setOffsetRange(iOffsetStart, iOffsetEnd);
+		updateOffsetLoop(pSamplUi->sample(), true);
+	}
+	--m_iUpdate;
+}
+
+
+// Offset end change.
+void samplv1widget::offsetEndChanged (void)
+{
+	if (m_iUpdate > 0)
+		return;
+
+	++m_iUpdate;
+	samplv1_ui *pSamplUi = ui_instance();
+	if (pSamplUi) {
+		const uint32_t iOffsetStart = pSamplUi->offsetStart();
+		const uint32_t iOffsetEnd = m_ui.Gen1OffsetEndSpinBox->value();
+		pSamplUi->setOffsetRange(iOffsetStart, iOffsetEnd);
+		updateOffsetLoop(pSamplUi->sample(), true);
 	}
 	--m_iUpdate;
 }
@@ -1063,7 +1259,7 @@ void samplv1widget::loopStartChanged (void)
 		const uint32_t iLoopEnd = pSamplUi->loopEnd();
 		pSamplUi->setLoopRange(iLoopStart, iLoopEnd);
 		m_ui.Gen1Sample->setLoopStart(iLoopStart);
-		updateSampleLoop(pSamplUi->sample(), true);
+		updateOffsetLoop(pSamplUi->sample(), true);
 	}
 	--m_iUpdate;
 }
@@ -1082,39 +1278,177 @@ void samplv1widget::loopEndChanged (void)
 		const uint32_t iLoopEnd = m_ui.Gen1LoopEndSpinBox->value();
 		pSamplUi->setLoopRange(iLoopStart, iLoopEnd);
 		m_ui.Gen1Sample->setLoopEnd(iLoopEnd);
-		updateSampleLoop(pSamplUi->sample(), true);
+		updateOffsetLoop(pSamplUi->sample(), true);
 	}
 	--m_iUpdate;
 }
 
 
-void samplv1widget::updateSampleLoop ( samplv1_sample *pSample, bool bDirty )
+// Loop fade change.
+void samplv1widget::loopFadeChanged (void)
 {
-	if (pSample) {
-		const bool bLoop = pSample->isLoop();
+	if (m_iUpdate > 0)
+		return;
+
+	++m_iUpdate;
+	samplv1_ui *pSamplUi = ui_instance();
+	if (pSamplUi) {
+		const bool bLoopFade = (m_ui.Gen1LoopFadeCheckBox->value() > 0.5f);
+		const uint32_t iLoopFade = m_ui.Gen1LoopFadeSpinBox->value();
+		if (bLoopFade && iLoopFade > 0) m_iLoopFade = iLoopFade;
+		pSamplUi->setLoopFade(bLoopFade ? m_iLoopFade : 0);
+		m_ui.StatusBar->showMessage(tr("Loop crossfade: %1")
+			.arg(bLoopFade ? QString::number(iLoopFade) : tr("Off")), 5000);
+		m_ui.Gen1LoopFadeSpinBox->setEnabled(bLoopFade);
+		updateDirtyPreset(true);
+	}
+	--m_iUpdate;
+}
+
+
+// Loop zero change.
+void samplv1widget::loopZeroChanged (void)
+{
+	if (m_iUpdate > 0)
+		return;
+
+	++m_iUpdate;
+	samplv1_ui *pSamplUi = ui_instance();
+	if (pSamplUi) {
+		const uint32_t iLoopStart = pSamplUi->loopStart();
+		const uint32_t iLoopEnd = pSamplUi->loopEnd();;
+		const bool bLoopZero = (m_ui.Gen1LoopZeroCheckBox->value() > 0.5f);
+		pSamplUi->setLoopZero(bLoopZero);
+		pSamplUi->setLoopRange(iLoopStart, iLoopEnd);
+		m_ui.StatusBar->showMessage(tr("Loop zero-crossing: %1")
+			.arg(bLoopZero ? tr("On") : tr("Off")), 5000);
+		updateDirtyPreset(true);
+	}
+	--m_iUpdate;
+}
+
+
+// Offset points changed (from UI).
+void samplv1widget::offsetRangeChanged (void)
+{
+	if (m_iUpdate > 0)
+		return;
+
+	++m_iUpdate;
+	samplv1_ui *pSamplUi = ui_instance();
+	if (pSamplUi) {
+		const uint32_t iOffsetStart = m_ui.Gen1Sample->offsetStart();
+		const uint32_t iOffsetEnd   = m_ui.Gen1Sample->offsetEnd();
+		pSamplUi->setOffsetRange(iOffsetStart, iOffsetEnd);
+		updateOffsetLoop(pSamplUi->sample(), true);
+	}
+	--m_iUpdate;
+}
+
+
+void samplv1widget::loopRangeChanged (void)
+{
+	if (m_iUpdate > 0)
+		return;
+
+	++m_iUpdate;
+	samplv1_ui *pSamplUi = ui_instance();
+	if (pSamplUi) {
+		const uint32_t iLoopStart = m_ui.Gen1Sample->loopStart();
+		const uint32_t iLoopEnd   = m_ui.Gen1Sample->loopEnd();
+		pSamplUi->setLoopRange(iLoopStart, iLoopEnd);
+		updateOffsetLoop(pSamplUi->sample(), true);
+	}
+	--m_iUpdate;
+}
+
+
+// Update offset/loop range change status.
+void samplv1widget::updateOffsetLoop ( samplv1_sample *pSample, bool bDirty )
+{
+	if (pSample && pSample->filename()) {
+		const bool     bOffset    = pSample->isOffset();
+		const uint32_t iOffsetStart = pSample->offsetStart();
+		const uint32_t iOffsetEnd = pSample->offsetEnd();
+		const bool     bLoop      = pSample->isLoop();
 		const uint32_t iLoopStart = pSample->loopStart();
-		const uint32_t iLoopEnd = pSample->loopEnd();
-		const uint32_t nframes = pSample->length();
+		const uint32_t iLoopEnd   = pSample->loopEnd();
+		const uint32_t iLoopFade  = pSample->loopCrossFade();
+		const bool     bLoopZero  = pSample->isLoopZeroCrossing();
+		const uint32_t nframes    = pSample->length();
+		const float    srate      = pSample->sampleRate();
+		m_ui.Gen1OffsetRangeLabel->setEnabled(bOffset);
+		m_ui.Gen1OffsetStartSpinBox->setSampleRate(srate);
+		m_ui.Gen1OffsetStartSpinBox->setEnabled(bOffset);
+		m_ui.Gen1OffsetStartSpinBox->setMinimum(0);
+		m_ui.Gen1OffsetStartSpinBox->setMaximum(bLoop ? iLoopStart : iOffsetEnd);
+		m_ui.Gen1OffsetStartSpinBox->setValue(iOffsetStart);
+		m_ui.Gen1OffsetEndSpinBox->setSampleRate(srate);
+		m_ui.Gen1OffsetEndSpinBox->setEnabled(bOffset);
+		m_ui.Gen1OffsetEndSpinBox->setMinimum(bLoop ? iLoopEnd : iOffsetStart);
+		m_ui.Gen1OffsetEndSpinBox->setMaximum(nframes);
+		m_ui.Gen1OffsetEndSpinBox->setValue(iOffsetEnd);
 		m_ui.Gen1LoopRangeLabel->setEnabled(bLoop);
+		m_ui.Gen1LoopStartSpinBox->setSampleRate(srate);
 		m_ui.Gen1LoopStartSpinBox->setEnabled(bLoop);
-		m_ui.Gen1LoopStartSpinBox->setMinimum(0);
-		m_ui.Gen1LoopStartSpinBox->setMaximum(
-		    iLoopEnd > 0 ? iLoopEnd : 0);
+		m_ui.Gen1LoopStartSpinBox->setMinimum(iOffsetStart);
+		m_ui.Gen1LoopStartSpinBox->setMaximum(bLoop ? iLoopEnd : iOffsetEnd);
+		m_ui.Gen1LoopEndSpinBox->setSampleRate(srate);
 		m_ui.Gen1LoopEndSpinBox->setEnabled(bLoop);
-		m_ui.Gen1LoopEndSpinBox->setMinimum(
-		    iLoopStart < nframes ? iLoopStart : nframes);
-		m_ui.Gen1LoopEndSpinBox->setMaximum(nframes);
+		m_ui.Gen1LoopEndSpinBox->setMinimum(bLoop ? iLoopStart : iOffsetStart);
+		m_ui.Gen1LoopEndSpinBox->setMaximum(iOffsetEnd);
 		m_ui.Gen1LoopStartSpinBox->setValue(iLoopStart);
 		m_ui.Gen1LoopEndSpinBox->setValue(iLoopEnd);
+		m_ui.Gen1LoopFadeCheckBox->setEnabled(bLoop);
+		m_ui.Gen1LoopFadeCheckBox->setValue(iLoopFade > 0 ? 1.0f : 0.0f);
+		m_ui.Gen1LoopFadeSpinBox->setEnabled(bLoop && iLoopFade > 0);
+		m_ui.Gen1LoopFadeSpinBox->setMinimum(0);
+		m_ui.Gen1LoopFadeSpinBox->setMaximum(
+			qMin(iLoopStart, (iLoopEnd - iLoopStart) >> 1));
+		if (iLoopFade > 0) m_iLoopFade = iLoopFade;
+		m_ui.Gen1LoopFadeSpinBox->setValue(m_iLoopFade);
+		m_ui.Gen1LoopZeroCheckBox->setValue(bLoopZero ? 1.0f : 0.0f);
+		m_ui.Gen1LoopZeroCheckBox->setEnabled(bLoop);
+		m_ui.Gen1Sample->setOffsetStart(iOffsetStart);
+		m_ui.Gen1Sample->setOffsetEnd(iOffsetEnd);
+		m_ui.Gen1Sample->setOffset(bOffset);
 		m_ui.Gen1Sample->setLoopStart(iLoopStart);
 		m_ui.Gen1Sample->setLoopEnd(iLoopEnd);
 		m_ui.Gen1Sample->setLoop(bLoop);
+		updateParam(samplv1::GEN1_OFFSET_1, float(iOffsetStart) / float(nframes));
+		updateParam(samplv1::GEN1_OFFSET_2, float(iOffsetEnd) / float(nframes));
+		updateParam(samplv1::GEN1_LOOP_1, float(iLoopStart) / float(nframes));
+		updateParam(samplv1::GEN1_LOOP_2, float(iLoopEnd) / float(nframes));
 		if (bDirty) {
-			m_ui.StatusBar->showMessage(tr("Loop start: %1, end: %2")
-				.arg(iLoopStart).arg(iLoopEnd), 5000);
+			QString sMessage;
+			if (bOffset) {
+				sMessage.append(tr("Offset: %1 - %2")
+					.arg(m_ui.Gen1Sample->textFromValue(iOffsetStart))
+					.arg(m_ui.Gen1Sample->textFromValue(iOffsetEnd)));
+			}
+			if (!sMessage.isEmpty()) {
+				sMessage.append(',');
+				sMessage.append(' ');
+			}
+			if (bLoop) {
+				sMessage.append(tr("Loop: %1 - %2")
+					.arg(m_ui.Gen1Sample->textFromValue(iLoopStart))
+					.arg(m_ui.Gen1Sample->textFromValue(iLoopEnd)));
+			}
+			if (!sMessage.isEmpty())
+				m_ui.StatusBar->showMessage(sMessage, 3000);
 			updateDirtyPreset(true);
 		}
 	} else {
+		m_ui.Gen1OffsetRangeLabel->setEnabled(false);
+		m_ui.Gen1OffsetStartSpinBox->setEnabled(false);
+		m_ui.Gen1OffsetStartSpinBox->setMinimum(0);
+		m_ui.Gen1OffsetStartSpinBox->setMaximum(0);
+		m_ui.Gen1OffsetStartSpinBox->setValue(0);
+		m_ui.Gen1OffsetEndSpinBox->setEnabled(false);
+		m_ui.Gen1OffsetEndSpinBox->setMinimum(0);
+		m_ui.Gen1OffsetEndSpinBox->setMaximum(0);
+		m_ui.Gen1OffsetEndSpinBox->setValue(0);
 		m_ui.Gen1LoopRangeLabel->setEnabled(false);
 		m_ui.Gen1LoopStartSpinBox->setEnabled(false);
 		m_ui.Gen1LoopStartSpinBox->setMinimum(0);
@@ -1124,6 +1458,15 @@ void samplv1widget::updateSampleLoop ( samplv1_sample *pSample, bool bDirty )
 		m_ui.Gen1LoopEndSpinBox->setMinimum(0);
 		m_ui.Gen1LoopEndSpinBox->setMaximum(0);
 		m_ui.Gen1LoopEndSpinBox->setValue(0);
+		m_ui.Gen1LoopFadeCheckBox->setEnabled(false);
+		m_ui.Gen1LoopFadeSpinBox->setEnabled(false);
+		m_ui.Gen1LoopFadeSpinBox->setMinimum(0);
+		m_ui.Gen1LoopFadeSpinBox->setMaximum(0);
+		m_ui.Gen1LoopFadeSpinBox->setValue(0);
+		m_ui.Gen1LoopZeroCheckBox->setEnabled(false);
+		m_ui.Gen1Sample->setOffsetStart(0);
+		m_ui.Gen1Sample->setOffsetEnd(0);
+		m_ui.Gen1Sample->setOffset(false);
 		m_ui.Gen1Sample->setLoopStart(0);
 		m_ui.Gen1Sample->setLoopEnd(0);
 		m_ui.Gen1Sample->setLoop(false);
@@ -1140,6 +1483,8 @@ void samplv1widget::activateParamKnobs ( bool bEnabled )
 	activateParamKnobsGroupBox(m_ui.Dca1GroupBox, bEnabled);
 	activateParamKnobsGroupBox(m_ui.Out1GroupBox, bEnabled);
 
+	m_ui.StatusBar->keybd()->setEnabled(bEnabled);
+
 	m_ui.Gen1Sample->setEnabled(true);
 }
 
@@ -1147,11 +1492,15 @@ void samplv1widget::activateParamKnobs ( bool bEnabled )
 void samplv1widget::activateParamKnobsGroupBox (
 	QGroupBox *pGroupBox, bool bEnabled )
 {
-	const QList<QWidget *>& children
-		= pGroupBox->findChildren<QWidget *> ();
-	QListIterator<QWidget *> iter(children);
-	while (iter.hasNext())
-		iter.next()->setEnabled(bEnabled);
+	if (pGroupBox->isCheckable()) {
+		pGroupBox->setEnabled(bEnabled);
+	} else {
+		const QList<QWidget *>& children
+			= pGroupBox->findChildren<QWidget *> ();
+		QListIterator<QWidget *> iter(children);
+		while (iter.hasNext())
+			iter.next()->setEnabled(bEnabled);
+	}
 }
 
 
@@ -1162,22 +1511,22 @@ void samplv1widget::contextMenuRequest ( const QPoint& pos )
 	QAction *pAction;
 
 	samplv1_ui *pSamplUi = ui_instance();
-	const char *pszSampleFile = NULL;
+	const char *pszSampleFile = nullptr;
 	if (pSamplUi)
 		pszSampleFile = pSamplUi->sampleFile();
 
 	pAction = menu.addAction(
 		QIcon(":/images/fileOpen.png"),
 		tr("Open Sample..."), this, SLOT(openSample()));
-	pAction->setEnabled(pSamplUi != NULL);
+	pAction->setEnabled(pSamplUi != nullptr);
 	pAction = menu.addAction(
 		QIcon(":/images/playSample.png"),
 		tr("Play"), this, SLOT(playSample()));
-	pAction->setEnabled(pSamplUi != NULL);
+	pAction->setEnabled(pSamplUi != nullptr);
 	menu.addSeparator();
 	pAction = menu.addAction(
 		tr("Reset"), this, SLOT(clearSample()));
-	pAction->setEnabled(pszSampleFile != NULL);
+	pAction->setEnabled(pszSampleFile != nullptr);
 
 	menu.exec(static_cast<QWidget *> (sender())->mapToGlobal(pos));
 }
@@ -1190,6 +1539,7 @@ void samplv1widget::updateLoadPreset ( const QString& sPreset )
 	if (pSamplUi)
 		updateSample(pSamplUi->sample());
 
+	resetParamKnobs();
 	updateParamValues();
 
 	m_ui.Preset->setPreset(sPreset);
@@ -1202,15 +1552,21 @@ void samplv1widget::updateLoadPreset ( const QString& sPreset )
 void samplv1widget::updateSchedNotify ( int stype, int sid )
 {
 	samplv1_ui *pSamplUi = ui_instance();
-	if (pSamplUi == NULL)
+	if (pSamplUi == nullptr)
 		return;
 
-#ifdef CONFIG_DEBUG
+#ifdef CONFIG_DEBUG_0
 	qDebug("samplv1widget::updateSchedNotify(%d, 0x%04x)", stype, sid);
 #endif
 
 	switch (samplv1_sched::Type(stype)) {
 	case samplv1_sched::MidiIn:
+		if (sid >= 0) {
+			const int key = (sid & 0x7f);
+			const int vel = (sid >> 7) & 0x7f;
+			m_ui.StatusBar->midiInNote(key, vel);
+		}
+		else
 		if (pSamplUi->midiInCount() > 0) {
 			m_ui.StatusBar->midiInLed(true);
 			QTimer::singleShot(200, this, SLOT(midiInLedTimeout()));
@@ -1240,6 +1596,7 @@ void samplv1widget::updateSchedNotify ( int stype, int sid )
 		updateSample(pSamplUi->sample());
 		if (sid > 0) {
 			updateParamValues();
+			resetParamKnobs();
 			updateDirtyPreset(false);
 		}
 		// Fall thru...
@@ -1249,6 +1606,40 @@ void samplv1widget::updateSchedNotify ( int stype, int sid )
 }
 
 
+// Direct note-on/off slot.
+void samplv1widget::directNoteOn ( int iNote, int iVelocity )
+{
+#ifdef CONFIG_DEBUG
+	qDebug("samplv1widget::directNoteOn(%d, %d)", iNote, iVelocity);
+#endif
+
+	samplv1_ui *pSamplUi = ui_instance();
+	if (pSamplUi)
+		pSamplUi->directNoteOn(iNote, iVelocity); // note-on!
+}
+
+
+// Keyboard note range change.
+void samplv1widget::noteRangeChanged (void)
+{
+	const int iNoteLow  = m_ui.StatusBar->keybd()->noteLow();
+	const int iNoteHigh = m_ui.StatusBar->keybd()->noteHigh();
+
+#ifdef CONFIG_DEBUG
+	qDebug("samplv1widget::noteRangeChanged(%d, %d)", iNoteLow, iNoteHigh);
+#endif
+
+	updateParam(samplv1::KEY1_LOW,  float(iNoteLow));
+	updateParam(samplv1::KEY1_HIGH, float(iNoteHigh));
+
+	m_ui.StatusBar->showMessage(QString("KEY Low: %1 (%2) High: %3 (%4)")
+		.arg(samplv1_ui::noteName(iNoteLow)).arg(iNoteLow)
+		.arg(samplv1_ui::noteName(iNoteHigh)).arg(iNoteHigh), 5000);
+
+	updateDirtyPreset(true);
+}
+
+
 // MIDI In LED timeout.
 void samplv1widget::midiInLedTimeout (void)
 {
@@ -1260,7 +1651,7 @@ void samplv1widget::midiInLedTimeout (void)
 void samplv1widget::helpConfigure (void)
 {
 	samplv1_ui *pSamplUi = ui_instance();
-	if (pSamplUi == NULL)
+	if (pSamplUi == nullptr)
 		return;
 
 	samplv1widget_config(pSamplUi, this).exec();
@@ -1311,7 +1702,7 @@ void samplv1widget::helpAbout (void)
 	sText += "</small>";
 	sText += "</p>\n";
 
-	QMessageBox::about(this, tr("About") + " " SAMPLV1_TITLE, sText);
+	QMessageBox::about(this, tr("About"), sText);
 }
 
 
@@ -1322,6 +1713,7 @@ void samplv1widget::helpAboutQt (void)
 }
 
 
+
 // Dirty flag (overridable virtual) methods.
 void samplv1widget::updateDirtyPreset ( bool bDirtyPreset )
 {
@@ -1339,15 +1731,15 @@ void samplv1widget::paramContextMenu ( const QPoint& pos )
 {
 	samplv1widget_param *pParam
 		= qobject_cast<samplv1widget_param *> (sender());
-	if (pParam == NULL)
+	if (pParam == nullptr)
 		return;
 
 	samplv1_ui *pSamplUi = ui_instance();
-	if (pSamplUi == NULL)
+	if (pSamplUi == nullptr)
 		return;
 
 	samplv1_controls *pControls = pSamplUi->controls();
-	if (pControls == NULL)
+	if (pControls == nullptr)
 		return;
 
 	if (!pControls->enabled())
@@ -1367,4 +1759,45 @@ void samplv1widget::paramContextMenu ( const QPoint& pos )
 }
 
 
+// Format changes (spinbox).
+void samplv1widget::spinboxContextMenu ( const QPoint& pos )
+{
+	samplv1widget_spinbox *pSpinBox
+		= qobject_cast<samplv1widget_spinbox *> (sender());
+	if (pSpinBox == nullptr)
+		return;
+
+	samplv1widget_spinbox::Format format = pSpinBox->format();
+
+	QMenu menu(this);
+	QAction *pAction;
+
+	pAction = menu.addAction(tr("&Frames"));
+	pAction->setCheckable(true);
+	pAction->setChecked(format == samplv1widget_spinbox::Frames);
+	pAction->setData(int(samplv1widget_spinbox::Frames));
+
+	pAction = menu.addAction(tr("&Time"));
+	pAction->setCheckable(true);
+	pAction->setChecked(format == samplv1widget_spinbox::Time);
+	pAction->setData(int(samplv1widget_spinbox::Time));
+
+	pAction = menu.exec(pSpinBox->mapToGlobal(pos));
+	if (pAction == nullptr)
+		return;
+
+	format = samplv1widget_spinbox::Format(pAction->data().toInt());
+	if (format != pSpinBox->format()) {
+		samplv1_config *pConfig = samplv1_config::getInstance();
+		if (pConfig) {
+			pConfig->iFrameTimeFormat = int(format);
+			m_ui.Gen1OffsetStartSpinBox->setFormat(format);
+			m_ui.Gen1OffsetEndSpinBox->setFormat(format);
+			m_ui.Gen1LoopStartSpinBox->setFormat(format);
+			m_ui.Gen1LoopEndSpinBox->setFormat(format);
+		}
+	}
+}
+
+
 // end of samplv1widget.cpp
