@@ -1,7 +1,7 @@
 // samplv1widget.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -60,17 +60,13 @@ public:
 	void closeSchedNotifier();
 
 	// Param port accessors.
-	void setParamValue(
-		samplv1::ParamIndex index, float fValue, bool bDefault = false);
+	void setParamValue(samplv1::ParamIndex index, float fValue, bool bIter = false);
 	float paramValue(samplv1::ParamIndex index) const;
 
 	// Param kbob (widget) mapper.
 	void setParamKnob(samplv1::ParamIndex index, samplv1widget_param *pKnob);
 	samplv1widget_param *paramKnob(samplv1::ParamIndex index) const;
 
-	// MIDI note/octave name helper.
-	static QString noteName(int note);
-
 	// Preset init.
 	void initPreset();
 	// Preset clear.
@@ -85,6 +81,9 @@ public slots:
 	void loadPreset(const QString& sFilename);
 	void savePreset(const QString& sFilename);
 
+	// Direct note-on/off slot.
+	void directNoteOn(int iNote, int iVelocity);
+
 protected slots:
 
 	// Preset renewal.
@@ -111,13 +110,25 @@ protected slots:
 	// Reset param knobs to default value.
 	void resetParams();
 
+	// Randomize params (partial).
+	void randomParams();
+
 	// Swap params A/B.
 	void swapParams(bool bOn);
 
-	// Loop range changes.
-	void loopRangeChanged();
+	// Offset point changed.
+	void offsetStartChanged();
+	void offsetEndChanged();
+
+	// Loop point changes.
 	void loopStartChanged();
 	void loopEndChanged();
+	void loopFadeChanged();
+	void loopZeroChanged();
+
+	// Offset/loop points changed (from UI).
+	void offsetRangeChanged();
+	void loopRangeChanged();
 
 	// Notification updater.
 	void updateSchedNotify(int stype, int sid);
@@ -125,9 +136,15 @@ protected slots:
 	// MIDI In LED timeout.
 	void midiInLedTimeout();
 
+	// Keyboard note range change.
+	void noteRangeChanged();
+
 	// Param knob context menu.
 	void paramContextMenu(const QPoint& pos);
 
+	// Format changes (spinbox).
+	void spinboxContextMenu(const QPoint& pos);
+
 	// Menu actions.
 	void helpConfigure();
 
@@ -162,14 +179,14 @@ protected:
 	// Sample updater.
 	void updateSample(samplv1_sample *pSample, bool bDirty = false);
 
-	// Update loop range change status.
-	void updateSampleLoop(samplv1_sample *pSample, bool bDirty = false);
+	// Update offset/loop range change status.
+	void updateOffsetLoop(samplv1_sample *pSample, bool bDirty = false);
 
 	// Param port methods.
 	virtual void updateParam(samplv1::ParamIndex index, float fValue) const = 0;
 
 	// Update local tied widgets.
-	void updateParamEx(samplv1::ParamIndex index, float fValue);
+	void updateParamEx(samplv1::ParamIndex index, float fValue, bool bIter = false);
 
 	// Update scheduled controllers param/knob widgets.
 	void updateSchedParam(samplv1::ParamIndex index, float fValue);
@@ -197,6 +214,8 @@ private:
 
 	float m_params_ab[samplv1::NUM_PARAMS];
 
+	uint32_t m_iLoopFade;
+
 	int m_iUpdate;
 };
 
@@ -212,7 +231,7 @@ class samplv1widget_sched : public QObject
 public:
 
 	// ctor.
-	samplv1widget_sched(samplv1 *pSampl, QObject *pParent = NULL)
+	samplv1widget_sched(samplv1 *pSampl, QObject *pParent = nullptr)
 		: QObject(pParent), m_notifier(pSampl, this) {}
 
 signals:
@@ -223,12 +242,12 @@ signals:
 protected:
 
 	// Notififier visitor.
-	class Notifier : public samplv1_sched_notifier
+	class Notifier : public samplv1_sched::Notifier
 	{
 	public:
 
 		Notifier(samplv1 *pSampl, samplv1widget_sched *pSched)
-			: samplv1_sched_notifier(pSampl), m_pSched(pSched) {}
+			: samplv1_sched::Notifier(pSampl), m_pSched(pSched) {}
 
 		void notify(samplv1_sched::Type stype, int sid) const
 			{ m_pSched->emit_notify(stype, sid); }
