@@ -1,7 +1,7 @@
 // samplv1widget_config.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -32,6 +32,9 @@ namespace Ui { class samplv1widget_config; }
 
 class samplv1_ui;
 
+class QComboBox;
+class QFileInfo;
+
 
 //----------------------------------------------------------------------------
 // samplv1widget_config -- UI wrapper form.
@@ -55,6 +58,8 @@ public:
 protected slots:
 
 	// command slots.
+	void editCustomColorThemes();
+
 	void controlsAddItem();
 	void controlsEditItem();
 	void controlsDeleteItem();
@@ -75,6 +80,12 @@ protected slots:
 	void controlsEnabled(bool);
 	void programsEnabled(bool);
 
+	void tuningTabChanged(int);
+	void tuningRefNoteClicked();
+	void tuningScaleFileClicked();
+	void tuningKeyMapFileClicked();
+
+	void tuningChanged();
 	void controlsChanged();
 	void programsChanged();
 	void optionsChanged();
@@ -85,6 +96,18 @@ protected slots:
 
 protected:
 
+	// Custom color/style themes settlers.
+	void resetCustomColorThemes(const QString& sCustomColorTheme);
+	void resetCustomStyleThemes(const QString& sCustomColorTheme);
+
+	// Combo box history persistence helper prototypes.
+	void loadComboBoxHistory(QComboBox *pComboBox);
+	void saveComboBoxHistory(QComboBox *pComboBox);
+
+	// Combo box settter/gettter helper prototypes.
+	bool setComboBoxCurrentItem(QComboBox *pComboBox, const QFileInfo& info);
+	QString comboBoxCurrentItem(QComboBox *pComboBox);
+
 	// stabilizer.
 	void stabilize();
 
@@ -98,6 +121,7 @@ private:
 	samplv1_ui *m_pSamplUi;
 
 	// Dialog dirty flag.
+	int m_iDirtyTuning;
 	int m_iDirtyControls;
 	int m_iDirtyPrograms;
 	int m_iDirtyOptions;
