@@ -1,7 +1,7 @@
 // samplv1widget_param.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -38,102 +38,95 @@ inline int iroundf(float x) { return int(x < 0.0f ? x - 0.5f : x + 0.5f); }
 
 
 //-------------------------------------------------------------------------
-// samplv1widget_dial - A better QDial widget.
+// samplv1widget_param_style - Custom widget style.
+//
 
-samplv1widget_dial::DialMode
-samplv1widget_dial::g_dialMode = samplv1widget_dial::DefaultMode;
-
-// Set knob dial mode behavior.
-void samplv1widget_dial::setDialMode ( DialMode dialMode )
-	{ g_dialMode = dialMode; }
-
-samplv1widget_dial::DialMode samplv1widget_dial::dialMode (void)
-	{ return g_dialMode; }
+#include <QProxyStyle>
+#include <QPainter>
+#include <QIcon>
 
 
-// Constructor.
-samplv1widget_dial::samplv1widget_dial ( QWidget *pParent )
-	: QDial(pParent), m_bMousePressed(false), m_fLastDragValue(0.0f)
+class samplv1widget_param_style : public QProxyStyle
 {
-}
+public:
 
-
-// Mouse angle determination.
-float samplv1widget_dial::mouseAngle ( const QPoint& pos )
-{
-	const float dx = pos.x() - (width() >> 1);
-	const float dy = (height() >> 1) - pos.y();
-	return 180.0f * ::atan2f(dx, dy) / float(M_PI);
-}
-
-
-// Alternate mouse behavior event handlers.
-void samplv1widget_dial::mousePressEvent ( QMouseEvent *pMouseEvent )
-{
-	if (g_dialMode == DefaultMode) {
-		QDial::mousePressEvent(pMouseEvent);
-	} else if (pMouseEvent->button() == Qt::LeftButton) {
-		m_bMousePressed = true;
-		m_posMouse = pMouseEvent->pos();
-		m_fLastDragValue = float(value());
-		emit sliderPressed();
-	}
-}
-
-
-void samplv1widget_dial::mouseMoveEvent ( QMouseEvent *pMouseEvent )
-{
-	if (g_dialMode == DefaultMode) {
-		QDial::mouseMoveEvent(pMouseEvent);
-		return;
+	// Constructor.
+	samplv1widget_param_style() : QProxyStyle()
+	{
+		m_icon.addPixmap(
+			QPixmap(":/images/ledOff.png"), QIcon::Normal, QIcon::Off);
+		m_icon.addPixmap(
+			QPixmap(":/images/ledOn.png"), QIcon::Normal, QIcon::On);
 	}
 
-	if (!m_bMousePressed)
-		return;
 
-	const QPoint& pos = pMouseEvent->pos();
-	const int dx = pos.x() - m_posMouse.x();
-	const int dy = pos.y() - m_posMouse.y();
-	float fAngleDelta =  mouseAngle(pos) - mouseAngle(m_posMouse);
-	int iNewValue = value();
-
-	switch (g_dialMode)	{
-	case LinearMode:
-		iNewValue = int(m_fLastDragValue) + dx - dy;
-		break;
-	case AngularMode:
-	default:
-		// Forget about the drag origin to be robust on full rotations
-		if (fAngleDelta > +180.0f) fAngleDelta -= 360.0f;
+	// Hints override.
+	int styleHint(StyleHint hint, const QStyleOption *option,
+		const QWidget *widget, QStyleHintReturn *retdata) const
+	{
+		if (hint == QStyle::SH_UnderlineShortcut)
+			return 0;
 		else
-		if (fAngleDelta < -180.0f) fAngleDelta += 360.0f;
-		m_fLastDragValue += float(maximum() - minimum()) * fAngleDelta / 270.0f;
-		if (m_fLastDragValue > float(maximum()))
-			m_fLastDragValue = float(maximum());
+			return QProxyStyle::styleHint(hint, option, widget, retdata);
+	}
+
+	// Paint job.
+	void drawPrimitive(PrimitiveElement element,
+		const QStyleOption *option,
+		QPainter *painter, const QWidget *widget) const
+	{
+		if (element == PE_IndicatorRadioButton ||
+			element == PE_IndicatorCheckBox) {
+			const QRect& rect = option->rect;
+			if (option->state & State_Enabled) {
+				if (option->state & State_On)
+					m_icon.paint(painter, rect,
+						Qt::AlignCenter, QIcon::Normal, QIcon::On);
+				else
+			//	if (option->state & State_Off)
+					m_icon.paint(painter, rect,
+						Qt::AlignCenter, QIcon::Normal, QIcon::Off);
+			} else {
+				m_icon.paint(painter, rect,
+					Qt::AlignCenter, QIcon::Disabled, QIcon::Off);
+			}
+		}
 		else
-		if (m_fLastDragValue < float(minimum()))
-			m_fLastDragValue = float(minimum());
-		m_posMouse = pos;
-		iNewValue = int(m_fLastDragValue + 0.5f);
-		break;
+		QProxyStyle::drawPrimitive(element, option, painter, widget);
 	}
 
-	setValue(iNewValue);
-	update();
-
-	emit sliderMoved(value());
-}
-
-
-void samplv1widget_dial::mouseReleaseEvent ( QMouseEvent *pMouseEvent )
-{
-	if (g_dialMode == DefaultMode
-		&& pMouseEvent->button() != Qt::MidButton) {
-		QDial::mouseReleaseEvent(pMouseEvent);
-	} else if (m_bMousePressed) {
-		m_bMousePressed = false;
+	// Spiced up text margins
+	void drawItemText(QPainter *painter, const QRect& rectangle,
+		int alignment, const QPalette& palette, bool enabled,
+		const QString& text, QPalette::ColorRole textRole) const
+	{
+		QRect rect = rectangle;
+		rect.setLeft(rect.left() - 4);
+		rect.setRight(rect.right() + 4);
+		QProxyStyle::drawItemText(painter, rect,
+			alignment, palette, enabled, text, textRole);
 	}
-}
+
+	static void addRef ()
+		{ if (++g_iRefCount == 1) g_pStyle = new samplv1widget_param_style(); }
+
+	static void releaseRef ()
+		{ if (--g_iRefCount == 0) { delete g_pStyle; g_pStyle = nullptr; } }
+
+	static samplv1widget_param_style *getRef ()
+		{ return g_pStyle; }
+
+private:
+
+	QIcon m_icon;
+
+	static samplv1widget_param_style *g_pStyle;
+	static unsigned int g_iRefCount;
+};
+
+
+samplv1widget_param_style *samplv1widget_param_style::g_pStyle = nullptr;
+unsigned int samplv1widget_param_style::g_iRefCount = 0;
 
 
 //-------------------------------------------------------------------------
@@ -178,11 +171,11 @@ QString samplv1widget_param::text (void) const
 }
 
 
-void samplv1widget_param::setValue ( float fValue, bool bDefault )
+void samplv1widget_param::setValue ( float fValue )
 {
 	QPalette pal;
 
-	if (bDefault) {
+	if (m_iDefaultValue == 0) {
 		m_fDefaultValue = fValue;
 		m_iDefaultValue++;
 	}
@@ -305,6 +298,105 @@ float samplv1widget_param::valueFromScale ( float fScale ) const
 }
 
 
+//-------------------------------------------------------------------------
+// samplv1widget_dial - A better QDial widget.
+
+samplv1widget_dial::DialMode
+samplv1widget_dial::g_dialMode = samplv1widget_dial::DefaultMode;
+
+// Set knob dial mode behavior.
+void samplv1widget_dial::setDialMode ( DialMode dialMode )
+	{ g_dialMode = dialMode; }
+
+samplv1widget_dial::DialMode samplv1widget_dial::dialMode (void)
+	{ return g_dialMode; }
+
+
+// Constructor.
+samplv1widget_dial::samplv1widget_dial ( QWidget *pParent )
+	: QDial(pParent), m_bMousePressed(false), m_fLastDragValue(0.0f)
+{
+}
+
+
+// Mouse angle determination.
+float samplv1widget_dial::mouseAngle ( const QPoint& pos )
+{
+	const float dx = pos.x() - (width() >> 1);
+	const float dy = (height() >> 1) - pos.y();
+	return 180.0f * ::atan2f(dx, dy) / float(M_PI);
+}
+
+
+// Alternate mouse behavior event handlers.
+void samplv1widget_dial::mousePressEvent ( QMouseEvent *pMouseEvent )
+{
+	if (g_dialMode == DefaultMode) {
+		QDial::mousePressEvent(pMouseEvent);
+	} else if (pMouseEvent->button() == Qt::LeftButton) {
+		m_bMousePressed = true;
+		m_posMouse = pMouseEvent->pos();
+		m_fLastDragValue = float(value());
+		emit sliderPressed();
+	}
+}
+
+
+void samplv1widget_dial::mouseMoveEvent ( QMouseEvent *pMouseEvent )
+{
+	if (g_dialMode == DefaultMode) {
+		QDial::mouseMoveEvent(pMouseEvent);
+		return;
+	}
+
+	if (!m_bMousePressed)
+		return;
+
+	const QPoint& pos = pMouseEvent->pos();
+	const int dx = pos.x() - m_posMouse.x();
+	const int dy = pos.y() - m_posMouse.y();
+	float fAngleDelta =  mouseAngle(pos) - mouseAngle(m_posMouse);
+	int iNewValue = value();
+
+	switch (g_dialMode)	{
+	case LinearMode:
+		iNewValue = int(m_fLastDragValue) + dx - dy;
+		break;
+	case AngularMode:
+	default:
+		// Forget about the drag origin to be robust on full rotations
+		if (fAngleDelta > +180.0f) fAngleDelta -= 360.0f;
+		else
+		if (fAngleDelta < -180.0f) fAngleDelta += 360.0f;
+		m_fLastDragValue += float(maximum() - minimum()) * fAngleDelta / 270.0f;
+		if (m_fLastDragValue > float(maximum()))
+			m_fLastDragValue = float(maximum());
+		else
+		if (m_fLastDragValue < float(minimum()))
+			m_fLastDragValue = float(minimum());
+		m_posMouse = pos;
+		iNewValue = int(m_fLastDragValue + 0.5f);
+		break;
+	}
+
+	setValue(iNewValue);
+	update();
+
+	emit sliderMoved(value());
+}
+
+
+void samplv1widget_dial::mouseReleaseEvent ( QMouseEvent *pMouseEvent )
+{
+	if (g_dialMode == DefaultMode
+		&& pMouseEvent->button() != Qt::MidButton) {
+		QDial::mouseReleaseEvent(pMouseEvent);
+	} else if (m_bMousePressed) {
+		m_bMousePressed = false;
+	}
+}
+
+
 //-------------------------------------------------------------------------
 // samplv1widget_knob - Custom knob/dial widget.
 //
@@ -343,10 +435,10 @@ QString samplv1widget_knob::text (void) const
 }
 
 
-void samplv1widget_knob::setValue ( float fValue, bool bDefault )
+void samplv1widget_knob::setValue ( float fValue )
 {
 	const bool bDialBlock = m_pDial->blockSignals(true);
-	samplv1widget_param::setValue(fValue, bDefault);
+	samplv1widget_param::setValue(fValue);
 	m_pDial->setValue(scaleFromValue(fValue));
 	m_pDial->blockSignals(bDialBlock);
 }
@@ -404,7 +496,7 @@ samplv1widget_edit::EditMode samplv1widget_edit::editMode (void)
 samplv1widget_edit::samplv1widget_edit ( QWidget *pParent )
 	: QDoubleSpinBox(pParent), m_iTextChanged(0)
 {
-	QObject::connect(lineEdit(),
+	QObject::connect(QDoubleSpinBox::lineEdit(),
 		SIGNAL(textChanged(const QString&)),
 		SLOT(lineEditTextChanged(const QString&)));
 	QObject::connect(this,
@@ -428,7 +520,7 @@ void samplv1widget_edit::spinBoxEditingFinished (void)
 {
 	if (g_editMode == DeferredMode) {
 		m_iTextChanged = 0;
-		emit valueChangedEx(value());
+		emit valueChangedEx(QDoubleSpinBox::value());
 	}
 }
 
@@ -440,6 +532,21 @@ void samplv1widget_edit::spinBoxValueChanged ( double spinValue )
 }
 
 
+// Inherited/override methods.
+QValidator::State samplv1widget_edit::validate ( QString& sText, int& iPos ) const
+{
+	const QValidator::State state
+		= QDoubleSpinBox::validate(sText, iPos);
+
+	if (state == QValidator::Acceptable
+		&& g_editMode == DeferredMode
+		&& m_iTextChanged == 0)
+		return QValidator::Intermediate;
+
+	return state;
+}
+
+
 //-------------------------------------------------------------------------
 // samplv1widget_spin - Custom knob/spin-box widget.
 //
@@ -473,10 +580,10 @@ samplv1widget_spin::samplv1widget_spin ( QWidget *pParent )
 
 
 // Virtual accessors.
-void samplv1widget_spin::setValue ( float fValue, bool bDefault )
+void samplv1widget_spin::setValue ( float fValue )
 {
 	const bool bSpinBlock = m_pSpinBox->blockSignals(true);
-	samplv1widget_knob::setValue(fValue, bDefault);
+	samplv1widget_knob::setValue(fValue);
 	m_pSpinBox->setValue(scaleFromValue(fValue));
 	m_pSpinBox->blockSignals(bSpinBlock);
 }
@@ -569,10 +676,10 @@ samplv1widget_combo::samplv1widget_combo ( QWidget *pParent )
 
 
 // Virtual accessors.
-void samplv1widget_combo::setValue ( float fValue, bool bDefault )
+void samplv1widget_combo::setValue ( float fValue )
 {
 	const bool bComboBlock = m_pComboBox->blockSignals(true);
-	samplv1widget_knob::setValue(fValue, bDefault);
+	samplv1widget_knob::setValue(fValue);
 	m_pComboBox->setCurrentIndex(iroundf(fValue));
 	m_pComboBox->blockSignals(bComboBlock);
 }
@@ -636,98 +743,6 @@ void samplv1widget_combo::wheelEvent ( QWheelEvent *pWheelEvent )
 }
 
 
-//-------------------------------------------------------------------------
-// samplv1widget_param_style - Custom widget style.
-//
-
-#include <QProxyStyle>
-#include <QPainter>
-#include <QIcon>
-
-
-class samplv1widget_param_style : public QProxyStyle
-{
-public:
-
-	// Constructor.
-	samplv1widget_param_style() : QProxyStyle()
-	{
-		m_icon.addPixmap(
-			QPixmap(":/images/ledOff.png"), QIcon::Normal, QIcon::Off);
-		m_icon.addPixmap(
-			QPixmap(":/images/ledOn.png"), QIcon::Normal, QIcon::On);
-	}
-
-
-	// Hints override.
-	int styleHint(StyleHint hint, const QStyleOption *option,
-		const QWidget *widget, QStyleHintReturn *retdata) const
-	{
-		if (hint == QStyle::SH_UnderlineShortcut)
-			return 0;
-		else
-			return QProxyStyle::styleHint(hint, option, widget, retdata);
-	}
-
-	// Paint job.
-	void drawPrimitive(PrimitiveElement element,
-		const QStyleOption *option,
-		QPainter *painter, const QWidget *widget) const
-	{
-		if (element == PE_IndicatorRadioButton ||
-			element == PE_IndicatorCheckBox) {
-			const QRect& rect = option->rect;
-			if (option->state & State_Enabled) {
-				if (option->state & State_On)
-					m_icon.paint(painter, rect,
-						Qt::AlignCenter, QIcon::Normal, QIcon::On);
-				else
-			//	if (option->state & State_Off)
-					m_icon.paint(painter, rect,
-						Qt::AlignCenter, QIcon::Normal, QIcon::Off);
-			} else {
-				m_icon.paint(painter, rect,
-					Qt::AlignCenter, QIcon::Disabled, QIcon::Off);
-			}
-		}
-		else
-		QProxyStyle::drawPrimitive(element, option, painter, widget);
-	}
-
-	// Spiced up text margins
-	void drawItemText(QPainter *painter, const QRect& rectangle,
-		int alignment, const QPalette& palette, bool enabled,
-		const QString& text, QPalette::ColorRole textRole) const
-	{
-		QRect rect = rectangle;
-		rect.setLeft(rect.left() - 4);
-		rect.setRight(rect.right() + 4);
-		QProxyStyle::drawItemText(painter, rect,
-			alignment, palette, enabled, text, textRole);
-	}
-
-	static void addRef ()
-		{ if (++g_iRefCount == 1) g_pStyle = new samplv1widget_param_style(); }
-
-	static void releaseRef ()
-		{ if (--g_iRefCount == 0) { delete g_pStyle; g_pStyle = NULL; } }
-
-	static samplv1widget_param_style *getRef ()
-		{ return g_pStyle; }
-
-private:
-
-	QIcon m_icon;
-
-	static samplv1widget_param_style *g_pStyle;
-	static unsigned int g_iRefCount;
-};
-
-
-samplv1widget_param_style *samplv1widget_param_style::g_pStyle = NULL;
-unsigned int samplv1widget_param_style::g_iRefCount = 0;
-
-
 //-------------------------------------------------------------------------
 // samplv1widget_radio - Custom radio/button widget.
 //
@@ -759,14 +774,14 @@ samplv1widget_radio::~samplv1widget_radio (void)
 
 
 // Virtual accessors.
-void samplv1widget_radio::setValue ( float fValue, bool bDefault )
+void samplv1widget_radio::setValue ( float fValue )
 {
 	const int iRadioValue = iroundf(fValue);
 	QRadioButton *pRadioButton
 		= static_cast<QRadioButton *> (m_group.button(iRadioValue));
 	if (pRadioButton) {
 		const bool bRadioBlock = pRadioButton->blockSignals(true);
-		samplv1widget_param::setValue(float(iRadioValue), bDefault);
+		samplv1widget_param::setValue(float(iRadioValue));
 		pRadioButton->setChecked(true);
 		pRadioButton->blockSignals(bRadioBlock);
 	}
@@ -910,11 +925,11 @@ Qt::Alignment samplv1widget_check::alignment (void) const
 
 
 // Virtual accessors.
-void samplv1widget_check::setValue ( float fValue, bool bDefault )
+void samplv1widget_check::setValue ( float fValue )
 {
 	const bool bCheckValue = (fValue > 0.5f * (maximum() + minimum()));
 	const bool bCheckBlock = m_pCheckBox->blockSignals(true);
-	samplv1widget_param::setValue(bCheckValue ? maximum() : minimum(), bDefault);
+	samplv1widget_param::setValue(bCheckValue ? maximum() : minimum());
 	m_pCheckBox->setChecked(bCheckValue);
 	m_pCheckBox->blockSignals(bCheckBlock);
 }
@@ -926,4 +941,85 @@ void samplv1widget_check::checkBoxValueChanged ( bool bCheckValue )
 }
 
 
+//-------------------------------------------------------------------------
+// samplv1widget_group - Custom checkable group-box widget.
+//
+
+// Constructor.
+samplv1widget_group::samplv1widget_group ( QWidget *pParent )
+	: QGroupBox(pParent)
+{
+	samplv1widget_param_style::addRef();
+#if 0
+	QGroupBox::setStyleSheet(
+	//	"QGroupBox::indicator { width: 16px; height: 16px; }"
+		"QGroupBox::indicator::unchecked { image: url(:/images/ledOff.png); }"
+		"QGroupBox::indicator::checked   { image: url(:/images/ledOn.png);  }"
+		);
+#endif
+	QGroupBox::setStyle(samplv1widget_param_style::getRef());
+
+	m_pParam = new samplv1widget_param(this);
+	m_pParam->setToolTip(QGroupBox::toolTip());
+
+	QObject::connect(m_pParam,
+		 SIGNAL(valueChanged(float)),
+		 SLOT(paramValueChanged(float)));
+
+	QObject::connect(this,
+		 SIGNAL(toggled(bool)),
+		 SLOT(groupBoxValueChanged(bool)));
+}
+
+
+// Destructor.
+samplv1widget_group::~samplv1widget_group (void)
+{
+	samplv1widget_param_style::releaseRef();
+
+	delete m_pParam;
+}
+
+
+// Accessors.
+void samplv1widget_group::setToolTip ( const QString& sToolTip )
+{
+	m_pParam->setToolTip(sToolTip);
+}
+
+
+QString samplv1widget_group::toolTip (void) const
+{
+	return m_pParam->toolTip();
+}
+
+
+samplv1widget_param *samplv1widget_group::param (void) const
+{
+	return m_pParam;
+}
+
+
+// Virtual accessors.
+void samplv1widget_group::paramValueChanged ( float fValue )
+{
+	const float fMaximum = m_pParam->maximum();
+	const float fMinimum = m_pParam->minimum();
+
+	const bool bGroupValue = (fValue > 0.5f * (fMaximum + fMinimum));
+	const bool bGroupBlock = QGroupBox::blockSignals(true);
+	QGroupBox::setChecked(bGroupValue);
+	QGroupBox::blockSignals(bGroupBlock);
+}
+
+
+void samplv1widget_group::groupBoxValueChanged ( bool bGroupValue )
+{
+	const float fMaximum = m_pParam->maximum();
+	const float fMinimum = m_pParam->minimum();
+
+	m_pParam->setValue(bGroupValue ? fMaximum : fMinimum);
+}
+
+
 // end of samplv1widget_param.cpp
