@@ -1,7 +1,7 @@
 // samplv1widget_preset.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -52,8 +52,8 @@ samplv1widget_preset::samplv1widget_preset ( QWidget *pParent )
 	m_pOpenButton->setIcon(QIcon(":/images/presetOpen.png"));
 	m_pComboBox->setEditable(true);
 	m_pComboBox->setMinimumWidth(240);
-#if QT_VERSION >= 0x040200
-	m_pComboBox->setCompleter(NULL);
+#if QT_VERSION >= QT_VERSION_CHECK(4, 2, 0)
+	m_pComboBox->setCompleter(nullptr);
 #endif
 	m_pComboBox->setInsertPolicy(QComboBox::NoInsert);
 	m_pSaveButton->setIcon(QIcon(":/images/presetSave.png"));
@@ -114,10 +114,6 @@ void samplv1widget_preset::clearPreset (void)
 {
 	++m_iInitPreset;
 
-	samplv1_config *pConfig = samplv1_config::getInstance();
-	if (pConfig)
-		pConfig->sPreset.clear();
-
 	const bool bBlockSignals = m_pComboBox->blockSignals(true);
 	m_pComboBox->clearEditText();
 	m_pComboBox->blockSignals(bBlockSignals);
@@ -126,10 +122,6 @@ void samplv1widget_preset::clearPreset (void)
 
 void samplv1widget_preset::setPreset ( const QString& sPreset )
 {
-	samplv1_config *pConfig = samplv1_config::getInstance();
-	if (pConfig)
-		pConfig->sPreset = sPreset;
-
 	const bool bBlockSignals = m_pComboBox->blockSignals(true);
 	m_pComboBox->setEditText(sPreset);
 	m_pComboBox->blockSignals(bBlockSignals);
@@ -148,14 +140,14 @@ bool samplv1widget_preset::queryPreset (void)
 		return true;
 
 	samplv1_config *pConfig = samplv1_config::getInstance();
-	if (pConfig == NULL)
+	if (pConfig == nullptr)
 		return false;
 
 	if (m_iDirtyPreset > 0) {
-		const QString sPreset(pConfig->sPreset);
+		const QString& sPreset(pConfig->sPreset);
 		if (sPreset.isEmpty()) {
 			if (QMessageBox::warning(this,
-				tr("Warning") + " - " SAMPLV1_TITLE,
+				tr("Warning"),
 				tr("Some parameters have been changed.\n\n"
 				"Do you want to discard the changes?"),
 				QMessageBox::Discard |
@@ -163,7 +155,7 @@ bool samplv1widget_preset::queryPreset (void)
 				return false;
 		} else {
 			switch (QMessageBox::warning(this,
-				tr("Warning") + " - " SAMPLV1_TITLE,
+				tr("Warning"),
 				tr("Some preset parameters have been changed:\n\n"
 				"\"%1\".\n\nDo you want to save the changes?")
 				.arg(sPreset),
@@ -203,8 +195,8 @@ void samplv1widget_preset::loadPreset ( const QString& sPreset )
 	if (pConfig) {
 		emit loadPresetFile(pConfig->presetFile(sPreset));
 		++m_iInitPreset;
-	//	pConfig->sPreset = sPreset;
-	//	setPreset(sPreset);
+		pConfig->sPreset = sPreset;
+		setPreset(sPreset);
 		refreshPreset();
 	}
 
@@ -214,8 +206,13 @@ void samplv1widget_preset::loadPreset ( const QString& sPreset )
 
 void samplv1widget_preset::newPreset (void)
 {
-	if (queryPreset()) {
+	if (!queryPreset())
+		return;
+
+	samplv1_config *pConfig = samplv1_config::getInstance();
+	if (pConfig) {
 		emit newPresetFile();
+		pConfig->sPreset.clear();
 		clearPreset();
 		refreshPreset();
 	}
@@ -227,24 +224,24 @@ void samplv1widget_preset::newPreset (void)
 void samplv1widget_preset::openPreset (void)
 {
 	samplv1_config *pConfig = samplv1_config::getInstance();
-	if (pConfig == NULL)
+	if (pConfig == nullptr)
 		return;
 
 	QStringList files;
 
 	const QString  sExt(SAMPLV1_TITLE);
-	const QString& sTitle  = tr("Open Preset") + " - " SAMPLV1_TITLE;
+	const QString& sTitle  = tr("Open Preset");
 	const QString& sFilter = tr("Preset files (*.%1)").arg(sExt);
 
-	QWidget *pParentWidget = NULL;
+	QWidget *pParentWidget = nullptr;
 	QFileDialog::Options options = 0;
 	if (pConfig->bDontUseNativeDialogs) {
 		options |= QFileDialog::DontUseNativeDialog;
-		pParentWidget = parentWidget();
+		pParentWidget = QWidget::window();
 	}
-#if 1//QT_VERSION < 0x040400
+#if 1//QT_VERSION < QT_VERSION_CHECK(4, 4, 0)
 	files = QFileDialog::getOpenFileNames(pParentWidget,
-		sTitle, pConfig->sPresetDir, sFilter, NULL, options);
+		sTitle, pConfig->sPresetDir, sFilter, nullptr, options);
 #else
 	QFileDialog fileDialog(pParentWidget,
 		sTitle, pConfig->sPresetDir, sFilter);
@@ -271,7 +268,7 @@ void samplv1widget_preset::openPreset (void)
 				if (++iPreset == 1) {
 					++m_iInitPreset;
 					emit loadPresetFile(sFilename);
-				//	pConfig->sPreset = sPreset;
+					pConfig->sPreset = sPreset;
 					pConfig->sPresetDir = fi.absolutePath();
 					setPreset(sPreset);
 				}
@@ -295,24 +292,24 @@ void samplv1widget_preset::savePreset ( const QString& sPreset )
 		return;
 
 	samplv1_config *pConfig = samplv1_config::getInstance();
-	if (pConfig == NULL)
+	if (pConfig == nullptr)
 		return;
 
 	const QString sExt(SAMPLV1_TITLE);
 	QFileInfo fi(QDir(pConfig->sPresetDir), sPreset + '.' + sExt);
 	QString sFilename = fi.absoluteFilePath();
 	if (!fi.exists()) {
-		const QString& sTitle  = tr("Save Preset") + " - " SAMPLV1_TITLE;
+		const QString& sTitle  = tr("Save Preset");
 		const QString& sFilter = tr("Preset files (*.%1)").arg(sExt);
-		QWidget *pParentWidget = NULL;
+		QWidget *pParentWidget = nullptr;
 		QFileDialog::Options options = 0;
 		if (pConfig->bDontUseNativeDialogs) {
 			options |= QFileDialog::DontUseNativeDialog;
-			pParentWidget = parentWidget();
+			pParentWidget = QWidget::window();
 		}
-	#if 1//QT_VERSION < 0x040400
+	#if 1//QT_VERSION < QT_VERSION_CHECK(4, 4, 0)
 		sFilename = QFileDialog::getSaveFileName(pParentWidget,
-			sTitle, sFilename, sFilter, NULL, options);
+			sTitle, sFilename, sFilter, nullptr, options);
 	#else
 		QFileDialog fileDialog(pParentWidget,
 			sTitle, sFilename, sFilter);
@@ -327,8 +324,8 @@ void samplv1widget_preset::savePreset ( const QString& sPreset )
 			sFilename = fileDialog.selectedFiles().first();
 	#endif
 	} else {
-		if (QMessageBox::warning(parentWidget(),
-			tr("Warning") + " - " SAMPLV1_TITLE,
+		if (QMessageBox::warning(QWidget::window(),
+			tr("Warning"),
 			tr("About to replace preset:\n\n"
 			"\"%1\"\n\n"
 			"Are you sure?")
@@ -361,11 +358,10 @@ void samplv1widget_preset::deletePreset (void)
 		return;
 
 	samplv1_config *pConfig = samplv1_config::getInstance();
-	if (pConfig == NULL)
+	if (pConfig == nullptr)
 		return;
-
-	if (QMessageBox::warning(parentWidget(),
-		tr("Warning") + " - " SAMPLV1_TITLE,
+	if (QMessageBox::warning(QWidget::window(),
+		tr("Warning"),
 		tr("About to remove preset:\n\n"
 		"\"%1\"\n\n"
 		"Are you sure?")
@@ -375,6 +371,7 @@ void samplv1widget_preset::deletePreset (void)
 		return;
 
 	pConfig->removePreset(sPreset);
+	pConfig->sPreset.clear();
 
 	clearPreset();
 	refreshPreset();
