@@ -1,7 +1,7 @@
 // samplv1widget_sample.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -26,6 +26,8 @@
 
 #include "samplv1_ui.h"
 
+#include "samplv1widget_spinbox.h"
+
 #include <sndfile.h>
 
 #include <QPainter>
@@ -50,8 +52,8 @@
 // Constructor.
 samplv1widget_sample::samplv1widget_sample (
 	QWidget *pParent, Qt::WindowFlags wflags )
-	: QFrame(pParent, wflags), m_pSamplUi(NULL),
-		m_pSample(NULL), m_iChannels(0), m_ppPolyg(NULL)
+	: QFrame(pParent, wflags), m_pSamplUi(nullptr),
+		m_pSample(nullptr), m_iChannels(0), m_ppPolyg(nullptr)
 {
 	QFrame::setMouseTracking(true);
 	QFrame::setFocusPolicy(Qt::ClickFocus);
@@ -65,11 +67,14 @@ samplv1widget_sample::samplv1widget_sample (
 	QFrame::setFrameShape(QFrame::Panel);
 	QFrame::setFrameShadow(QFrame::Sunken);
 
+	m_bOffset = false;
+	m_iOffsetStart = m_iOffsetEnd = 0;
+
 	m_bLoop = false;
 	m_iLoopStart = m_iLoopEnd = 0;
 
 	m_dragCursor  = DragNone;
-	m_pDragSample = NULL;
+	m_pDragSample = nullptr;
 
 	m_iDirectNoteOn = -1;
 
@@ -80,7 +85,7 @@ samplv1widget_sample::samplv1widget_sample (
 // Destructor.
 samplv1widget_sample::~samplv1widget_sample (void)
 {
-	setSample(NULL);
+	setSample(nullptr);
 }
 
 
@@ -110,10 +115,13 @@ void samplv1widget_sample::setSample ( samplv1_sample *pSample )
 
 	m_pSample = pSample;
 
+//	m_bOffset = 0;
+//	m_iOffsetStart = m_iOffsetEnd = 0;
+
 //	m_bLoop = false;
 //	m_iLoopStart = m_iLoopEnd = 0;
 
-	m_pDragSample = NULL;
+	m_pDragSample = nullptr;
 
 	if (m_pSample)
 		m_iChannels = m_pSample->channels();
@@ -121,31 +129,31 @@ void samplv1widget_sample::setSample ( samplv1_sample *pSample )
 		const int h = height();
 		const int w = width() & 0x7ffe; // force even.
 		const int w2 = (w >> 1);
-		const unsigned int nframes = m_pSample->length();
-		const float period = float(nframes) / float(w2);
+		const uint32_t nframes = m_pSample->length();
+		const uint32_t nperiod = nframes / w2;
 		const int h0 = h / m_iChannels;
-		const float h1 = float(h0 >> 1);
+		const int h1 = (h0 >> 1);
 		int y0 = h1;
 		m_ppPolyg = new QPolygon* [m_iChannels];
-		for (unsigned short k = 0; k < m_iChannels; ++k) {
+		for (uint16_t k = 0; k < m_iChannels; ++k) {
 			m_ppPolyg[k] = new QPolygon(w);
 			const float *pframes = m_pSample->frames(k);
 			float vmax = 0.0f;
 			float vmin = 0.0f;
 			int n = 0;
 			int x = 1;
-			unsigned int j = 0;
-			for (unsigned int i = 0; i < nframes; ++i) {
+			uint32_t j = 0;
+			for (uint32_t i = 0; i < nframes; ++i) {
 				const float v = *pframes++;
-				if (vmax < v)
+				if (vmax < v || j == 0)
 					vmax = v;
-				if (vmin > v)
+				if (vmin > v || j == 0)
 					vmin = v;
-				if (i > j) {
+				if (++j > nperiod) {
 					m_ppPolyg[k]->setPoint(n, x, y0 - int(vmax * h1));
 					m_ppPolyg[k]->setPoint(w - n - 1, x, y0 - int(vmin * h1));
 					vmax = vmin = 0.0f;
-					++n; x += 2; j = (unsigned int) (float(n) * period);
+					++n; x += 2; j = 0;
 				}
 			}
 			while (n < w2) {
@@ -181,6 +189,50 @@ const QString& samplv1widget_sample::sampleName (void) const
 }
 
 
+// Offset mode.
+void samplv1widget_sample::setOffset ( bool bOffset )
+{
+	m_bOffset = bOffset;
+
+	updateToolTip();
+	update();
+}
+
+bool samplv1widget_sample::isOffset (void) const
+{
+	return m_bOffset;
+}
+
+
+void samplv1widget_sample::setOffsetStart ( uint32_t iOffsetStart )
+{
+	m_iOffsetStart = iOffsetStart;
+
+	updateToolTip();
+	update();
+}
+
+uint32_t samplv1widget_sample::offsetStart (void) const
+{
+	return m_iOffsetStart;
+}
+
+
+void samplv1widget_sample::setOffsetEnd ( uint32_t iOffsetEnd )
+{
+	m_iOffsetEnd = iOffsetEnd;
+
+	updateToolTip();
+	update();
+}
+
+uint32_t samplv1widget_sample::offsetEnd (void) const
+{
+	return m_iOffsetEnd;
+}
+
+
+// Loop mode.
 void samplv1widget_sample::setLoop ( bool bLoop )
 {
 	m_bLoop = bLoop;
@@ -251,13 +303,19 @@ void samplv1widget_sample::mousePressEvent ( QMouseEvent *pMouseEvent )
 		if (m_dragCursor == DragNone) {
 			m_dragState = DragStart;
 			m_posDrag = pMouseEvent->pos();
-		} else if (m_bLoop) {
-			const int w = QFrame::width();
+		} else {
 			const uint32_t nframes = m_pSample->length();
-			if (nframes > 0) {
-				m_iDragStartX = safeX((m_iLoopStart * w) / nframes);
-				m_iDragEndX   = safeX((m_iLoopEnd   * w) / nframes);
-				m_dragState   = m_dragCursor;
+			if (nframes > 0 && (m_bOffset || m_bLoop)) {
+				const int w = QFrame::width();
+				if (m_bOffset) {
+					m_iDragOffsetStartX = safeX((m_iOffsetStart * w) / nframes);
+					m_iDragOffsetEndX   = safeX((m_iOffsetEnd   * w) / nframes);
+				}
+				if (m_bLoop) {
+					m_iDragLoopStartX = safeX((m_iLoopStart * w) / nframes);
+					m_iDragLoopEndX   = safeX((m_iLoopEnd   * w) / nframes);
+				}
+				m_dragState = m_dragCursor;
 			}
 		}
 	}
@@ -271,95 +329,200 @@ void samplv1widget_sample::mouseMoveEvent ( QMouseEvent *pMouseEvent )
 	const int x = pMouseEvent->pos().x();
 
 	switch (m_dragState) {
-	case DragNone: {
-		if (m_bLoop && m_pSample) {
-			const int w = QFrame::width();
+	case DragNone:
+		if (m_pSample) {
 			const uint32_t nframes = m_pSample->length();
 			if (nframes > 0) {
-				const int x1 = (m_iLoopStart * w) / nframes;
-				const int x2 = (m_iLoopEnd   * w) / nframes;
-				if (abs(x1 - x) < QApplication::startDragDistance()) {
-					m_dragCursor = DragLoopStart;
-					QFrame::setCursor(QCursor(Qt::SizeHorCursor));
-				}
-				else
-				if (abs(x2 - x) < QApplication::startDragDistance()) {
+				const int w  = QFrame::width();
+				const int dx = QApplication::startDragDistance();
+				const int x1 = (m_iOffsetStart * w) / nframes;
+				const int x2 = (m_iOffsetEnd * w) / nframes;
+				const int x3 = (m_iLoopStart * w) / nframes;
+				const int x4 = (m_iLoopEnd * w) / nframes;
+				if (abs(x4 - x) < dx && m_bLoop) {
 					m_dragCursor = DragLoopEnd;
 					QFrame::setCursor(QCursor(Qt::SizeHorCursor));
+					QToolTip::showText(
+						QCursor::pos(),
+						tr("Loop end: %1")
+							.arg(textFromValue(m_iLoopEnd)), this);
+				}
+				else
+				if (abs(x3 - x) < dx && m_bLoop) {
+					m_dragCursor = DragLoopStart;
+					QFrame::setCursor(QCursor(Qt::SizeHorCursor));
+					QToolTip::showText(
+						QCursor::pos(),
+						tr("Loop start: %1")
+							.arg(textFromValue(m_iLoopStart)), this);
+				}
+				else
+				if (abs(x2 - x) < dx && m_bOffset) {
+					m_dragCursor = DragOffsetEnd;
+					QFrame::setCursor(QCursor(Qt::SizeHorCursor));
+					QToolTip::showText(
+						QCursor::pos(),
+						tr("Offset end: %1")
+							.arg(textFromValue(m_iOffsetEnd)), this);
+				}
+				else
+				if (abs(x1 - x) < dx && m_bOffset) {
+					m_dragCursor = DragOffsetStart;
+					QFrame::setCursor(QCursor(Qt::SizeHorCursor));
+					QToolTip::showText(
+						QCursor::pos(),
+						tr("Offset start: %1")
+							.arg(textFromValue(m_iOffsetStart)), this);
 				}
 				else
 				if (m_dragCursor != DragNone) {
-					m_dragCursor =  DragNone;
+					m_dragCursor  = DragNone;
 					QFrame::unsetCursor();
 				}
 			}
 		}
 		break;
-	}
-	case DragLoopStart: {
-		if (m_iDragEndX > x) {
-			m_iDragStartX = safeX(x);
-			update();
-			if (m_pSample) {
-				const int w = QFrame::width();
-				if (w > 0) {
-					const uint32_t nframes = m_pSample->length();
-					QToolTip::showText(
-						QCursor::pos(),
-						tr("Loop start: %1")
-							.arg((m_iDragStartX * nframes) / w), this);
-				}
-			}
-		}
-		break;
-	}
-	case DragLoopEnd: {
-		if (m_iDragStartX < x) {
-			m_iDragEndX = safeX(x);
-			update();
-			if (m_pSample) {
-				const int w = QFrame::width();
-				if (w > 0) {
-					const uint32_t nframes = m_pSample->length();
-					QToolTip::showText(
-						QCursor::pos(),
-						tr("Loop end: %1")
-							.arg((m_iDragEndX * nframes) / w), this);
-				}
-			}
-		}
-		break;
-	}
-	case DragSelect: {
-		// Rubber-band selection...
-		const QRect& rect = QRect(m_posDrag, pMouseEvent->pos()).normalized();
-		m_iDragStartX = safeX(rect.left());
-		m_iDragEndX   = safeX(rect.right());
-		update();
+	case DragOffsetStart:
 		if (m_pSample) {
+			m_iDragOffsetStartX = safeX(x);
+			if (m_iDragOffsetStartX > m_iDragOffsetEndX)
+				m_iDragOffsetStartX = m_iDragOffsetEndX;
+			if (m_bLoop && m_iDragOffsetStartX > m_iDragLoopStartX)
+				m_iDragOffsetStartX = m_iDragLoopStartX;
+			update();
 			const int w = QFrame::width();
 			if (w > 0) {
 				const uint32_t nframes = m_pSample->length();
+				const uint32_t iOffsetStart
+					= (m_iDragOffsetStartX * nframes) / w;
 				QToolTip::showText(
 					QCursor::pos(),
+					tr("Offset start: %1")
+						.arg(textFromValue(iOffsetStart)), this);
+			}
+		}
+		break;
+	case DragOffsetEnd:
+		if (m_pSample) {
+			m_iDragOffsetEndX = safeX(x);
+			if (m_iDragOffsetEndX < m_iDragOffsetStartX)
+				m_iDragOffsetEndX = m_iDragOffsetStartX;
+			if (m_bLoop && m_iDragOffsetEndX < m_iDragLoopEndX)
+				m_iDragOffsetEndX = m_iDragLoopEndX;
+			update();
+			const int w = QFrame::width();
+			if (w > 0) {
+				const uint32_t nframes = m_pSample->length();
+				const uint32_t iOffsetEnd
+					= (m_iDragOffsetEndX * nframes) / w;
+				QToolTip::showText(
+					QCursor::pos(),
+					tr("Offset end: %1")
+						.arg(textFromValue(iOffsetEnd)), this);
+			}
+		}
+		break;
+	case DragOffsetRange:
+		// Rubber-band offset selection...
+		if (m_pSample) {
+			const QRect& rect = QRect(m_posDrag, pMouseEvent->pos()).normalized();
+			m_iDragOffsetStartX = safeX(rect.left());
+			m_iDragOffsetEndX   = safeX(rect.right());
+			update();
+			const int w = QFrame::width();
+			if (w > 0) {
+				const uint32_t nframes      = m_pSample->length();
+				const uint32_t iOffsetStart = (m_iDragOffsetStartX * nframes) / w;
+				const uint32_t iOffsetEnd   = (m_iDragOffsetEndX   * nframes) / w;
+				QToolTip::showText(
+					QCursor::pos(),
+					tr("Offset start: %1, end: %2")
+						.arg(textFromValue(iOffsetStart))
+						.arg(textFromValue(iOffsetEnd)), this);
+			}
+		}
+		break;
+	case DragLoopStart:
+		if (m_pSample) {
+			m_iDragLoopStartX = safeX(x);
+			if (m_bOffset && m_iDragLoopStartX < m_iDragOffsetStartX)
+				m_iDragLoopStartX = m_iDragOffsetStartX;
+			if (m_iDragLoopStartX > m_iDragLoopEndX)
+				m_iDragLoopStartX = m_iDragLoopEndX;
+			update();
+			const int w = QFrame::width();
+			if (w > 0) {
+				const uint32_t nframes = m_pSample->length();
+				const uint32_t iLoopStart
+					= (m_iDragLoopStartX * nframes) / w;
+				QToolTip::showText(
+					QCursor::pos(),
+					tr("Loop start: %1")
+						.arg(textFromValue(iLoopStart)), this);
+			}
+		}
+		break;
+	case DragLoopEnd:
+		if (m_pSample) {
+			m_iDragLoopEndX = safeX(x);
+			if (m_bOffset && m_iDragLoopEndX > m_iDragOffsetEndX)
+				m_iDragLoopEndX = m_iDragOffsetEndX;
+			if (m_iDragLoopEndX < m_iDragLoopStartX)
+				m_iDragLoopEndX = m_iDragLoopStartX;
+			update();
+			const int w = QFrame::width();
+			if (w > 0) {
+				const uint32_t nframes = m_pSample->length();
+				const uint32_t iLoopEnd
+					= (m_iDragLoopEndX * nframes) / w;
+				QToolTip::showText(
+					QCursor::pos(),
+					tr("Loop end: %1")
+						.arg(textFromValue(iLoopEnd)), this);
+			}
+		}
+		break;
+	case DragLoopRange:
+		// Rubber-band loop selection...
+		if (m_pSample) {
+			const QRect& rect = QRect(m_posDrag, pMouseEvent->pos()).normalized();
+			m_iDragLoopStartX = safeX(rect.left());
+			m_iDragLoopEndX   = safeX(rect.right());
+			update();
+			const int w = QFrame::width();
+			if (w > 0) {
+				const uint32_t nframes    = m_pSample->length();
+				const uint32_t iLoopStart = (m_iDragLoopStartX * nframes) / w;
+				const uint32_t iLoopEnd   = (m_iDragLoopEndX   * nframes) / w;
+				QToolTip::showText(
+					QCursor::pos(),
 					tr("Loop start: %1, end: %2")
-						.arg((m_iDragStartX * nframes) / w)
-						.arg((m_iDragEndX   * nframes) / w), this);
+						.arg(textFromValue(iLoopStart))
+						.arg(textFromValue(iLoopEnd)), this);
 			}
 		}
 		break;
-	}
 	case DragStart:
 		// Rubber-band starting...
 		if ((m_posDrag - pMouseEvent->pos()).manhattanLength()
 			> QApplication::startDragDistance()) {
 			// Start dragging alright...
-			if (m_bLoop && (pMouseEvent->modifiers()
-				& (Qt::ShiftModifier | Qt::ControlModifier))) {
-				m_dragState = m_dragCursor = DragSelect;
-				m_iDragStartX = m_iDragEndX = m_posDrag.x();
+			if (m_dragCursor != DragNone)
+				m_dragState = m_dragCursor;
+			else
+			if (m_bOffset && pMouseEvent->modifiers() & Qt::ShiftModifier) {
+				m_dragState = m_dragCursor = DragOffsetRange;
+				m_iDragOffsetStartX = m_iDragOffsetEndX = m_posDrag.x();
 				QFrame::setCursor(QCursor(Qt::SizeHorCursor));
-			} else if (m_pSample && m_pSample->filename()) {
+			}
+			else
+			if (m_bLoop && (pMouseEvent->modifiers() & Qt::ControlModifier)) {
+				m_dragState = m_dragCursor = DragLoopRange;
+				m_iDragLoopStartX = m_iDragLoopEndX = m_posDrag.x();
+				QFrame::setCursor(QCursor(Qt::SizeHorCursor));
+			}
+			else
+			if (m_pSample && m_pSample->filename()) {
 				QList<QUrl> urls;
 				m_pDragSample = m_pSample;
 				urls.append(QUrl::fromLocalFile(m_pDragSample->filename()));
@@ -385,13 +548,35 @@ void samplv1widget_sample::mouseReleaseEvent ( QMouseEvent *pMouseEvent )
 	QFrame::mouseReleaseEvent(pMouseEvent);
 
 	switch (m_dragState) {
-	case DragSelect: {
+	case DragOffsetStart: {
 		const int w = QFrame::width();
 		if (m_pSample && w > 0) {
 			const uint32_t nframes = m_pSample->length();
-			m_iLoopStart = (m_iDragStartX * nframes) / w;
-			m_iLoopEnd   = (m_iDragEndX   * nframes) / w;
-			emit loopRangeChanged();
+			m_iOffsetStart = (m_iDragOffsetStartX * nframes) / w;
+			emit offsetRangeChanged();
+			updateToolTip();
+			update();
+		}
+		break;
+	}
+	case DragOffsetEnd: {
+		const int w = QFrame::width();
+		if (m_pSample && w > 0) {
+			const uint32_t nframes = m_pSample->length();
+			m_iOffsetEnd = (m_iDragOffsetEndX * nframes) / w;
+			emit offsetRangeChanged();
+			updateToolTip();
+			update();
+		}
+		break;
+	}
+	case DragOffsetRange: {
+		const int w = QFrame::width();
+		if (m_pSample && w > 0) {
+			const uint32_t nframes = m_pSample->length();
+			m_iOffsetStart = (m_iDragOffsetStartX * nframes) / w;
+			m_iOffsetEnd   = (m_iDragOffsetEndX   * nframes) / w;
+			emit offsetRangeChanged();
 			updateToolTip();
 			update();
 		}
@@ -401,7 +586,7 @@ void samplv1widget_sample::mouseReleaseEvent ( QMouseEvent *pMouseEvent )
 		const int w = QFrame::width();
 		if (m_pSample && w > 0) {
 			const uint32_t nframes = m_pSample->length();
-			m_iLoopStart = (m_iDragStartX * nframes) / w;
+			m_iLoopStart = (m_iDragLoopStartX * nframes) / w;
 			emit loopRangeChanged();
 			updateToolTip();
 			update();
@@ -412,7 +597,19 @@ void samplv1widget_sample::mouseReleaseEvent ( QMouseEvent *pMouseEvent )
 		const int w = QFrame::width();
 		if (m_pSample && w > 0) {
 			const uint32_t nframes = m_pSample->length();
-			m_iLoopEnd = (m_iDragEndX * nframes) / w;
+			m_iLoopEnd = (m_iDragLoopEndX * nframes) / w;
+			emit loopRangeChanged();
+			updateToolTip();
+			update();
+		}
+		break;
+	}
+	case DragLoopRange: {
+		const int w = QFrame::width();
+		if (m_pSample && w > 0) {
+			const uint32_t nframes = m_pSample->length();
+			m_iLoopStart = (m_iDragLoopStartX * nframes) / w;
+			m_iLoopEnd   = (m_iDragLoopEndX   * nframes) / w;
 			emit loopRangeChanged();
 			updateToolTip();
 			update();
@@ -425,7 +622,7 @@ void samplv1widget_sample::mouseReleaseEvent ( QMouseEvent *pMouseEvent )
 
 	directNoteOff();
 
-	m_pDragSample = NULL;
+	m_pDragSample = nullptr;
 	resetDragState();
 }
 
@@ -441,7 +638,7 @@ void samplv1widget_sample::keyPressEvent ( QKeyEvent *pKeyEvent )
 {
 	switch (pKeyEvent->key()) {
 	case Qt::Key_Escape:
-		m_pDragSample = NULL;
+		m_pDragSample = nullptr;
 		resetDragState();
 		update();
 		break;
@@ -485,7 +682,8 @@ void samplv1widget_sample::resetDragState (void)
 	if (m_dragCursor != DragNone)
 		QFrame::unsetCursor();
 
-	m_iDragStartX = m_iDragEndX = 0;
+	m_iDragOffsetStartX = m_iDragOffsetEndX = 0;
+	m_iDragLoopStartX = m_iDragLoopEndX = 0;
 
 	m_dragState = m_dragCursor = DragNone;
 }
@@ -509,23 +707,26 @@ void samplv1widget_sample::paintEvent ( QPaintEvent *pPaintEvent )
     painter.fillRect(rect, rgbDark);
 
 	if (m_pSample && m_ppPolyg) {
+		const bool bEnabled = isEnabled();
+		const uint32_t nframes = m_pSample->length();
 		const int w2 = (w << 1);
 		painter.setRenderHint(QPainter::Antialiasing, true);
-		if (m_bLoop && isEnabled()) {
-			int x1, x2;
-			if (m_iDragStartX < m_iDragEndX) {
-				x1 = m_iDragStartX;
-				x2 = m_iDragEndX;
-			} else {
-				const uint32_t nframes = m_pSample->length();
-				if (nframes > 0) {
-					x1 = (m_iLoopStart * w) / nframes;
-					x2 = (m_iLoopEnd   * w) / nframes;
-				}
-				else x1 = x2 = 0;
+		// Loop point lines...
+		if (m_bLoop && bEnabled) {
+			int x1 = 0, x2 = 0;
+			if (m_dragState == DragLoopStart ||
+				m_dragState == DragLoopEnd   ||
+				m_dragState == DragLoopRange) {
+				x1 = m_iDragLoopStartX;
+				x2 = m_iDragLoopEndX;
+			}
+			else
+			if (nframes > 0) {
+				x1 = (m_iLoopStart * w) / nframes;
+				x2 = (m_iLoopEnd   * w) / nframes;
 			}
 			QLinearGradient grad1(0, 0, w2, h);
-			painter.setPen(pal.highlight().color());
+			painter.setPen(rgbLite);
 			grad1.setColorAt(0.0f, rgbLite.darker());
 			grad1.setColorAt(0.5f, pal.dark().color());
 			painter.fillRect(x1, 0, x2 - x1, h, grad1);
@@ -542,6 +743,7 @@ void samplv1widget_sample::paintEvent ( QPaintEvent *pPaintEvent )
 			polyg.putPoints(0, 3, x2 - 8, h, x2, h - 8, x2, h);
 			painter.drawPolygon(polyg);
 		}
+		// Sample waveform...
 		QLinearGradient grad(0, 0, w2, h);
 		painter.setPen(bDark ? Qt::gray : Qt::darkGray);
 		grad.setColorAt(0.0f, rgbLite);
@@ -549,6 +751,38 @@ void samplv1widget_sample::paintEvent ( QPaintEvent *pPaintEvent )
 		painter.setBrush(grad);
 		for (unsigned short k = 0; k < m_iChannels; ++k)
 			painter.drawPolygon(*m_ppPolyg[k]);
+		// Offset line...
+		if (m_bOffset && bEnabled) {
+			int x1 = 0, x2 = 0;
+			if (m_dragState == DragOffsetStart ||
+				m_dragState == DragOffsetEnd   ||
+				m_dragState == DragOffsetRange) {
+				x1 = m_iDragOffsetStartX;
+				x2 = m_iDragOffsetEndX;
+			}
+			else
+			if (nframes > 0) {
+				x1 = (m_iOffsetStart * w) / nframes;
+				x2 = (m_iOffsetEnd   * w) / nframes;
+			}
+			QColor rgbOver = rgbDark.darker();
+			rgbOver.setAlpha(120);
+			painter.setPen(rgbLite.darker(160));
+			painter.setBrush(rgbDark.lighter(160));
+			QPolygon polyg(3);
+		//	polyg.putPoints(0, 3, x1 + 8, 0, x1, 8, x1, 0);
+		//	painter.drawPolygon(polyg);
+			polyg.putPoints(0, 3, x1 + 8, h, x1, h - 8, x1, h);
+			painter.drawPolygon(polyg);
+			painter.fillRect(0, 0, x1, h, rgbOver);
+			painter.drawLine(x1, 0, x1, h);
+			polyg.putPoints(0, 3, x2 - 8, 0, x2, 8, x2, 0);
+			painter.drawPolygon(polyg);
+		//	polyg.putPoints(0, 3, x2 - 8, h, x2, h - 8, x2, h);
+		//	painter.drawPolygon(polyg);
+			painter.fillRect(x2, 0, w, h, rgbOver);
+			painter.drawLine(x2, 0, x2, h);
+		}
 		painter.setRenderHint(QPainter::Antialiasing, false);
 	} else {
 		painter.setPen(pal.midlight().color());
@@ -582,7 +816,7 @@ void samplv1widget_sample::paintEvent ( QPaintEvent *pPaintEvent )
 void samplv1widget_sample::openSample (void)
 {
 	samplv1_config *pConfig = samplv1_config::getInstance();
-	if (pConfig == NULL)
+	if (pConfig == nullptr)
 		return;
 
 	QString sFilename = pConfig->sSampleDir;
@@ -597,10 +831,10 @@ void samplv1widget_sample::openSample (void)
 		QStringList exts;
 		SF_FORMAT_INFO sffinfo;
 		int iCount = 0;
-		::sf_command(NULL, SFC_GET_FORMAT_MAJOR_COUNT, &iCount, sizeof(int));
+		::sf_command(nullptr, SFC_GET_FORMAT_MAJOR_COUNT, &iCount, sizeof(int));
 		for (int i = 0 ; i < iCount; ++i) {
 			sffinfo.format = i;
-			::sf_command(NULL, SFC_GET_FORMAT_MAJOR, &sffinfo, sizeof(sffinfo));
+			::sf_command(nullptr, SFC_GET_FORMAT_MAJOR, &sffinfo, sizeof(sffinfo));
 			const QString sFilterName = QString(sffinfo.name)
 				.replace('/', '-') // Replace some illegal characters.
 				.remove('(').remove(')');
@@ -619,17 +853,17 @@ void samplv1widget_sample::openSample (void)
 		s_filters.append(sFilterMask.arg(tr("All files")).arg("*.*"));
 	}
 
-	const QString& sTitle  = tr("Open Sample") + " - " SAMPLV1_TITLE;
+	const QString& sTitle  = tr("Open Sample");
 	const QString& sFilter = s_filters.join(";;");
-	QWidget *pParentWidget = NULL;
+	QWidget *pParentWidget = nullptr;
 	QFileDialog::Options options = 0;
 	if (pConfig->bDontUseNativeDialogs) {
 		options |= QFileDialog::DontUseNativeDialog;
-		pParentWidget = parentWidget();
+		pParentWidget = QWidget::window();
 	}
-#if 1//QT_VERSION < 0x040400
+#if 1//QT_VERSION < QT_VERSION_CHECK(4, 4, 0)
 	sFilename = QFileDialog::getOpenFileName(pParentWidget,
-		sTitle, sFilename, sFilter, NULL, options);
+		sTitle, sFilename, sFilter, nullptr, options);
 #else
 	QFileDialog fileDialog(pParentWidget, sTitle, sFilename, sFilter);
 	fileDialog.setAcceptMode(QFileDialog::AcceptOpen);
@@ -655,6 +889,29 @@ void samplv1widget_sample::loadSample ( samplv1_sample *pSample )
 }
 
 
+// Value/text format converter utilities.
+uint32_t samplv1widget_sample::valueFromText ( const QString& text ) const
+{
+	samplv1widget_spinbox::Format format = samplv1widget_spinbox::Frames;
+	samplv1_config *pConfig = samplv1_config::getInstance();
+	if (pConfig)
+		format = samplv1widget_spinbox::Format(pConfig->iFrameTimeFormat);
+	const float srate = (m_pSample ? m_pSample->sampleRate() : 44100.0f);
+	return samplv1widget_spinbox::valueFromText(text, format, srate);
+}
+
+
+QString samplv1widget_sample::textFromValue ( uint32_t value ) const
+{
+	samplv1widget_spinbox::Format format = samplv1widget_spinbox::Frames;
+	samplv1_config *pConfig = samplv1_config::getInstance();
+	if (pConfig)
+		format = samplv1widget_spinbox::Format(pConfig->iFrameTimeFormat);
+	const float srate = (m_pSample ? m_pSample->sampleRate() : 44100.0f);
+	return samplv1widget_spinbox::textFromValue(value, format, srate);
+}
+
+
 // Update tool-tip.
 void samplv1widget_sample::updateToolTip (void)
 {
@@ -673,11 +930,18 @@ void samplv1widget_sample::updateToolTip (void)
 			.arg(m_pSample->rate());
 	}
 
+	if (m_bOffset && m_iOffsetStart < m_iOffsetEnd) {
+		if (!sToolTip.isEmpty()) sToolTip += '\n';
+		sToolTip += tr("Offset start: %1, end: %2")
+			.arg(textFromValue(m_iOffsetStart))
+			.arg(textFromValue(m_iOffsetEnd));
+	}
+
 	if (m_bLoop && m_iLoopStart < m_iLoopEnd) {
 		if (!sToolTip.isEmpty()) sToolTip += '\n';
 		sToolTip += tr("Loop start: %1, end: %2")
-			.arg(m_iLoopStart)
-			.arg(m_iLoopEnd);
+			.arg(textFromValue(m_iLoopStart))
+			.arg(textFromValue(m_iLoopEnd));
 	}
 
 	setToolTip(sToolTip);
@@ -694,7 +958,7 @@ QSize samplv1widget_sample::sizeHint (void) const
 // Direct note-on/off methods.
 void samplv1widget_sample::directNoteOn (void)
 {
-	if (m_pSamplUi == NULL || m_pSample == NULL)
+	if (m_pSamplUi == nullptr || m_pSample == nullptr)
 		return;
 
 	const int key = int(m_pSamplUi->paramValue(samplv1::GEN1_SAMPLE));
@@ -711,7 +975,7 @@ void samplv1widget_sample::directNoteOn (void)
 
 void samplv1widget_sample::directNoteOff (void)
 {
-	if (m_pSamplUi == NULL || m_iDirectNoteOn < 0)
+	if (m_pSamplUi == nullptr || m_iDirectNoteOn < 0)
 		return;
 
 	m_pSamplUi->directNoteOn(m_iDirectNoteOn, 0); // note-off!
