@@ -1,7 +1,7 @@
 // samplv1widget_sample.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -60,6 +60,15 @@ public:
 	void setSampleName(const QString& sName);
 	const QString& sampleName() const;
 
+	// Offset mode.
+	void setOffset(bool bOffset);
+	bool isOffset() const;
+
+	// Offset getters.
+	uint32_t offsetStart() const;
+	uint32_t offsetEnd() const;
+
+	// Loop mode.
 	void setLoop(bool bLoop);
 	bool isLoop() const;
 
@@ -70,12 +79,17 @@ public:
 	// Direct note-on methods.
 	void directNoteOn();
 
+	// Value/text format converter utilities.
+	uint32_t valueFromText (const QString& text) const;
+	QString textFromValue (uint32_t value) const;
+
 signals:
 
 	// Load new sample file.
 	void loadSampleFile(const QString&);
 
-	// Loop range changed.
+	// Offset/loop range changed.
+	void offsetRangeChanged();
 	void loopRangeChanged();
 
 public slots:
@@ -86,6 +100,10 @@ public slots:
 	// Effective sample slot.
 	void loadSample(samplv1_sample *pSample);
 
+	// Offset point setters.
+	void setOffsetStart(uint32_t iOffsetStart);
+	void setOffsetEnd(uint32_t iOffsetEnd);
+
 	// Loop point setters.
 	void setLoopStart(uint32_t iLoopStart);
 	void setLoopEnd(uint32_t iLoopEnd);
@@ -142,19 +160,28 @@ private:
 
 	// Drag state.
 	enum DragState {
-		DragNone = 0, DragStart, DragSelect,
-		DragLoopStart, DragLoopEnd,
+		DragNone = 0, DragStart,
+		DragOffsetRange, DragOffsetStart, DragOffsetEnd,
+		DragLoopRange, DragLoopStart, DragLoopEnd,
 	} m_dragState, m_dragCursor;
 
 	QPoint m_posDrag;
 
-	int m_iDragStartX;
-	int m_iDragEndX;
+	int m_iDragOffsetStartX;
+	int m_iDragOffsetEndX;
+
+	int m_iDragLoopStartX;
+	int m_iDragLoopEndX;
 
 	samplv1_sample *m_pDragSample;
 
 	int m_iDirectNoteOn;
 
+	// Offset state.
+	bool     m_bOffset;
+	uint32_t m_iOffsetStart;
+	uint32_t m_iOffsetEnd;
+
 	// Loop state.
 	bool     m_bLoop;
 	uint32_t m_iLoopStart;
